﻿-- Author      : Kurapica
-- Create Date : 04/17/2009
-- ChangeLog   :
--				2010/01/23 When VARIABLES_LOADED, the SavedVariables in addon's namespace will be replaced with the _G one.
--				2010/01/26 Add AddSlashCmd function.
--				2010/01/29 Make Addon can be rewrited.
--				2010/01/29 Add module system.
--				2010/02/06 Remove or rename some function that the wow system used.
--				2010/02/24 Add EventSystem to Module.
--				2010/02/25 Auto add global functions and tables to addon's environment when used, auto update frame to igas's frame

--[[
    Addon
--]]
do
	-- Check Version
	local version = 6
	if not IGAS:RegisterFile("IGAS.Addon.Addon", version) then
		return
	end
	
	local rawset = rawset
	local rawget = rawget
	local type = type
	local setmetatable = setmetatable
	local tinsert = table.insert
	local tremove = table.remove
	local twipe = table.wipe
	local getn = table.getn
	local pcall = pcall
	local _G = _G
	local setfenv = setfenv
	local getfenv = getfenv
	local strtrim = strtrim
	local strfind = strfind
	local strsplit = strsplit
	local gsub = gsub
	local tinsert = tinsert
	local IGAS = IGAS
	
    local _RegisterAddon = IGAS.Addon or {}	
	IGAS.Addon = _RegisterAddon
	
	-- Infomations
	local _ScriptHandler = {
		["OnEnable"] = true,
		["OnDisable"] = true,
		["OnDispose"] = true,
	}
	
	local _UsedTable = {
		["__StackScripts"] = true,
		["__Meta"] = true,
	}
	
	local _WeakTable = {
		["__Frames"] = true,
	}
	
	local _UsedEvent = {
		["VARIABLES_LOADED"] = true,
	}
	
	local _metaWeak = {__mode = "k"}
	
	--  EventHandler
	local _EventCtrl = IGAS:NewFrame("Frame", "IGAS_ADDON_EVENTCTRL")
	local _EventDistribution = _EventCtrl._EventDistribution or {}
	_EventCtrl._EventDistribution = _EventDistribution

	function _EventCtrl:OnEvent(event, ...)
		if _EventDistribution[event] then
			for v in pairs(_EventDistribution[event]) do
				if not v.__Disabled and v[event] and type(v[event]) == "function" then
					pcall(v[event], v, ...)
				end
			end
		end
	end
	
	for event in pairs(_UsedEvent) do
		_EventCtrl:RegisterEvent(event)
	end
	
	-- SavedVariable
	local _SavedVariables = _EventCtrl._SavedVariables or {}
	_EventCtrl._SavedVariables = _SavedVariables	
	
	-- Special Events
	function _EventCtrl:VARIABLES_LOADED()
		for addon, sv in pairs(_SavedVariables) do
			for svn in pairs(sv) do
				_G[svn] = _G[svn] or {}
				addon[svn] = _G[svn]
			end
		end
		twipe(_SavedVariables)
		_EventCtrl._VariablesLoaded = true
	end
	
	-- Modules
	local  _Modules = _EventCtrl._Modules or setmetatable({}, {__mode="kv"})
	_EventCtrl._Modules = _Modules
	
	-- MetaTable For Module
	local _metaModule = _EventCtrl._metaModule or {}
	_EventCtrl._metaModule = _metaModule
	
	-- MetaTable For Addon
	local _metaAddon = _EventCtrl._metaAddon or {}
	_EventCtrl._metaAddon = _metaAddon
	
	-- Create Addon Or Module
	local function CreateNameSpace(parent, ...)
		local name
		
		if select('#', ...) == 0 then
			return
		end
		
		for i = 1, select('#', ...) do
			name = select(i, ...)
			
			if not parent[name] or type(parent[name]) ~= "table" then
				if parent == _RegisterAddon then
					parent[name] = setmetatable({}, _metaAddon)
					parent[name]._Addon = parent[name]
					parent[name]._Name = name
				else
					parent[name] = setmetatable({}, _metaModule)
					parent[name]._M = parent[name]
					parent[name]._Name = name
					_Modules[parent[name]] = parent
				end
			end
			
			parent = parent[name]
		end
		
		return parent
	end
	
	local _FuncModule = {
		-- Dispose
		["Dispose"] = function(self)
			self:UnregisterAllEvents()
			
			setmetatable(self, nil)
			
			if self._Name and _Modules[self] and _Modules[self][self._Name] == self then
				_Modules[self][self._Name] = nil
			end
			
			twipe(self)			
			
			if getfenv(2) == self then
				if _Modules[self] then
					setfenv(2, _Modules[self])
				else
					setfenv(2, _G)
				end
			end
		end,
		
		-- NewModule
		["NewModule"] = function(self, name, setenv)
			if name and type(name) == "string" and strtrim(name) ~= "" then
				local module = CreateNameSpace(self, strsplit(".", name))
				
				if setenv ~= false then
					setfenv(2, module)
				end
				
				return module
			end
		end,
		
		-- Event API
	    -- IsEventRegistered("event") - Returns true if the given event is registered to the frame.
        ["IsEventRegistered"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				-- Custom Event
				return (_EventDistribution[event] and _EventDistribution[event][self] and true) or false
			else
				return false
			end
        end,
		
	    -- RegisterEvent("event") - Indicate that this frame should be notified when event occurs. 
        ["RegisterEvent"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				_EventDistribution[event] = _EventDistribution[event] or setmetatable({}, _metaWeak)
				
				_EventDistribution[event][self] = true
				
				return _EventCtrl:RegisterEvent(event)
			end
        end,
		
	    -- UnregisterEvent("event") - Indicate that this frame should no longer be notified when event occurs. 
        ["UnregisterEvent"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				if _EventDistribution[event] then
					_EventDistribution[event][self] = nil					
					if (not _UsedEvent[event]) and (not next(_EventDistribution[event])) then
						_EventCtrl:UnregisterEvent(event)
					end
				end				
			end
        end,
		
		-- UnregisterAllEvents
		["UnregisterAllEvents"] = function(self)
			for event, list in pairs(_EventDistribution) do
				if list[self] then
					list[self] = nil					
					if (not _UsedEvent[event]) and (not next(list)) then
						_EventCtrl:UnregisterEvent(event)
					end
				end
			end
		end,
		
		-- FireEvent, can only fire custom event, fire system event will do nothing
		["FireEvent"] = function(self, event, ...)
			if not event or type(event) ~= "string" or event == "" then
				return
			end
			
			return _EventCtrl:FireEvent(event, ...)
		end,
	}
	
	local _FuncProxy = {
		-- AddSlashCmd
		["AddSlashCmd"] = function(self, ...)
			if select('#', ...) == 0 then
				return nil
			end
			
			local cmd = select(1, ...)
			if not cmd or type(cmd) ~= "string" then
				error("The identifier must be a string", 2)
			end
			
			local slash = {}
			local sl
			for i = 2, select('#', ...) do
				sl = select(i, ...)
				if sl and type(sl) == "string" and strfind(sl, "^/") then
					tinsert(slash, select(3, strfind(sl, "^(/%S+)")))
				end
			end
			
			if not next(slash) then
				error("There must be some slash commands", 2)
			end
			
			local func = select(select('#',...), ...)
			if not func or type(func) ~= "function" then
				error("The last parameter must be a handler fucntion.", 2)
			end
			
			for i, v in ipairs(slash) do
				_G["SLASH_"..cmd..i] = v
			end
			
			_G["SlashCmdList"][cmd] = func
		end,
		
		-- AddSavedVariable
		["AddSavedVariable"] = function(self, name)
			if name and type(name) == "string" and strtrim(name) ~= "" then
				name = strtrim(name)
				if not _EventCtrl._VariablesLoaded then
					_SavedVariables[self] = _SavedVariables[self] or {}
					_SavedVariables[self][name] = true
				end
				
				_G[name] = _G[name] or {}
				self[name] = _G[name]
				return self[name]
			else
				error("The SavedVariable's name must be a string", 2)
			end
		end,
		
		-- Event API
	    -- IsEventRegistered("event") - Returns true if the given event is registered to the frame.
        ["IsEventRegistered"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				-- Custom Event
				return (_EventDistribution[event] and _EventDistribution[event][self] and true) or false
			else
				return false
			end
        end,
		
	    -- RegisterEvent("event") - Indicate that this frame should be notified when event occurs. 
        ["RegisterEvent"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				_EventDistribution[event] = _EventDistribution[event] or setmetatable({}, _metaWeak)
				
				_EventDistribution[event][self] = true
				
				return _EventCtrl:RegisterEvent(event)
			end
        end,
		
	    -- UnregisterEvent("event") - Indicate that this frame should no longer be notified when event occurs. 
        ["UnregisterEvent"] = function(self, event)
			if event and type(event) == "string" and event ~= "" then
				if _EventDistribution[event] then
					_EventDistribution[event][self] = nil					
					if (not _UsedEvent[event]) and (not next(_EventDistribution[event])) then
						_EventCtrl:UnregisterEvent(event)
					end
				end				
			end
        end,
		
		-- UnregisterAllEvents
		["UnregisterAllEvents"] = function(self)
			for event, list in pairs(_EventDistribution) do
				if list[self] then
					list[self] = nil					
					if (not _UsedEvent[event]) and (not next(list)) then
						_EventCtrl:UnregisterEvent(event)
					end
				end
			end
		end,
		
		-- FireEvent, can only fire custom event, fire system event will do nothing
		["FireEvent"] = function(self, event, ...)
			if not event or type(event) ~= "string" or event == "" then
				return
			end
			
			return _EventCtrl:FireEvent(event, ...)
		end,
		
		-- Fire
		["Fire"] = function(self, handler, ...)
			local chk = false
			
			if not handler then return end
			
			if self.__StackScripts[handler] then
				for i = 1, getn(self.__StackScripts[handler]) do
					chk = self.__StackScripts[handler][i](self, ...)
					if not self.__StackScripts then
						-- means it's disposed
						chk = true
					end
					if chk then
						break
					end
				end
				
				if not chk and self.__StackScripts[handler][0] then
					self.__StackScripts[handler][0](self, ...)
				end
			elseif self[handler] and type(self[handler]) == "function" then
				self[handler](self, ...)
			end
		end,
		
		-- HasScript
		["HasScript"] = function(self, handler)
			if not handler or type(handler) ~= "string" then
				return false
			end
			return (_ScriptHandler[handler] and true) or false
		end,
		
		-- GetScript
		["GetScript"] = function(self, handler)
			if not handler or type(handler) ~= "string" then
				return nil
			end
			return (self.__StackScripts[handler] and self.__StackScripts[handler][0]) or nil
		end,
		
		-- SetScript
		["SetScript"] = function(self, handler, func)
			if not handler or type(handler) ~= "string" then
				error("The handler can't be use", 2)
			end
			if not self:HasScript(handler) then
				error("The handler is not exist.", 2)
			end
			if func and type(func) ~= "function" then
				error("This is a script handler, must be a function", 2)
			end
			
			if not self.__StackScripts[handler] or type(self.__StackScripts[handler]) ~= "table" then
				self.__StackScripts[handler] = {}
			end
			
			rawset(self.__StackScripts[handler], 0, func)
		end,
		
		-- StackScript
		["StackScript"] = function(self, handler, func, flag)
			if type(handler) ~= "string" then
				error("The handler's name must be a string", 2)
			end
			if not self:HasScript(handler) then
				error("The handler is not exist.", 2)
			end
			if func and type(func) ~= "function" then
				error("The handler must be a function", 2)
			end
			
			if not self.__StackScripts[handler] or type(self.__StackScripts[handler]) ~= "table" then
				self.__StackScripts[handler] = {}
			end
			
			if flag then
				-- Clear this handler's stack
				for i = getn(self.__StackScripts[handler]), 1, -1 do					
					tremove(self.__StackScripts[handler], i)
				end
			end
			
			-- Check if it exist
			for i = getn(self.__StackScripts[handler]), 1, -1 do
				if self.__StackScripts[handler][i] == func then
					return
				end
			end
			
			tinsert(self.__StackScripts[handler], func)			
		end,
		
		-- UnStackScript
		["UnStackScript"] = function(self, handler, func)
			if type(handler) ~= "string" then
				error("The handler's name must be a string", 2)
			end
			if not self:HasScript(handler) then
				error("The handler is not exist.", 2)
			end
			if not func then
				error("Don't use nil for unstack function.", 2)
			elseif func and type(func) ~= "function" then
				error("The unstack function must be a function", 2)
			end
			
			if not self.__StackScripts[handler] or type(self.__StackScripts[handler]) ~= "table" then
				return
			end
			
			-- Clear this handler's stack
			for i = getn(self.__StackScripts[handler]), 1, -1 do
				if self.__StackScripts[handler][i] == func then
					tremove(self.__StackScripts[handler], i)
					return
				end
			end
		end,
		
		-- Insert or Clear parameters
		["GetMetadata"] = function(self, field)
			return self.__Meta[field] or GetAddOnMetadata(self.__Meta["Name"], field)
		end,
		
		["SetMetadata"] = function(self, field, value)
			if field and type(field) == "string" then
				self.__Meta[field] = tostring(value)
			end
		end,
		
		-- Enable or Disable
		["Enable"] = function(self)
			v.__Disabled = nil
			return self:Fire("OnEnable")
		end,
		
		["Disable"] = function(self)
			v.__Disabled = true
			return self:Fire("OnDisable")
		end,
		
		["IsEnabled"] = function(self)
			return (not v.__Disabled and true) or false
		end,
		
		-- Frames
		["AddFrame"] = function(self, frame)
			if frame and type(frame) == "table" and frame.Name then
				self.__Frames[frame] = true
				self[frame.Name] = frame
			end
		end,
		
		["RemoveFrame"] = function(self, frame)
			if frame and type(frame) == "table" and frame.Name then
				self.__Frames[frame] = nil
				if self[frame.Name] == frame then
					self[frame.Name] = nil
				end
				frame:Dispose()
			end
		end,
		
		-- Dispose
		["Dispose"] = function(self)
			self:Fire("OnDispose")
			
			for fr in pairs(self.__Frames) do
				if fr.Dispose then
					fr:Dispose()
				end
			end
			
			self:UnregisterAllEvents()
			
			setmetatable(self, nil)
			
			twipe(self)
			
			for i, v in pairs(_RegisterAddon) do
				if v == self then
					_RegisterAddon[i] = nil
					break
				end
			end
			
			_SavedVariables[self] = nil
			
			if getfenv(2) == self then
				setfenv(2, _G)
			end
		end,
		
		-- NewModule
		["NewModule"] = function(self, name, setenv)
			if name and type(name) == "string" and strtrim(name) ~= "" then
				local module = CreateNameSpace(self, strsplit(".", name))
				
				if setenv ~= false then
					setfenv(2, module)
				end
				
				return module
			end
		end,
	}
	
	-- Metatable Define
	_metaModule.__index = function(_table, _key)
		if _FuncModule[_key] then
			return _FuncModule[_key]
		end
		
		if _Modules[_table] then
			return _Modules[_table][_key]
		end
		
		return _G[_key]
	end
	
    _metaAddon.__index = function(_table, _key)
		-- Special key for table
		if _WeakTable[_key] then
			rawset(_table, _key, setmetatable({}, _metaWeak))
			return rawget(_table, _key)
		end
		
		if _UsedTable[_key] then
			rawset(_table, _key, {})
			return rawget(_table, _key)
		end

		if _FuncProxy[_key] then
			return _FuncProxy[_key]
		end

		-- Scripts
		if _table.__StackScripts[_key] then
			return rawget(_table.__StackScripts[_key], 0)
		end
		
		if _G[_key] then
			if type(_G[_key]) == "function" or type(_G[_key]) == "table" then
				rawset(_table, _key, IGAS:GetWrapper(_G[_key]))
				return rawget(_table, _key)
			end
			
			return _G[_key]
		end
	end
	
	_metaAddon.__newindex = function(_table, _key, _value)		
		-- Scripts
		if _ScriptHandler[_key] then
			if _value and type(_value) ~= "function" then
				error("The script handler must be a function", 2)
			end
			
			if not _table.__StackScripts[_key] or type(_table.__StackScripts[_key]) ~= "table" then
				rawset(_table.__StackScripts[_key], {})
			end
			
			rawset(_table.__StackScripts[_key], 0, _value)
			
			return
		end
		
		rawset(_table,_key,_value)			-- Other key can be set as usual
	end
    	
	-- Register Function to IGAS
	IGAS:RegisterFunction("NewAddon", function(self, addonName, setEnv, info)		
		if addonName and type(addonName) == "string" and strtrim(addonName) ~= "" then		
			local namespace = CreateNameSpace(_RegisterAddon, strsplit(".", addonName))			
			local addon = namespace
			
			-- Get addon
			while _Modules[addon] do
				addon = _Modules[addon]
			end
			
			if info and type(info) == "table" and next(info) then
				for k, v in pairs(info) do
					if type(k) == "string" then
						addon.__Meta[k] = tostring(v)
					end
				end
			end
			
			-- Set the environment to addon self. So the defines of the functions and variables will set to the addon directly.
			if setEnv ~= false then
				setfenv(2, namespace)
			end
			
			return namespace
		end
	end)
	
	IGAS:RegisterFunction("IsAddonExist", function(self, addonName)
		return (_RegisterAddon[addonName] and true) or false
	end)
end