-- Author  : Kurapica
-- Create Date : 2009/11/16

do
	-- Check Exist
	if IGAS.RegisterCustomEvent then
		return
	end

	local geterrorhandler = geterrorhandler
	local pcall = pcall
	
	local function errorhandler(err)
		return geterrorhandler()(err)
	end
	
	-- Custom Event System
	local _CEDistribution = {}
	local _CESource = {}
	
	-- Weak Table
	local _meta = {__mode = "k"}
	
	-- Register API
	
	--- RegisterCustomEvent
	IGAS:RegisterFunction("RegisterCustomEvent", function(self, frame, event)
		if frame and type(frame) == "table" and event and type(event) == "string" and event ~= "" then
			_CEDistribution[event] = _CEDistribution[event] or setmetatable({}, _meta)
			
			_CEDistribution[event][frame] = true
		end
	end)
	
	--- IsCustomEventRegistered
	IGAS:RegisterFunction("IsCustomEventRegistered", function(self, frame, event)
		if frame and type(frame) == "table" and event and type(event) == "string" and event ~= "" then
			return (_CEDistribution[event] and _CEDistribution[event][frame] and true) or false
		else
			return false
		end
	end)
	
	-- UnregisterAllCustomEvents
	IGAS:RegisterFunction("UnregisterAllCustomEvents", function(self, frame)
		if frame and type(frame) == "table" and event and type(event) == "string" and event ~= "" then
			for _, s in pairs(_CEDistribution) do
				s[frame] = nil
			end
		end
	end)
	
	-- UnregisterCustomEvent
	IGAS:RegisterFunction("UnregisterCustomEvent", function(self, frame, event)
		if frame and type(frame) == "table" and event and type(event) == "string" and event ~= "" then
			if _CEDistribution[event] then
				_CEDistribution[event][frame] = nil
			end
		end
	end)
		
	-- FireCustomEvent
	IGAS:RegisterFunction("FireCustomEvent", function(self, frame, event, ...)
		local chk, ret
		
		if not frame or type(frame) ~= "table" or not event or type(event) ~= "string" or event == "" then
			return
		end
		
		if _CESource[event] then
			-- On time one event can only have one source,
			return
		end
		
		-- Set the event's source
		_CESource[event] = frame			
		
		if _CEDistribution[event] then
			for v in pairs(_CEDistribution[event]) do
				if v.Fire and type(v.Fire) == "function" then
					pcall(v.Fire, v, "OnEvent", event, ...)
				elseif v[event] and type(v[event]) == "function" then
					pcall(v[event], v, event, ...)
				end
			end
		end
		
		-- Finish, clear the event's source
		_CESource[event] = nil
	end)
end