﻿-- Author      : Kurapica
-- Create Date : 8/18/2009

-- Animation
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Animation", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Animation"
    local _Base = "UIObject"

	-- Enum
	local _SmoothingEnum = IGAS:NewEnum("SmoothingType", {"IN", "OUT", "IN_OUT"})
	
	-- Property
	local _SmoothingType = IGAS:NewPropertyType("SmoothingType", {
			["Type"] = "enum",
			["EnumType"] = _SmoothingEnum,
		})
	
	IGAS:NewPropertyType("Origin", {
		["Type"] = "table",
		["Members"] = {
			["point"] = {
				["Type"] = "enum",
				["EnumType"] = IGAS:GetEnum("FramePoint"),
			},
			["x"] = {
				["Type"] = "number",
			},
			["y"] = {
				["Type"] = "number",
			}, 
		},
	})
		
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- Animation:Play() - Play the animation. 
		["Play"] = function(self)
			return self.__UI:Play()
		end,
	    -- Animation:Pause() - Pause the animation. 
		["Pause"] = function(self)
			return self.__UI:Pause()
		end,
	    -- Animation:Stop() - Stop the animation. 
		["Stop"] = function(self)
			return self.__UI:Stop()
		end,
	    -- Animation:IsDone() - Returns true if the animation has finished playing. 
		["IsDone"] = function(self)
			return self.__UI:IsDone()
		end,
	    -- Animation:IsPlaying() - Returns true if the animation is playing. 
		["IsPlaying"] = function(self)
			return self.__UI:IsPlaying()
		end,
	    -- Animation:IsPaused() - Returns true if the animation is paused. 
		["IsPaused"] = function(self)
			return self.__UI:IsPaused()
		end,
	    -- Animation:IsStopped() - Returns true if the animation is stopped. 
		["IsStopped"] = function(self)
			return self.__UI:IsStopped()
		end,
	    -- Animation:IsDelaying() - Returns true if the animation is in the middle of a start or end delay. 
		["IsDelaying"] = function(self)
			return self.__UI:IsDelaying()
		end,
	    -- Animation:GetElapsed() - Gets the amount of time in seconds that the animation has been playing for. 
		["GetElapsed"] = function(self)
			return self.__UI:GetElapsed()
		end,
	    -- Animation:SetStartDelay(delaySec) - Set the number of seconds that the animation delays before it starts to progress. 
		["SetStartDelay"] = function(self, delaySec)
			return self.__UI:SetStartDelay(delaySec)
		end,
	    -- Animation:GetStartDelay() - Get the number of seconds that the animation delays before it starts to progress. 
		["GetStartDelay"] = function(self)
			return self.__UI:GetStartDelay()
		end,
	    -- Animation:SetEndDelay(delaySec) - Set the number of seconds the animation delays after finishing. 
		["SetEndDelay"] = function(self, delaySec)
			return self.__UI:SetEndDelay(delaySec)
		end,
	    -- Animation:GetEndDelay() - Get the number of seconds the animation delays after finishing. 
		["GetEndDelay"] = function(self)
			return self.__UI:GetEndDelay()
		end,
	    -- Animation:SetDuration(durationSec) - Set the number of seconds it takes for the animation to progress from start to finish. 
		["SetDuration"] = function(self, durationSec)
			return self.__UI:SetDuration(durationSec)
		end,
	    -- Animation:GetDuration() - Get the number of seconds it takes for the animation to progress from start to finish. 
		["GetDuration"] = function(self)
			return self.__UI:GetDuration()
		end,
	    -- Animation:GetProgress() - Returns the progress of the animation as a unit value [0,1]. Ignores start and end delay. 
		["GetProgress"] = function(self)
			return self.__UI:GetProgress()
		end,
	    -- Animation:GetSmoothProgress() - Returns a smoothed, [0,1] progress value for the animation. 
		["GetSmoothProgress"] = function(self)
			return self.__UI:GetSmoothProgress()
		end,
	    -- Animation:GetProgressWithDelay() - Returns the progress of the animation combined with its start and end delay. 
		["GetProgressWithDelay"] = function(self)
			return self.__UI:GetProgressWithDelay()
		end,
	    -- Animation:SetMaxFramerate(framerate) - Sets the maximum frames per second that the animation will update its progress. 
		["SetMaxFramerate"] = function(self, framerate)
			return self.__UI:SetMaxFramerate(framerate)
		end,
	    -- Animation:GetMaxFramerate() - Gets the maximum frames per second that the animation will update its progress. 
		["GetMaxFramerate"] = function(self)
			return self.__UI:GetMaxFramerate()
		end,
	    -- Animation:SetOrder(order) - Sets the order that the animation plays within its parent group. Range is [1,100]. 
		["SetOrder"] = function(self, order)
			return self.__UI:SetOrder(order)
		end,
	    -- Animation:GetOrder() - Gets the order of the animation within its parent group. 
		["GetOrder"] = function(self)
			return self.__UI:GetOrder()
		end,
	    -- Animation:SetSmoothing(smoothType) - Sets the smoothing type for the animation. Input is [IN,OUT, or IN_OUT]. 
		["SetSmoothing"] = function(self, smoothType)
			return self.__UI:SetSmoothing(smoothType)
		end,
	    -- Animation:GetSmoothing() - Gets the smoothing type for the animation. 
		["GetSmoothing"] = function(self)
			return self.__UI:GetSmoothing()
		end,
	    -- Animation:GetRegionParent() - Gets the Region object that the animation operates on. The region object is this Animation's parent's parent (the AnimationGroup's parent). 
		["GetRegionParent"] = function(self)
			return IGAS:GetWrapper(self.__UI:GetRegionParent())
		end,
	}
	
	-- Property
	local _Property = {
        -- StartDelay
        ["StartDelay"] = {
			["Set"] = function(self, delaySec)
				self:SetStartDelay(delaySec)
			end,
			
			["Get"] = function(self)
				return self:GetStartDelay()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- EndDelay
        ["EndDelay"] = {
			["Set"] = function(self, delaySec)
				self:SetEndDelay(delaySec)
			end,
			
			["Get"] = function(self)
				return self:GetEndDelay()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Duration
        ["Duration"] = {
			["Set"] = function(self, durationSec)
				self:SetDuration(durationSec)
			end,
			
			["Get"] = function(self)
				return self:GetDuration()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- MaxFramerate
        ["MaxFramerate"] = {
			["Set"] = function(self, framerate)
				self:SetMaxFramerate(framerate)
			end,
			
			["Get"] = function(self)
				return self:GetMaxFramerate()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Order
        ["Order"] = {
			["Set"] = function(self, order)
				self:SetOrder(order)
			end,
			
			["Get"] = function(self)
				return self:GetOrder()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Smoothing
        ["Smoothing"] = {
			["Set"] = function(self, smoothType)
				if _SmoothingEnum[smoothType] then
					self:SetSmoothing(smoothType)
				end
			end,
			
			["Get"] = function(self)
				return self:GetSmoothing()
			end,
			
			["Type"] = _SmoothingType,
		},
	}
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
    }
    
    IGAS:NewWidget(_WidgetInfo, true)
end