﻿-- Author      : Kurapica
-- Create Date : 8/18/2009

-- AnimationGroup
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.AnimationGroup", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "AnimationGroup"
    local _Base = "UIObject"

	-- Enum
	local _LoopEnum = IGAS:NewEnum("LoopType", {"NONE", "REPEAT", "BOUNCE"})
	local _LoopState = IGAS:NewEnum("LoopState", {"NONE", "FORWARD", "REVERSE"})
	
	-- Property
	local _LoopEnumType = IGAS:NewPropertyType("LoopType", {
			["Type"] = "enum",
			["EnumType"] = _LoopEnum,
		})
	local _LoopStateType = IGAS:NewPropertyType("LoopState", {
			["Type"] = "enum",
			["EnumType"] = _LoopState,
		})
	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		--AnimationGroup:Play() - Start playing the animations in this group. 
		["Play"] = function(self)
			return self.__UI:Play()
		end,
	    --AnimationGroup:Pause() - Pause the animations in this group. 
		["Pause"] = function(self)
			return self.__UI:Pause()
		end,
	    --AnimationGroup:Stop() - Stop all animations in this group. 
		["Stop"] = function(self)
			return self.__UI:Stop()
		end,
	    --AnimationGroup:Finish() - Notify this group to stop playing once the current loop cycle is done. Does nothing if this group is not playing. 
		["Finish"] = function(self)
			return self.__UI:Finish()
		end,
	    --AnimationGroup:GetProgress() - Returns the progress of this animation as a unit value [0,1]. 
		["GetProgress"] = function(self)
			return self.__UI:GetProgress()
		end,
	    --AnimationGroup:IsDone() - Returns true if the group has finished playing. 
		["IsDone"] = function(self)
			return self.__UI:IsDone()
		end,
	    --AnimationGroup:IsPlaying() - Returns true if the group is playing. 
		["IsPlaying"] = function(self)
			return self.__UI:IsPlaying()
		end,
	    --AnimationGroup:IsPaused() - Returns true if the group is paused. 
		["IsPaused"] = function(self)
			return self.__UI:IsPaused()
		end,
	    --AnimationGroup:GetDuration() - Gets the total duration across all child Animations that the group will take to complete one loop cycle. 
		["GetDuration"] = function(self)
			return self.__UI:GetDuration()
		end,
	    --AnimationGroup:SetLooping(loopType) - Sets the type of looping for the group. Input is [NONE, REPEAT, or BOUNCE]. 
		["SetLooping"] = function(self, loopType)
			return self.__UI:SetLooping(loopType)
		end,
	    --AnimationGroup:GetLooping() - Gets the type of looping for the group. 
		["GetLooping"] = function(self)
			return self.__UI:GetLooping()
		end,
	    --AnimationGroup:GetLoopState() - Gets the current loop state of the group. Output is [NONE, FORWARD, or REVERSE]. 
		["GetLoopState"] = function(self)
			return self.__UI:GetLoopState()
		end,
	    --AnimationGroup:CreateAnimation("animationType", ["name"[,"inheritsFrom"]]) - Create and return an Animation as a child of this group. 
		["CreateAnimation"] = function(self, animationType, name, inheritsFrom)
			return IGAS:NewFrame(animationType, name, self, inheritsFrom)
		end,
	}
	
	-- Property
	local _Property = {	
        -- Looping
        ["Looping"] = {
			["Set"] = function(self, loopType)
				if _LoopEnum[loopType] then
					self:SetLooping(loopType)
				end
			end,
			
			["Get"] = function(self)
				return self:GetLooping()
			end,
			
			["Type"] = _LoopEnumType,
		},
        -- LoopState
        ["LoopState"] = {			
			["Get"] = function(self)
				return self:GetLoopState()
			end,
			
			["Type"] = _LoopStateType,
		},
	}
	    
    -- New      
    local _New = function(_parent,inheritsFrom)
		if _parent and _parent["CreateAnimationGroup"] then
			return IGAS:GetUI(_parent):CreateAnimationGroup(nil, inheritsFrom)
		end
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end