﻿-- Author      : Kurapica
-- Create Date : 7/12/2008 14:38
     
-- Button
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Button", version) then
		return
	end
    
    local CreateFrame = CreateFrame
    
	local _WidgetName = "Button"
    local _Base = "Frame"
    
    -- Button Template
    local TEMPLATE_NONE = "NONE"
    local TEMPLATE_NORMAL = "NORMAL"
    local TEMPLATE_GRAY = "GRAY"
    local TEMPLATE_CLOSEBTN = "CLOSE"
	local TEMPLATE_ADD = "PLUS"
	local TEMPLATE_MINUS = "MINUS"
    
    -- Define Block
	local _StyleEnum = IGAS:NewEnum("ButtonStyle", {
        TEMPLATE_NONE,
        TEMPLATE_NORMAL,
        TEMPLATE_GRAY,
        TEMPLATE_CLOSEBTN,
		TEMPLATE_ADD,
		TEMPLATE_MINUS,
    })
    
	local _StyleType = IGAS:NewPropertyType("ButtonStyle", {
		["Type"] = "enum",
		["EnumType"] = _StyleEnum,
	})
	
    -- Scripts
    local function OnClick_Close(self)
		if not self.Parent.InDesignMode then
			self.Parent.Visible = false
		end
    end
    
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- Button:Click() - Execute the click action of the button. 
        ["Click"] = function(self, ...)
            return self.__UI:Click(...)
        end,
		
	    -- Button:Disable() - Disable the Button so that it cannot be clicked. 
        ["Disable"] = function(self, ...)
            return self.__UI:Disable(...)
        end,
		
	    -- Button:Enable() - Enable to the Button so that it may be clicked. 
        ["Enable"] = function(self, ...)
            return self.__UI:Enable(...)
        end,
		
	    -- Button:GetButtonState() - Return the current state ("PUSHED","NORMAL") of the Button. 
        ["GetButtonState"] = function(self, ...)
            return self.__UI:GetButtonState(...)
        end,
		
	    -- Button:GetDisabledFontObject() - Return the font object for the Button when disabled - New in 1.10. 
        ["GetDisabledFontObject"] = function(self)
            return IGAS:GetWrapper(self.__UI:GetDisabledFontObject())
        end,
		
	    -- Button:GetDisabledTexture() - Get the texture for this button when disabled - New in 1.11. 
        ["GetDisabledTexture"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetDisabledTexture(...))
        end,
		
	    -- Button:GetFontString() - Get this button's label FontString - New in 1.11. 
        ["GetFontString"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetFontString(...))
        end,
		
	    -- Button:GetHighlightFontObject() - Return the font object for the Button when highlighted - New in 1.10. 
        ["GetHighlightFontObject"] = function(self)
            return IGAS:GetWrapper(self.__UI:GetHighlightFontObject())
        end,
		
	    -- Button:GetHighlightTexture() - Get the texture for this button when highlighted - New in 1.11. 
        ["GetHighlightTexture"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetHighlightTexture(...))
        end,
		
	    -- Button:GetNormalTexture() - Get the normal texture for this button - New in 1.11. 
        ["GetNormalTexture"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetNormalTexture(...))
        end,
		
	    -- Button:GetNormalFontObject() - Get the Normal Font Object of the button - new in 3.0.0. 
        ["GetNormalFontObject"] = function(self)
            return IGAS:GetWrapper(self.__UI:GetNormalFontObject())
        end,
		
	    -- Button:GetPushedTextOffset() - Get the text offset when this button is pushed (x, y) - New in 1.11. 
        ["GetPushedTextOffset"] = function(self, ...)
            return self.__UI:GetPushedTextOffset(...)
        end,
		
	    -- Button:GetPushedTexture() - Get the texture for this button when pushed - New in 1.11. 
        ["GetPushedTexture"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetPushedTexture(...))
        end,
		
	    -- Button:GetText() - Get the text label for the Button. 
        ["GetText"] = function(self, ...)
            return self.__UI:GetText(...)
        end,
		
	    -- Button:GetTextHeight() - Get the height of the Button's text. 
        ["GetTextHeight"] = function(self, ...)
            return self.__UI:GetTextHeight(...)
        end,
		
	    -- Button:GetTextWidth() - Get the width of the Button's text. 
        ["GetTextWidth"] = function(self, ...)
            return self.__UI:GetTextWidth(...)
        end,
		
	    -- Button:IsEnabled() - Determine whether the Button is enabled. 
        ["IsEnabled"] = function(self, ...)
            return self.__UI:IsEnabled(...)
        end,
		
	    -- Button:LockHighlight() - Set the Button to always be drawn highlighted. 
        ["LockHighlight"] = function(self, ...)
            return self.__UI:LockHighlight(...)
        end,
		
	    -- Button:RegisterForClicks("clickType"[,"clickType"...]) - Specify which mouse button up/down actions cause this button to receive an OnClick notification
        ["RegisterForClicks"] = function(self, ...)
            return self.__UI:RegisterForClicks(...)
        end,
		
	    -- Button:SetButtonState("state"[, lock]) - Set the state of the Button ("PUSHED", "NORMAL") and whether it is locked. 
        ["SetButtonState"] = function(self, state, locked)
            if locked ~= nil then
                self.__Locked = locked
            end
            
            self.__UI:SetButtonState(state, self.__Locked)
        end,
		
	    -- Button:SetDisabledFontObject([font]) - Set the font object for settings when disabled - New in 1.10. 
        ["SetDisabledFontObject"] = function(self, font)
            return self.__UI:SetDisabledFontObject(IGAS:GetUI(font))
        end,
		
	    -- Button:SetDisabledTextColor(r, g, b[, a]) - Set the disabled text color for the Button. 
        ["SetDisabledTextColor"] = function(self, ...)
            return self.__UI:SetDisabledTextColor(...)
        end,
		
	    -- Button:SetDisabledTexture(texture or "texturePath") - Set the disabled texture for the Button - Updated in 1.10. 
        ["SetDisabledTexture"] = function(self, texture, ...)
            return self.__UI:SetDisabledTexture(IGAS:GetUI(texture), ...)
        end,
		
	    -- Button:SetFont("font", size[,"flags"]) - Set the font to use for display. 
        ["SetFont"] = function(self, ...)
            return self.__UI:SetFont(...)
        end,
		
	    -- Button:SetFontString(fontString) - Set the button's label FontString - New in 1.11. 
        ["SetFontString"] = function(self, fontString)
            return self.__UI:SetFontString(IGAS:GetUI(fontString))
        end,
		
	    -- Button:SetFormattedText("formatstring"[, ...]) - Set the formatted text label for the Button. - New in 2.3. 
        ["SetFormattedText"] = function(self, ...)
            return self.__UI:SetFormattedText(...)
        end,
		
	    -- Button:SetHighlightFontObject([font]) - Set the font object for settings when highlighted - New in 1.10. 
        ["SetHighlightFontObject"] = function(self, font)
            return self.__UI:SetHighlightFontObject(IGAS:GetUI(font))
        end,
		
	    -- Button:SetHighlightTextColor(r, g, b[, a]) - Set the highlight text color for the Button. 
        ["SetHighlightTextColor"] = function(self, ...)
            return self.__UI:SetHighlightTextColor(...)
        end,
		
	    -- Button:SetHighlightTexture(texture or "texturePath"[,alphaMode]) - Set the highlight texture for the Button - Updated in 1.10. 
        ["SetHighlightTexture"] = function(self, texture, ...)
            return self.__UI:SetHighlightTexture(IGAS:GetUI(texture), ...)
        end,
		
	    -- Button:SetNormalTexture(texture or "texturePath") - Set the normal texture for the Button - Updated in 1.10. 
        ["SetNormalTexture"] = function(self, texture, ...)
            return self.__UI:SetNormalTexture(IGAS:GetUI(texture), ...)
        end,
		
	    -- Button:SetNormalFontObject(FontString) - Replaces SetTextFontObject - Updated in 3.00 
        ["SetNormalFontObject"] = function(self, font)
            return self.__UI:SetNormalFontObject(IGAS:GetUI(font))
        end,
		
	    -- Button:SetPushedTextOffset(x, y) - Set the text offset for this button when pushed - New in 1.11. 
        ["SetPushedTextOffset"] = function(self, ...)
            return self.__UI:SetPushedTextOffset(...)
        end,
		
	    -- Button:SetPushedTexture(texture or "texturePath") - Set the pushed texture for the Button - Updated in 1.10. 
        ["SetPushedTexture"] = function(self, texture, ...)
            return self.__UI:SetPushedTexture(IGAS:GetUI(texture), ...)
        end,
		
	    -- Button:SetText("text") - Set the text label for the Button. 
        ["SetText"] = function(self, ...)
            return self.__UI:SetText(...)
        end,
		
	    -- Button:UnlockHighlight() - Set the Button to not always be drawn highlighted. 
        ["UnlockHighlight"] = function(self, ...)
            return self.__UI:UnlockHighlight(...)
        end,	
		        
        -- Used for ButtonTemplate
        ["SetStyle"] = function(self, style)
            local t
            
            -- Check Style
            if not style or type(style) ~= "string" then
                return
            end
                        
            if (not _StyleEnum[style]) or style == self.__Style then
                return
            end
			
            self.__Style = style
			
            -- Change Style
            if style == TEMPLATE_NORMAL then
                --- Font
                self:SetNormalFontObject(GameFontNormal)
                self:SetDisabledFontObject(GameFontDisable)
                self:SetHighlightFontObject(GameFontHighlight)
                
                --- Texture
                self:SetNormalTexture("Interface\\Buttons\\UI-Panel-Button-Up")
                t = self:GetNormalTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetPushedTexture("Interface\\Buttons\\UI-Panel-Button-Down")
                t = self:GetPushedTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetDisabledTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
                t = self:GetDisabledTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetHighlightTexture("Interface\\Buttons\\UI-Panel-Button-Highlight")
                t = self:GetHighlightTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
            elseif style == TEMPLATE_GRAY then
                --- Font
                self:SetNormalFontObject(GameFontHighlight)
                self:SetDisabledFontObject(GameFontDisable)
                self:SetHighlightFontObject(GameFontHighlight)
                
                --- Texture
                self:SetNormalTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
                t = self:GetNormalTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetPushedTexture("Interface\\Buttons\\UI-Panel-Button-Disabled-Down")
                t = self:GetPushedTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetDisabledTexture("Interface\\Buttons\\UI-Panel-Button-Disabled")
                t = self:GetDisabledTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
                
                self:SetHighlightTexture("Interface\\Buttons\\UI-Panel-Button-Highlight")
                t = self:GetHighlightTexture()
                t:SetTexCoord(0,0.625,0,0.6875)
            elseif style == TEMPLATE_CLOSEBTN then
                --- Texture
                self:SetNormalTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Up")
                
                self:SetPushedTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Down")
                                
                self:SetHighlightTexture("Interface\\Buttons\\UI-Panel-MinimizeButton-Highlight")
                t = self:GetHighlightTexture()
                t:SetBlendMode("ADD")
                
                self.Width = 32
                self.Height = 32
                self.OnClick = OnClick_Close
			elseif style == TEMPLATE_ADD then
				self:SetNormalTexture("Interface\\Buttons\\UI-PlusButton-UP")
				self:SetPushedTexture("Interface\\Buttons\\UI-PlusButton-DOWN")
				self:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
				t = self:GetHighlightTexture()
				t:SetBlendMode("ADD")
				self.Width = 24
				self.Height = 24
			elseif style == TEMPLATE_MINUS then				
				self:SetNormalTexture("Interface\\Buttons\\UI-MinusButton-UP")
				self:SetPushedTexture("Interface\\Buttons\\UI-MinusButton-DOWN")
				self:SetHighlightTexture("Interface\\Buttons\\UI-PlusButton-Hilight")
				t = self:GetHighlightTexture()
				t:SetBlendMode("ADD")
				self.Width = 24
				self.Height = 24
            end
        end,
        
        ["GetStyle"] = function(self)
            return self.__Style or TEMPLATE_NONE
        end,
    }
    
	-- Property
	local _Property = {
        -- Style
        ["Style"] = {
			["Set"] = function(self, style)
                self:SetStyle(style)
			end,
			
			["Get"] = function(self)
				return self:GetStyle()
			end,
			
			["Type"] = _StyleType,
		},
        
        -- Enabled
        ["Enabled"] = {
			["Set"] = function(self, enabled)
                if enabled then
				    self:Enable()
                else
				    self:Disable()
                end
			end,
			
			["Get"] = function(self)
				return (self:IsEnabled() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
    
        -- ButtonState
        ["ButtonState"] = {
			["Set"] = function(self, state)
                self:SetButtonState(state, self.__Locked)
			end,
			
			["Get"] = function(self)
				return self:GetButtonState()
			end,
			
			["Type"] = IGAS:GetPropertyType("ButtonState"),
		},
        
        -- Locked
        ["Locked"] = {
			["Set"] = function(self, enabled)
                self.__Locked = enabled
                self:SetButtonState(self.__UI:GetButtonState(), enabled)
			end,
			
			["Get"] = function(self)
				return (self.__Locked and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
        
        -- DisabledFontObject
        ["DisabledFontObject"] = {
			["Set"] = function(self, font)
                self:SetDisabledFontObject(font)
			end,
			
			["Get"] = function(self)
				return self:GetDisabledFontObject()
			end,
			
			["Type"] = IGAS:GetPropertyType("Object"),
		},
     
     	-- DisabledTexture
     	["DisabledTexture"] = {
			["Set"] = function(self, texture)
				self:SetDisabledTexture(texture)
			end,
			
			["Get"] = function(self)
				return self:GetDisabledTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),     	
     	},
     	
     	-- HighlightTexture
     	["HighlightTexture"] = {
			["Set"] = function(self, texture)
				self:SetHighlightTexture(texture)
			end,
			
			["Get"] = function(self)
				return self:GetHighlightTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),     	
     	},    
    
     	-- NormalTexture
     	["NormalTexture"] = {
			["Set"] = function(self, texture)
				self:SetNormalTexture(texture)
			end,
			
			["Get"] = function(self)
				return self:GetNormalTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),     	
     	},    
  	
     	-- PushedTexture
     	["PushedTexture"] = {
			["Set"] = function(self, texture)
				self:SetPushedTexture(texture)
			end,
			
			["Get"] = function(self)
				return self:GetPushedTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
     	},  
  			
		--- FontString
		["FontString"] = {
			["Set"] = function(self, label)
				self:SetFontString(label)
			end,
			
			["Get"] = function(self)
				return self:GetFontString()
			end,
			
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		
		--- HighlightFontObject
		["HighlightFontObject"] = { 
			["Set"] = function(self, fontObject)
				self:SetHighlightFontObject(fontObject)
			end,

			["Get"] = function(self)
				return self:GetHighlightFontObject()
			end,

			["Type"] = IGAS:GetPropertyType("Object"),
		}, 
		
		--- NormalFontObject
		["NormalFontObject"] = { 
			["Set"] = function(self, fontObject)
				self:SetNormalFontObject(fontObject)
			end,

			["Get"] = function(self)
				return self:GetNormalFontObject()
			end,

			["Type"] = IGAS:GetPropertyType("Object"),
		}, 
  	   
		--- PushedTextOffset
		["PushedTextOffset"] = { 
			["Set"] = function(self, offset)
				self:SetPushedTextOffset(offset.x, offset.y)
			end,

			["Get"] = function(self)
				local offset = {}
				offset.x, offset.y = self:GetPushedTextOffset()

				return offset
			end,

			["Type"] = IGAS:GetPropertyType("Offset"),
		},
        
		--- Text
		["Text"] = {
			["Set"] = function(self, text)
				self:SetText(text)
			end,
			
			["Get"] = function(self)
				return self:GetText()
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
    
        -- HighlightLocked
        ["HighlightLocked"] = {
			["Set"] = function(self, locked)
                if locked then
				    self.__UI:LockHighlight()
				    self.__HighlightLocked = true
                else
				    self.__UI:UnlockHighlight()
				    self.__HighlightLocked = false
                end
			end,
			
			["Get"] = function(self)
				return (self.__HighlightLocked and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
    }
	
    -- New      
    local _New = function(_parent, inheritFrame)
        -- New Frame
		return CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end