﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 15:01
     
-- CheckButton
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.CheckButton", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "CheckButton"
    local _Base = "Button"
   	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- CheckButton:GetChecked() - Get the status of the checkbox. 
		["GetChecked"] = function(self, ...)
			return self.__UI:GetChecked(...)
		end,
	    -- CheckButton:GetCheckedTexture() - Get the texture used for a checked box - New in 1.11. 
		["GetCheckedTexture"] = function(self, ...)
			return IGAS:GetWrapper(self.__UI:GetCheckedTexture(...))
		end,
	    -- CheckButton:GetDisabledCheckedTexture() - Get the texture used for a disabled checked box - New in 1.11. 
		["GetDisabledCheckedTexture"] = function(self, ...)
			return IGAS:GetWrapper(self.__UI:GetDisabledCheckedTexture(...))
		end,
	    -- CheckButton:SetChecked([state]) - Set the status of the checkbox. 
		["SetChecked"] = function(self, ...)
			return self.__UI:SetChecked(...)
		end,
	    -- CheckButton:SetCheckedTexture(texture) - Set the texture to use for a checked box. 
		["SetCheckedTexture"] = function(self, texture, ...)
			return self.__UI:SetCheckedTexture(IGAS:GetUI(texture), ...)
		end,
	    -- CheckButton:SetDisabledCheckedTexture(texture) - Set the texture to use for a disabled but checked box. 
		["SetDisabledCheckedTexture"] = function(self, texture, ...)
			return self.__UI:SetDisabledCheckedTexture(IGAS:GetUI(texture), ...)
		end,
	
		-- Override
        ["Enable"] = function(self)
            self.__UI:Enable()
            self:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        end,
        ["Disable"] = function(self)
            self.__UI:Disable()
            self:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled.blp")
        end,
    }
    
	-- Property
	local _Property = {
        -- Checked
        ["Checked"] = {
			["Set"] = function(self, value)
				self.__UI:SetChecked(value)
			end,
			
			["Get"] = function(self)				
				return (self.__UI:GetChecked() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
    
        -- CheckedTexture
        ["CheckedTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetCheckedTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetCheckedTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
    
        -- DisabledCheckedTexture
        ["DisabledCheckedTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetDisabledCheckedTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetDisabledCheckedTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
    }
	
    -- New      
    local _New = function(_parent,inheritFrame)
        -- New Frame
		local frame = CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
        
        frame:SetNormalTexture("Interface\\Buttons\\UI-CheckBox-Up.blp")
        frame:SetPushedTexture("Interface\\Buttons\\UI-CheckBox-Down.blp")
        frame:SetHighlightTexture("Interface\\Buttons\\UI-CheckBox-Highlight.blp", "ADD")
        frame:SetCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        frame:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check.blp")
        --frame:SetDisabledCheckedTexture("Interface\\Buttons\\UI-CheckBox-Check-Disabled.blp")
        
        frame:SetWidth(24)
		frame:SetHeight(24)
        
        return frame
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end