﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 15:19
-- ChangeLog   :
--				2010.01.12	Add the default parameters for _New
   
-- FontString
do
	-- Check Version
	local version = 3
	if not IGAS:RegisterFile("IGAS.GUI.Widget.FontString", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "FontString"
    local _Base = "LayeredRegion"
	
    -- FuncProxy
	local _FuncProxy = {
		
        -- SetParent(parent or "parentName") - Set the parent for this frame
        ["SetParent"] = function(self, parent)
			-- Get frame from name
			if parent and type(parent) == "string" then
				parent = _G[parent]
				
				if not parent then
					error("No such frame", 2)
				end
			end
			
			parent = IGAS:GetWrapper(parent)
			
			-- Check parent
			if parent and (type(parent) ~= "table" or not parent["IsIGASFrame"] or not parent["AddChild"]) then 
				error("This object can't add child.", 2)
			end
			
			-- Set parent
			if parent then
				if self.__Name then
	                -- Check
	                if parent.__Childs[self.__Name] then
	                    if parent.__Childs[self.__Name] ~= self then
	                        error("The parent has a child with the same name")
	                    else
	                        return
	                    end
	                end
	                
	                -- Remove ob from it's old parent
					self:SetParent(nil)
	                
	                -- Add ob to new parent
	                self.__Parent = parent
	                parent.__Childs[self.__Name] = self
					
					-- SetParent
					if IGAS:GetUI(self) ~= self and IGAS:GetUI(self)["SetParent"] then
						IGAS:GetUI(self):SetParent(IGAS:GetUI(parent))
					end
	            else
	                error("can't add this child", 2)
	            end
			else
				if self.__Name and self.__Parent then
					self.__Parent.__Childs[self.__Name] = nil
				end
				self.__Parent = nil
			end
        end,
		
		--FontString:CanNonSpaceWrap() - Get whether long strings without spaces are wrapped or truncated - New in 1.11. 
		["CanNonSpaceWrap"] = function(self, ...)
			return self.__UI:CanNonSpaceWrap(...)
		end,
	    --FontString:GetStringHeight() - Returns the height in pixels of the current string in the current font (without line wrapping). - New in 2.3. 
		["GetStringHeight"] = function(self, ...)
			return self.__UI:GetStringHeight(...)
		end,
	    --FontString:GetStringWidth() - Returns the width in pixels of the current string in the current font (without line wrapping). 
		["GetStringWidth"] = function(self, ...)
			return self.__UI:GetStringWidth(...)
		end,
	    --FontString:GetText() - Get the displayed text. 
		["GetText"] = function(self, ...)
			return self.__UI:GetText(...)
		end,
	    --FontString:SetAlphaGradient(start, length) - Create or remove an alpha gradient over the text. 
		["SetAlphaGradient"] = function(self, ...)
			return self.__UI:SetAlphaGradient(...)
		end,
	    --FontString:SetFormattedText("formatstring"[, ...]) - Set the formatted display text. - New in 2.3. 
		["SetFormattedText"] = function(self, ...)
			return self.__UI:SetFormattedText(...)
		end,
	    --FontString:SetNonSpaceWrap(wrapFlag) - Set whether long strings without spaces are wrapped or truncated. 
		["SetNonSpaceWrap"] = function(self, ...)
			return self.__UI:SetNonSpaceWrap(...)
		end,
	    --FontString:SetText("text") - Set the displayed text. 
		["SetText"] = function(self, ...)
			return self.__UI:SetText(...)
		end,
	    --FontString:SetTextHeight(pixelHeight) - Set the height of the text by scaling graphics (Note: Can distort text). 
		["SetTextHeight"] = function(self, ...)
			return self.__UI:SetTextHeight(...)
		end,
	
        -- GetFont() - Return the font file, height, and flags. 
        ["GetFont"] = function(self)
            return self.__UI:GetFont()
        end,
        
        -- GetFontObject() - Return the 'parent' Font object, or nil if none. 
        ["GetFontObject"] = function(self)
            return IGAS:GetWrapper(self.__UI:GetFontObject())
        end,
        
        -- GetJustifyH() - Return the horizontal text justification. 
        ["GetJustifyH"] = function(self)
            return self.__UI:GetJustifyH()
        end,
        
        -- GetJustifyV() - Return thevertical text justification. 
        ["GetJustifyV"] = function(self)
            return self.__UI:GetJustifyV()
        end,
        
        -- GetShadowColor() - Returns the color of text shadow (r, g, b, a). 
        ["GetShadowColor"] = function(self)
            return self.__UI:GetShadowColor()
        end,
        
        -- GetShadowOffset() - Returns the text shadow offset (x, y). 
        ["GetShadowOffset"] = function(self)
            return self.__UI:GetShadowOffset()
        end,
        
        -- GetSpacing() - Returns the text spacing. 
        ["GetSpacing"] = function(self)
            return self.__UI:GetSpacing()
        end,
        
        -- GetTextColor() - Returns the default text color. 
        ["GetTextColor"] = function(self)
            return self.__UI:GetTextColor()
        end,
        
        -- SetFont("path", height[,"flags"]) - Sets the font to use for text, returns 1 if the path was valid, nil otherwise (no change occurs). 
        ["SetFont"] = function(self, path, height, flags)
            self.__UI:SetFont(path, height, flags)
        end,
        
        -- SetFontObject(fontObject) - Sets the 'parent' Font object from which this object inherits properties. 
        ["SetFontObject"] = function(self, fontObject)
            self.__UI:SetFontObject(IGAS:GetUI(fontObject))
        end,
        
        -- SetJustifyH("justifyH") - Sets horizontal text justification ("LEFT","RIGHT", or "CENTER") 
        ["SetJustifyH"] = function(self, justifyH)
            self.__UI:SetJustifyH(justifyH)
        end,
        
        -- SetJustifyV("justifyV") - Sets vertical text justification ("TOP","BOTTOM", or "MIDDLE") 
        ["SetJustifyV"] = function(self, justifyV)
            self.__UI:SetJustifyV(justifyV)
        end,
        
        -- SetShadowColor(r, g, b[, a]) - Sets the text shadow color. 
        ["SetShadowColor"] = function(self, r,g,b,a)
            self.__UI:SetShadowColor(r,g,b,a)
        end,
        
        -- SetShadowOffset(x, y) - Sets the text shadow offset. 
        ["SetShadowOffset"] = function(self, x,y)
            self.__UI:SetShadowOffset(x,y)
        end,
        
        -- SetSpacing(spacing) - Sets the spacing between lines of text in the object. 
        ["SetSpacing"] = function(self, spacing)
            self.__UI:SetSpacing(spacing)
        end,
        
        -- SetTextColor(r, g, b[, a]) - Sets the default text color. 
        ["SetTextColor"] = function(self, r,g,b,a)
            self.__UI:SetTextColor(r,g,b,a)
        end,
    }
    
	-- Property
	local _Property = {
		--- Font
		["Font"] = { 
			["Set"] = function(self, font)
				local flags = ""
				if font.outline then
					if font.outline == "NORMAL" then
						flags = flags.."OUTLINE"
					elseif font.outline == "THICK" then
						flags = flags.."THICKOUTLINE"
					end
				end
				if font.monochrome then
					if flags ~= "" then
						flags = flags..","
					end 
					flags = flags.."MONOCHROME"
				end

				self:SetFont(font.path, font.size, flags)
			end,

			["Get"] = function(self)
				local font = {}
				local flags
				font.path, font.size, flags = self:GetFont()
				flags = flags or ""
				if string.find(flags, "THICKOUTLINE") then
					font.outline = "THICK"
				elseif string.find(flags, "OUTLINE") then
					font.outline = "NORMAL"
				end
				if string.find(flags, "MONOCHROME") then
					font.monochrome = true 
				end 

				return font
			end,

			["Type"] = IGAS:GetPropertyType("Font"),
		}, 
		--- FontObject
		["FontObject"] = { 
			["Set"] = function(self, fontObject)
				self:SetFontObject(fontObject)
			end,

			["Get"] = function(self)
				return self:GetFontObject()
			end,

			["Type"] = IGAS:GetPropertyType("Object"),
		}, 
		--- JustifyH
		["JustifyH"] = { 
			["Set"] = function(self, justifyH)
				self:SetJustifyH(justifyH)
			end,

			["Get"] = function(self)
				return self:GetJustifyH()
			end,

			["Type"] = IGAS:GetPropertyType("JustifyH"),
		}, 
		--- JustifyV
		["JustifyV"] = { 
			["Set"] = function(self, justifyV)
				self:SetJustifyV(justifyV)
			end,

			["Get"] = function(self)
				return self:GetJustifyV()
			end,

			["Type"] = IGAS:GetPropertyType("JustifyV"),
		},
		--- ShadowColor
		["ShadowColor"] = { 
			["Set"] = function(self, color)
				self:SetShadowColor(color.red, color,green, color.blue, color.alpha)
			end,

			["Get"] = function(self)
				local color = {}
				color.red, color,green, color.blue, color.alpha = self:GetShadowColor()

				return color
			end,

			["Type"] = IGAS:GetPropertyType("Color"),
		},
		--- ShadowOffset
		["ShadowOffset"] = { 
			["Set"] = function(self, offset)
				self:SetShadowOffset(offset.x, offset.y)
			end,

			["Get"] = function(self)
				local offset = {}
				offset.x, offset.y = self:GetShadowOffset()

				return offset
			end,

			["Type"] = IGAS:GetPropertyType("Offset"),
		},
		--- Spacing
		["Spacing"] = { 
			["Set"] = function(self, spacing)
				self:SetSpacing(spacing)
			end,

			["Get"] = function(self)
				return self:GetSpacing()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		}, 
		--- TextColor
		["TextColor"] = { 
			["Set"] = function(self, color)
				self:SetTextColor(color.red, color,green, color.blue, color.alpha)
			end,

			["Get"] = function(self)
				local color = {}
				color.red, color,green, color.blue, color.alpha = self:GetTextColor()

				return color
			end,

			["Type"] = IGAS:GetPropertyType("Color"),
		},		
		--- NonSpaceWrap
		["NonSpaceWrap"] = { 
			["Set"] = function(self, flag)
				self.__UI:SetNonSpaceWrap(flag)
			end,

			["Get"] = function(self)
				return (self.__UI:CanNonSpaceWrap() and true) or false
			end,

			["Type"] = IGAS:GetPropertyType("Boolean"),
		}, 
		--- StringHeight
		["StringHeight"] = {			
			["Set"] = function(self, height)
				self.__UI:SetTextHeight(height)
			end,

			["Get"] = function(self)
				return self.__UI:GetStringHeight()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		},
		--- Text
		["Text"] = {			
			["Set"] = function(self, text)
				self:SetText(text)
			end,

			["Get"] = function(self)
				return self:GetText()
			end,

			["Type"] = IGAS:GetPropertyType("String"),
		},
	}

    local _New = function(_parent, layer, inheritsFrom)
		if _parent and _parent["CreateFontString"] then
			return IGAS:GetUI(_parent):CreateFontString(nil, layer or "OVERLAY", inheritsFrom or "GameFontNormal")
		end
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end																 