﻿-- Author      : Kurapica
-- Create Date : 6/12/2008 1:14:03 AM

-- Frame
do
	-- Check Version
	local version = 5
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Frame", version) then
		return
	end

    local CreateFrame = CreateFrame
	local pcall = pcall
	local tinsert = tinsert
	local tremove = tremove
    
	local _WidgetName = "Frame"
    local _Base = "Region"
	
	-- Event Checker
	local _EventChkr = CreateFrame("Frame", nil, UIParent)
	_EventChkr:RegisterAllEvents()
		
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- Dispose, release resource
        ["Dispose"] = function(self)
            self:UnregisterAllEvents()
            
            -- Call super's dispose
			self:CallMethod(_Base, "Dispose")
        end,
        
		-- Frame:CreateFontString(["name"[,"layer"[,"inheritsFrom"]]]) - Create and return a new FontString as a child of this Frame - Can instantiate virtuals in 1.11. 
        ["CreateFontString"] = function(self, name, layer, inheritsFrom)
			return IGAS:NewFrame("FontString", name, self, layer, inheritsFrom)
        end,
		
	    -- Frame:CreateTexture(["name"[,"layer"]][,"inheritsFrom"]) - Create and return a new Texture as a child of this Frame. Good for solid colors. - Can instantiate virtuals in 1.11. 
        ["CreateTexture"] = function(self, name, layer, inheritsFrom)
			return IGAS:NewFrame("Texture", name, self, layer, inheritsFrom)
        end,
		
	    -- Frame:CreateTitleRegion() - Create a title region for the frame if it does not have one. - New in 1.11 
        ["CreateTitleRegion"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:CreateTitleRegion(...))
        end,
		
	    -- Frame:DisableDrawLayer("layer") - Disable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
        ["DisableDrawLayer"] = function(self, ...)
            return self.__UI:DisableDrawLayer(...)
        end,
		
	    -- Frame:EnableDrawLayer("layer") - Enable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
        ["EnableDrawLayer"] = function(self, ...)
            return self.__UI:EnableDrawLayer(...)
        end,
		
	    -- Frame:EnableKeyboard(enableFlag) - Set whether this frame will get keyboard input. 
        ["EnableKeyboard"] = function(self, ...)
            return self.__UI:EnableKeyboard(...)
        end,
		
	    -- Frame:EnableMouse(enableFlag) - Set whether this frame will get mouse input. 
        ["EnableMouse"] = function(self, ...)
            return self.__UI:EnableMouse(...)
        end,
		
	    -- Frame:EnableMouseWheel(enableFlag) - Set whether this frame will get mouse wheel notifications. 
        ["EnableMouseWheel"] = function(self, ...)
            return self.__UI:EnableMouseWheel(...)
        end,
		
	    -- Frame:GetAttribute(prefix, name, suffix) - Returns the first existing attribute of (prefix..name..suffix), ("*"..name..suffix), (prefix..name.."*"), ("*"..name.."*"), (name). 
        ["GetAttribute"] = function(self, ...)
            return self.__UI:GetAttribute(...)
        end,
		
	    -- Frame:GetBackdrop() - Creates and returns a backdrop table suitable for use in SetBackdrop - New in 1.11. 
        ["GetBackdrop"] = function(self, ...)
            return self.__UI:GetBackdrop(...)
        end,
		
	    -- Frame:GetBackdropBorderColor() - Gets the frame's backdrop border color (r, g, b, a)- New in 1.11. 
        ["GetBackdropBorderColor"] = function(self, ...)
            return self.__UI:GetBackdropBorderColor(...)
        end,
		
	    -- Frame:GetBackdropColor() - Gets the frame's backdrop color (r, g, b, a)- New in 1.11. 
        ["GetBackdropColor"] = function(self, ...)
            return self.__UI:GetBackdropColor(...)
        end,
		
	    -- Frame:GetChildren() - Get the list of "children" (frames and things derived from frames) of this frame. 
        ["GetChildren"] = function(self, ...)
            return self:GetChilds()
        end,
		
	    -- Frame:GetClampRectInsets() - Gets the modifiers to the frame's rectangle used for clamping the frame to screen. 
        ["GetClampRectInsets"] = function(self, ...)
            return self.__UI:GetClampRectInsets(...)
        end,
		
	    -- Frame:GetDepth() - New in 3.0.8 
        ["GetDepth"] = function(self, ...)
            return self.__UI:GetDepth(...)
        end,
		
	    -- Frame:GetEffectiveAlpha() - Returns the effective alpha of a frame. - Since 2.1. 
        ["GetEffectiveAlpha"] = function(self, ...)
            return self.__UI:GetEffectiveAlpha(...)
        end,
		
	    -- Frame:GetEffectiveDepth() - New in 3.0.8 
        ["GetEffectiveDepth"] = function(self, ...)
            return self.__UI:GetEffectiveDepth(...)
        end,
		
	    -- Frame:GetEffectiveScale() - Get the scale factor of this object relative to the root window. 
        ["GetEffectiveScale"] = function(self, ...)
            return self.__UI:GetEffectiveScale(...)
        end,
		
	    -- Frame:GetFrameLevel() - Get the level of this frame. 
        ["GetFrameLevel"] = function(self, ...)
            return self.__UI:GetFrameLevel(...)
        end,
		
	    -- Frame:GetFrameStrata() - Get the strata of this frame. 
        ["GetFrameStrata"] = function(self, ...)
            return self.__UI:GetFrameStrata(...)
        end,
		
	    -- Frame:GetFrameType() - Get the type of this frame. 
        ["GetFrameType"] = function(self, ...)
            return self:GetObjectType().WidgetName
        end,
		
	    -- Frame:GetHitRectInsets() - Gets the frame's hit rectangle inset distances (l, r, t, b) - new in 1.11. 
        ["GetHitRectInsets"] = function(self, ...)
            return self.__UI:GetHitRectInsets(...)
        end,
		
	    -- Frame:GetID() - Get the ID of this frame. 
        ["GetID"] = function(self, ...)
            return self.__UI:GetID(...)
        end,
		
	    -- Frame:GetMaxResize() - Gets the frame's maximum allowed resize bounds (w, h) - new in 1.11. 
        ["GetMaxResize"] = function(self, ...)
            return self.__UI:GetMaxResize(...)
        end,
		
	    -- Frame:GetMinResize() - Gets the frame's minimum allowed resize bounds (w, h) - new in 1.11. 
        ["GetMinResize"] = function(self, ...)
            return self.__UI:GetMinResize(...)
        end,
		
	    -- Frame:GetNumChildren() - Get the number of "children" (frames and things derived from frames) this frame has. 
        ["GetNumChildren"] = function(self, ...)
			local cnt = 0
			for i in pairs(self.__Childs) do
				cnt = cnt + 1
			end
			return cnt
            --return self.__UI:GetNumChildren(...)
        end,
		
	    -- Frame:GetNumRegions() - Return the number of "regions" (fontstrings, textures) belonging to this frame. 
        ["GetNumRegions"] = function(self, ...)
            return self.__UI:GetNumRegions(...)
        end,
		
	    -- Frame:GetRegions() - Return the "regions" (fontstrings, textures) of the frame (multiple return values) belonging to this frame. 
        --["GetRegions"] = function(self, ...)
        --    return self.__UI:GetRegions(...)
        --end,
		
	    -- Frame:GetScale() - Get the scale factor of this object relative to its parent. 
        ["GetScale"] = function(self, ...)
            return self.__UI:GetScale(...)
        end,
		
	    -- Frame:GetTitleRegion() - Return the frame's title region - New in 1.11. 
        ["GetTitleRegion"] = function(self, ...)
            return IGAS:GetWrapper(self.__UI:GetTitleRegion(...))
        end,
		
	    -- Frame:HookScript("handler", function) - Hook a secure frame script. - Since 2.1. 
        ["HookScript"] = function(self, ...)
			error("Please use StackScript instead of HookScript", 2)
            --return self.__UI:HookScript(...)
        end,
		
	    -- Frame:IgnoreDepth(ignoreFlag) - New in 3.0.8 
        ["IgnoreDepth"] = function(self, ...)
            return self.__UI:IgnoreDepth(...)
        end,
		
	    -- Frame:IsClampedToScreen() - Gets whether the frame is prohibited from being dragged off screen - New in 1.11. 
        ["IsClampedToScreen"] = function(self, ...)
            return self.__UI:IsClampedToScreen(...)
        end,
		
	    -- Frame:IsFrameType("type") - Determine if this frame is of the specified type, or a subclass of that type. 
        ["IsFrameType"] = function(self, ...)
            return self:IsObjectType(...)
        end,
		
	    -- Frame:IsIgnoringDepth() - New in 3.0.8 
        ["IsIgnoringDepth"] = function(self, ...)
            return self.__UI:IsIgnoringDepth(...)
        end,
		
	    -- Frame:IsKeyboardEnabled() - Get whether this frame will get keyboard input. - New in 1.11. 
        ["IsKeyboardEnabled"] = function(self, ...)
            return self.__UI:IsKeyboardEnabled(...)
        end,
		
	    -- Frame:IsMouseEnabled() - Get whether this frame will get mouse input. - New in 1.11. 
        ["IsMouseEnabled"] = function(self, ...)
            return self.__UI:IsMouseEnabled(...)
        end,
		
	    -- Frame:IsMouseWheelEnabled() - Get whether this frame will get mouse wheel notifications. New in 1.11. 
        ["IsMouseWheelEnabled"] = function(self, ...)
            return self.__UI:IsMouseWheelEnabled(...)
        end,
		
	    -- Frame:IsMovable() - Determine if the frame can be moved. 
        ["IsMovable"] = function(self, ...)
            return self.__UI:IsMovable(...)
        end,
		
	    -- Frame:IsResizable() - Determine if the frame can be resized. 
        ["IsResizable"] = function(self, ...)
            return self.__UI:IsResizable(...)
        end,
		
	    -- Frame:IsToplevel() - Get whether the frame is set as toplevel - New in 1.10.2. 
        ["IsToplevel"] = function(self, ...)
            return self.__UI:IsToplevel(...)
        end,
		
	    -- Frame:Lower() - Lower this frame behind other frames. 
        ["Lower"] = function(self, ...)
            return self.__UI:Lower(...)
        end,
		
	    -- Frame:Raise() - Raise this frame above other frames. 
        ["Raise"] = function(self, ...)
            return self.__UI:Raise(...)
        end,
				
	    -- Frame:RegisterForDrag("buttonType"[,"buttonType"...]) - Inidicate that this frame should be notified of drag events for the specified buttons. 
        ["RegisterForDrag"] = function(self, ...)
            return self.__UI:RegisterForDrag(...)
        end,
		
	    -- Frame:SetBackdrop([backdropTable]) - Set the backdrop of the frame according to the specification provided. 
		["SetBackdrop"] = function(self, backdropTable)
			return self.__UI:SetBackdrop(backdropTable or nil)
		end,
		
	    -- Frame:SetBackdropBorderColor(r, g, b[, a]) - Set the frame's backdrop's border's color. 
        ["SetBackdropBorderColor"] = function(self, ...)
            return self.__UI:SetBackdropBorderColor(...)
        end,
		
	    -- Frame:SetBackdropColor(r, g, b[, a]) - Set the frame's backdrop color. 
        ["SetBackdropColor"] = function(self, ...)
            return self.__UI:SetBackdropColor(...)
        end,
		
	    -- Frame:SetClampedToScreen(clamped) - Set whether the frame is prohibited from being dragged off screen - New in 1.11. 
        ["SetClampedToScreen"] = function(self, ...)
            return self.__UI:SetClampedToScreen(...)
        end,
		
	    -- Frame:SetClampRectInsets(left, right, top, bottom) - Modify the frame's rectangle used to prevent dragging offscreen. 
        ["SetClampRectInsets"] = function(self, ...)
            return self.__UI:SetClampRectInsets(...)
        end,
		
	    -- Frame:SetDepth(depth) - New in 3.0.8 
        ["SetDepth"] = function(self, ...)
            return self.__UI:SetDepth(...)
        end,
		
	    -- Frame:SetFrameLevel(level) - Set the level of this frame (determines which of overlapping frames shows on top). 
        ["SetFrameLevel"] = function(self, ...)
            return self.__UI:SetFrameLevel(...)
        end,
		
	    -- Frame:SetFrameStrata("strata") - Set the strata of this frame. 
        ["SetFrameStrata"] = function(self, ...)
            return self.__UI:SetFrameStrata(...)
        end,
		
	    -- Frame:SetHitRectInsets(left, right, top, bottom) - Set the inset distances for the frame's hit rectangle - New in 1.11. 
        ["SetHitRectInsets"] = function(self, ...)
            return self.__UI:SetHitRectInsets(...)
        end,
		
	    -- Frame:SetID(id) - Set the ID of this frame. 
        ["SetID"] = function(self, ...)
            return self.__UI:SetID(...)
        end,
		
	    -- Frame:SetMaxResize(maxWidth, maxHeight) - Set the maximum dimensions this frame can be resized to. 
        ["SetMaxResize"] = function(self, ...)
            return self.__UI:SetMaxResize(...)
        end,
		
	    -- Frame:SetMinResize(minWidth, minHeight) - Set the minimum dimensions this frame can be resized to. 
        ["SetMinResize"] = function(self, ...)
            return self.__UI:SetMinResize(...)
        end,
		
	    -- Frame:SetMovable(isMovable) - Set whether the frame can be moved. 
        ["SetMovable"] = function(self, ...)
            return self.__UI:SetMovable(...)
        end,
		
	    -- Frame:SetResizable(isResizable) - Set whether the frame can be resized. 
        ["SetResizable"] = function(self, ...)
            return self.__UI:SetResizable(...)
        end,
		
	    -- Frame:SetScale(scale) - Set the scale factor of this frame relative to its parent. 
        ["SetScale"] = function(self, ...)
            return self.__UI:SetScale(...)
        end,
				
	    -- Frame:SetToplevel(isTopLevel) - Set whether the frame should raise itself when clicked - New in 1.10.2. 
        ["SetToplevel"] = function(self, ...)
            return self.__UI:SetToplevel(...)
        end,
		
	    -- Frame:SetUserPlaced(isUserPlaced) - Set whether the frame has been relocated by the user (and will thus be saved in the layout cache). 
        ["SetUserPlaced"] = function(self, ...)
            return self.__UI:SetUserPlaced(...)
        end,
		
	    -- Frame:IsUserPlaced() - Determine if this frame has been relocated by the user. 
        ["IsUserPlaced"] = function(self, ...)
            return self.__UI:IsUserPlaced(...)
        end,		
		
	    -- Frame:StartMoving() - Start moving this frame. 
        ["StartMoving"] = function(self, ...)
            return self.__UI:StartMoving(...)
        end,
		
	    -- Frame:StartSizing("point") - Start sizing this frame using the specified anchor point. 
        ["StartSizing"] = function(self, ...)
            return self.__UI:StartSizing(...)
        end,
		
	    -- Frame:StopMovingOrSizing() - Stop moving and/or sizing this frame. 
        ["StopMovingOrSizing"] = function(self, ...)
            return self.__UI:StopMovingOrSizing(...)
        end,
		
		-- IsSystemEvent
		["IsSystemEvent"] = function(self, event)
			if not event or type(event) ~= "string" then
				return false
			end
			
			return (_EventChkr:IsEventRegistered(event) and true) or false
		end,
		
	    -- Frame:IsEventRegistered("event") - Returns true if the given event is registered to the frame. - New in 2.3 
        ["IsEventRegistered"] = function(self, event)
			if self:IsSystemEvent(event) then
				return self.__UI:IsEventRegistered(event)
			else
				-- Custom Event
				return IGAS:IsCustomEventRegistered(self, event)
			end
        end,
		
	    -- Frame:RegisterAllEvents() - Register this frame to receive all events (For debugging purposes only!) - New in 1.11. 
        ["RegisterAllEvents"] = function(self, ...)
            return self.__UI:RegisterAllEvents(...)
        end,
		
	    -- Frame:RegisterEvent("event") - Indicate that this frame should be notified when event occurs. 
        ["RegisterEvent"] = function(self, event)			
			if self.InDesignMode then
				self.__RegisterEventList = self.__RegisterEventList or {}
				self.__RegisterEventList[event] = true
				return
			end
			if self:IsSystemEvent(event) then
				return self.__UI:RegisterEvent(event)
			else
				return IGAS:RegisterCustomEvent(self, event)
			end
        end,
		
	    -- Frame:UnregisterAllEvents() - Indicate that this frame should no longer be notified when any events occur. 
        ["UnregisterAllEvents"] = function(self)
			self.__UI:UnregisterAllEvents()
            return IGAS:UnregisterAllCustomEvents(self)
        end,
		
	    -- Frame:UnregisterEvent("event") - Indicate that this frame should no longer be notified when event occurs. 
        ["UnregisterEvent"] = function(self, event)		
			if self.InDesignMode then
				self.__RegisterEventList = self.__RegisterEventList or {}
				self.__RegisterEventList[event] = nil
			end
			if self:IsSystemEvent(event) then
				return self.__UI:UnregisterEvent(event)
			else
				return IGAS:UnregisterCustomEvent(self, event)
			end
        end,
		
		-- FireEvent, can only fire custom event, fire system event will do nothing
		["FireEvent"] = function(self, event, ...)
			return IGAS:FireCustomEvent(self, event, ...)
		end,
	}
	
	-- Property
	local _Property = {	
		-- Frame		
		["KeyboardEnabled"] = {
			["Set"] = function(self, enabled)
				if not self.InDesignMode then
					self.__UI:EnableKeyboard(enabled)
				else
					self.__KeyboardEnabled = (enabled and true) or false
				end
			end,
			
			["Get"] = function(self)
				if not self.InDesignMode then 
					return (self.__UI:IsKeyboardEnabled() and true) or false
				else
					return (self.__KeyboardEnabled and true) or false
				end
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		["MouseEnabled"] = {
			["Set"] = function(self, enabled)
				if not self.InDesignMode then
					self.__UI:EnableMouse(enabled)
				else
					self.__MouseEnabled = (enabled and true) or false
				end
			end,
			
			["Get"] = function(self)
				if not self.InDesignMode then
					return (self.__UI:IsMouseEnabled() and true) or false
				else
					return (self.__MouseEnabled and true) or false
				end
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		["MouseWheelEnabled"] = {
			["Set"] = function(self, enabled)
				self.__UI:EnableMouseWheel(enabled)
			end,
			
			["Get"] = function(self)
				return (self.__UI:IsMouseWheelEnabled() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},

		["Backdrop"] = {
			["Set"] = function(self, backdropTable)
				self:SetBackdrop(backdropTable)
			end,
			
			["Get"] = function(self)
				return self.__UI:GetBackdrop()
			end,
			
			["Type"] = IGAS:GetPropertyType("Backdrop"),
		},

		["BackdropBorderColor"] = {
			["Set"] = function(self, colorTable)
				self.__UI:SetBackdropBorderColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetBackdropBorderColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		["BackdropColor"] = {
			["Set"] = function(self, colorTable)
				self.__UI:SetBackdropColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetBackdropColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		    
		["ClampedToScreen"] = {
			["Set"] = function(self, enabled)
				self.__UI:SetClampedToScreen(enabled)
			end,
			
			["Get"] = function(self)
				return (self.__UI:IsClampedToScreen() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		     
		["ClampRectInsets"] = {
			["Set"] = function(self, RectInset)
				self.__UI:SetClampRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,
			
			["Get"] = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self.__UI:GetClampRectInsets()
				return RectInset 
			end,
			
			["Type"] = IGAS:GetPropertyType("RectInset"),
		},
		                
		["FrameLevel"] = {
			["Set"] = function(self, level)
				self.__UI:SetFrameLevel(level)
			end,
			
			["Get"] = function(self)
				return self.__UI:GetFrameLevel()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		["FrameStrata"] = {
			["Set"] = function(self, strata)
				self.__UI:SetFrameStrata(strata) 
			end,
			
			["Get"] = function(self)
				return self.__UI:GetFrameStrata()
			end,
			
			["Type"] = IGAS:GetPropertyType("FrameStrata"),
		},
		
		["HitRectInsets"] = {
			["Set"] = function(self, RectInset)
				self.__UI:SetHitRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,
			
			["Get"] = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self.__UI:GetHitRectInsets()
				return RectInset 
			end,
			
			["Type"] = IGAS:GetPropertyType("RectInset"),
		},
		                
		["ID"] = {
			["Set"] = function(self, id)
				self.__UI:SetID(id) 
			end,
			
			["Get"] = function(self)
				return self.__UI:GetID()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		["MaxResize"] = {
			["Set"] = function(self, Size)
				self.__UI:SetMaxResize(Size.width, Size.height)
			end,
			
			["Get"] = function(self)
				local Size = {}
				Size.width, Size.height = self.__UI:GetMaxResize()
				return Size 
			end,
			
			["Type"] = IGAS:GetPropertyType("Size"),
		},
		                
		["MinResize"] = {
			["Set"] = function(self, Size)
				self.__UI:SetMinResize(Size.width, Size.height)
			end,
			
			["Get"] = function(self)
				local Size = {}
				Size.width, Size.height = self.__UI:GetMinResize()
				return Size 
			end,
			
			["Type"] = IGAS:GetPropertyType("Size"),
		},
		                   
		["Scale"] = {
			["Set"] = function(self, scale)
				self.__UI:SetScale(scale) 
			end,
			
			["Get"] = function(self)
				return self.__UI:GetScale()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		        
		["Movable"] = {
			["Set"] = function(self, enabled)
				if not self.InDesignMode then
					self.__UI:SetMovable(enabled)
				else
					self.__Movable = (enabled and true) or false
				end
			end,
			
			["Get"] = function(self)
				if not self.InDesignMode then
					return (self.__UI:IsMovable() and true) or false
				else
					return (self.__Movable and true) or false
				end
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		          
		["Resizable"] = {
			["Set"] = function(self, enabled)
				if not self.InDesignMode then
					self.__UI:SetResizable(enabled)
				else
					self.__Resizable = (enabled and true) or false
				end
			end,
			
			["Get"] = function(self)
				if not self.InDesignMode then
					return (self.__UI:IsResizable() and true) or false
				else
					return (self.__Resizable and true) or false
				end
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		          
		["Toplevel"] = {
			["Set"] = function(self, enabled)
				self.__UI:SetToplevel(enabled)
			end,
			
			["Get"] = function(self)
				return (self.__UI:IsToplevel() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		["Depth"] = {
			["Set"] = function(self, depth)
				self.__UI:SetDepth(depth) 
			end,
			
			["Get"] = function(self)
				return self.__UI:GetDepth()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}
	    
    -- New      
    local _New = function(_parent,inheritFrame)
        -- New Frame
		return CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end