﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 11:55
     
-- GameTooltip
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.GameTooltip", version) then
		return
	end

	local CreateFrame = CreateFrame
	
	local _WidgetName = "GameTooltip"
	local _Base = "Frame"
	local _G = _G
	
	-- Scripts
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 5, right = 5, top = 5, bottom = 5}
	}
	-- ScriptType
	local _ScriptType = {
	}
	
	-- FuncProxy
	local _FuncProxy = {
		-- GameTooltip:AddDoubleLine(textL, textR, rL, gL, bL, rR, gR, bR) 
		["AddDoubleLine"] = function(self, ...)
			return self.__UI:AddDoubleLine(...)
		end,
		-- GameTooltip:AddFontStrings(leftstring, rightstring) - Dynamically expands the size of a tooltip - New in 1.11. 
		["AddFontStrings"] = function(self, leftstring, rightstring, ...)
			return self.__UI:AddFontStrings(IGAS:GetUI(leftstring), IGAS:GetUI(rightstring),...)
		end,
		-- GameTooltip:AddLine("tooltipText" [, textColor.r, textColor.g, textColor.b [, wrapText]]) - Appends the new line to the tooltip. 
		["AddLine"] = function(self, ...)
			return self.__UI:AddLine(...)
		end,
		-- GameTooltip:AddTexture("texture") - Add a texture to the last line added. 
		["AddTexture"] = function(self, texture, ...)
			return self.__UI:AddTexture(IGAS:GetUI(texture), ...)
		end,
		-- GameTooltip:AppendText("text") - Append text to the end of the first line of the tooltip. 
		["AppendText"] = function(self, ...)
			return self.__UI:AppendText(...)
		end,
		-- GameTooltip:ClearLines() - Clear all lines of tooltip (both left and right ones) 
		["ClearLines"] = function(self, ...)
			return self.__UI:ClearLines(...)
		end,
		-- GameTooltip:FadeOut 
		["FadeOut"] = function(self, ...)
			return self.__UI:FadeOut(...)
		end,
		-- GameTooltip:GetAnchorType() - Returns the current anchoring type. 
		["GetAnchorType"] = function(self, ...)
			return self.__UI:GetAnchorType(...)
		end,
		-- GameTooltip:GetItem() - Returns name, link. 
		["GetItem"] = function(self, ...)
			return self.__UI:GetItem(...)
		end,
		-- GameTooltip:GetMinimumWidth() - 
		["GetMinimumWidth"] = function(self, ...)
			return self.__UI:GetMinimumWidth(...)
		end,

		-- GameTooltip:GetSpell() - Returns name, rank. 
		["GetSpell"] = function(self, ...)
			return self.__UI:GetSpell(...)
		end,
		-- GameTooltip:GetOwner() - Returns owner frame, anchor. 
		["GetOwner"] = function(self, ...)
			return IGAS:GetWrapper(self.__UI:GetOwner(...))
		end,
		-- GameTooltip:GetUnit() - Returns unit name, unit id. 
		["GetUnit"] = function(self, ...)
			return self.__UI:GetUnit(...)
		end,
		-- GameTooltip:IsUnit("unit") - Returns bool. 
		["IsUnit"] = function(self, ...)
			return self.__UI:IsUnit(...)
		end,
		-- GameTooltip:NumLines() - Get the number of lines in the tooltip. 
		["NumLines"] = function(self, ...)
			return self.__UI:NumLines(...)
		end,
		-- GameTooltip:SetAction(slot) - Shows the tooltip for the specified action button. 
		["SetAction"] = function(self, ...)
			return self.__UI:SetAction(...)
		end,
		-- GameTooltip:SetAuctionCompareItem("type", index[, offset]) 
		["SetAuctionCompareItem"] = function(self, ...)
			return self.__UI:SetAuctionCompareItem(...)
		end,
		-- GameTooltip:SetAuctionItem("type", index) - Shows the tooltip for the specified auction item. 
		["SetAuctionItem"] = function(self, ...)
			return self.__UI:SetAuctionItem(...)
		end,
		-- GameTooltip:SetAuctionSellItem 
		["SetAuctionSellItem"] = function(self, ...)
			return self.__UI:SetAuctionSellItem(...)
		end,
		-- GameTooltip:SetBackpackToken(id) - 
		["SetBackpackToken"] = function(self, ...)
			return self.__UI:SetBackpackToken(...)
		end,

		-- GameTooltip:SetBagItem(bag, slot) 
		["SetBagItem"] = function(self, ...)
			return self.__UI:SetBagItem(...)
		end,
		-- GameTooltip:SetBuybackItem 
		["SetBuybackItem"] = function(self, ...)
			return self.__UI:SetBuybackItem(...)
		end,
		-- GameTooltip:SetCurrencyToken(tokenId) - Shows the tooltip for the specified token 
		["SetCurrencyToken"] = function(self, ...)
			return self.__UI:SetCurrencyToken(...)
		end,
		-- GameTooltip:SetGlyph(id) - 
		["SetGlyph"] = function(self, ...)
			return self.__UI:SetGlyph(...)
		end,

		-- GameTooltip:SetGuildBankItem(tab, id) - Shows the tooltip for the specified guild bank item 
		["SetGuildBankItem"] = function(self, ...)
			return self.__UI:SetGuildBankItem(...)
		end,
		-- GameTooltip:SetHyperlink("itemString" or "itemLink") - Changes the item which is displayed in the tooltip according to the passed argument. 
		["SetHyperlink"] = function(self, ...)
			return self.__UI:SetHyperlink(...)
		end,
		-- GameTooltip:SetHyperlinkCompareItem("itemLink", index) - Sets a comparison tooltip to show the index'th comparison item to the item specified as link. Will return a true value if there is an index'th comparison item (index is 1 through 3) 
		["SetHyperlinkCompareItem"] = function(self, ...)
			return self.__UI:SetHyperlinkCompareItem(...)
		end,
		-- GameTooltip:SetInboxItem(index) - Shows the tooltip for the specified mail inbox item. 
		["SetInboxItem"] = function(self, ...)
			return self.__UI:SetInboxItem(...)
		end,
		-- GameTooltip:SetInventoryItem(unit, slot[, nameOnly]) 
		["SetInventoryItem"] = function(self, ...)
			return self.__UI:SetInventoryItem(...)
		end,
		-- GameTooltip:SetLootItem 
		["SetLootItem"] = function(self, ...)
			return self.__UI:SetLootItem(...)
		end,
		-- GameTooltip:SetLootRollItem(id) - Shows the tooltip for the specified loot roll item. 
		["SetLootRollItem"] = function(self, ...)
			return self.__UI:SetLootRollItem(...)
		end,
		-- GameTooltip:SetMerchantCompareItem("slot"[, offset]) 
		["SetMerchantCompareItem"] = function(self, ...)
			return self.__UI:SetMerchantCompareItem(...)
		end,
		-- GameTooltip:SetMerchantItem 
		["SetMerchantItem"] = function(self, ...)
			return self.__UI:SetMerchantItem(...)
		end,
		-- GameTooltip:SetMinimumWidth(width) - (Formerly SetMoneyWidth) 
		["SetMinimumWidth"] = function(self, ...)
			return self.__UI:SetMinimumWidth(...)
		end,
		-- GameTooltip:SetOwner(owner, "anchor"[, +x, +y]) 
		["SetOwner"] = function(self, owner, ...)
			return self.__UI:SetOwner(IGAS:GetUI(owner), ...)
		end,
		-- GameTooltip:SetPadding 
		["SetPadding"] = function(self, ...)
			return self.__UI:SetPadding(...)
		end,
		-- GameTooltip:SetPetAction(slot) - Shows the tooltip for the specified pet action. 
		["SetPetAction"] = function(self, ...)
			return self.__UI:SetPetAction(...)
		end,
		-- GameTooltip:SetQuestItem 
		["SetQuestItem"] = function(self, ...)
			return self.__UI:SetQuestItem(...)
		end,
		-- GameTooltip:SetQuestLogItem 
		["SetQuestLogItem"] = function(self, ...)
			return self.__UI:SetQuestLogItem(...)
		end,
		-- GameTooltip:SetQuestLogRewardSpell - Shows the tooltip for the spell reward of the currently selected quest. 
		["SetQuestLogRewardSpell"] = function(self, ...)
			return self.__UI:SetQuestLogRewardSpell(...)
		end,
		-- GameTooltip:SetQuestRewardSpell 
		["SetQuestRewardSpell"] = function(self, ...)
			return self.__UI:SetQuestRewardSpell(...)
		end,
		-- GameTooltip:SetSendMailItem 
		["SetSendMailItem"] = function(self, ...)
			return self.__UI:SetSendMailItem(...)
		end,
		-- GameTooltip:SetShapeshift(slot) - Shows the tooltip for the specified shapeshift form. 
		["SetShapeshift"] = function(self, ...)
			return self.__UI:SetShapeshift(...)
		end,
		-- GameTooltip:SetSpell(spellId, bookType) - Shows the tooltip for the specified spell. 
		["SetSpell"] = function(self, ...)
			return self.__UI:SetSpell(...)
		end,
		-- GameTooltip:SetTalent(tabIndex, talentIndex) - Shows the tooltip for the specified talent. 
		["SetTalent"] = function(self, ...)
			return self.__UI:SetTalent(...)
		end,
		-- GameTooltip:SetText("text", r, g, b[, alphaValue[, textWrap]]) - Set the text of the tooltip. 
		["SetText"] = function(self, ...)
			return self.__UI:SetText(...)
		end,
		-- GameTooltip:SetTracking 
		["SetTracking"] = function(self, ...)
			return self.__UI:SetTracking(...)
		end,
		-- GameTooltip:SetTradePlayerItem 
		["SetTradePlayerItem"] = function(self, ...)
			return self.__UI:SetTradePlayerItem(...)
		end,
		-- GameTooltip:SetTradeSkillItem 
		["SetTradeSkillItem"] = function(self, ...)
			return self.__UI:SetTradeSkillItem(...)
		end,
		-- GameTooltip:SetTradeTargetItem 
		["SetTradeTargetItem"] = function(self, ...)
			return self.__UI:SetTradeTargetItem(...)
		end,
		-- GameTooltip:SetTrainerService 
		["SetTrainerService"] = function(self, ...)
			return self.__UI:SetTrainerService(...)
		end,
		-- GameTooltip:SetUnit 
		["SetUnit"] = function(self, ...)
			return self.__UI:SetUnit(...)
		end,
		-- GameTooltip:SetUnitAura("unitId", auraIndex[, filter]) - Shows the tooltip for a unit's aura. (Exclusive to 3.x.x / WotLK) 
		["SetUnitAura"] = function(self, ...)
			return self.__UI:SetUnitAura(...)
		end,
		-- GameTooltip:SetUnitBuff("unitId", buffIndex[, raidFilter]) - Shows the tooltip for a unit's buff. 
		["SetUnitBuff"] = function(self, ...)
			return self.__UI:SetUnitBuff(...)
		end,
		-- GameTooltip:SetUnitDebuff("unitId", buffIndex[, raidFilter]) - Shows the tooltip for a unit's debuff. 
		["SetUnitDebuff"] = function(self, ...)
			return self.__UI:SetUnitDebuff(...)
		end,
		---- Custom Methods
		["GetLeftText"] = function(self, index)
			local name = self.__UI:GetName()
			
			if not name or not index or type(index) ~= "number" then
				return
			end
			
			name = name.."TextLeft"..index
			
			if _G[name] and type(_G[name]) == "table" and _G[name].GetText then
				return _G[name]:GetText()
			end
		end,
		["GetRightText"] = function(self, index)
			local name = self.__UI:GetName()
			
			if not name or not index or type(index) ~= "number" then
				return
			end
			
			name = name.."TextRight"..index
			
			if _G[name] and type(_G[name]) == "table" and _G[name].GetText then
				return _G[name]:GetText()
			end
		end,
		["GetTexture"] = function(self, index)
			local name = self.__UI:GetName()
			
			if not name or not index or type(index) ~= "number" then
				return
			end
			
			name = name.."Texture"..index
			
			if _G[name] and type(_G[name]) == "table" and _G[name].GetTexture then
				return _G[name]:GetTexture()
			end
		end,
	}
	
	-- Property
	local _Property = {
	}
	
	local _Cnt = 1
	
	-- New      
	local _New = function(_parent, inheritsFrom)
		-- New Frame
		frame = CreateFrame("GameTooltip","IGAS_GameTooltip".._Cnt,IGAS:GetUI(_parent), inheritsFrom or "GameTooltipTemplate")
		--[[
		frame:SetFrameStrata("TOOLTIP")
		frame:SetClampedToScreen(true)
		frame:Hide()
		
		frame:SetBackdrop(_FrameBackdrop)
		frame:SetBackdropBorderColor(1, 1, 1)
		frame:SetBackdropColor(0.09, 0.09, 0.19)
		frame:SetOwner(IGAS:GetUI(_parent), "ANCHOR_NONE")

		local lftText = frame:CreateFontString("IGAS_GameTooltip".._Cnt.."_TextLeft1", "ARTWORK", "GameTooltipHeaderText")
		local ritText = frame:CreateFontString("IGAS_GameTooltip".._Cnt.."_TextRight1", "ARTWORK", "GameTooltipHeaderText")
		local texutre = frame:CreateTexture("IGAS_GameTooltip".._Cnt.."_Texture1", "ARTWORK")
		
		lftText:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -10)
		ritText:SetPoint("RIGHT", lftText, "LEFT", 40, 0)
		
		texutre:SetWidth(12)
		texutre:SetHeight(12)

		frame:AddFontStrings(lftText, ritText)
		frame:AddTexture(texutre)
--]]
		_Cnt = _Cnt + 1
		
		return frame
	end
	
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}
	
	IGAS:NewWidget(_WidgetInfo, true, true)
end