﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 12:29
     
-- Model
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Model", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Model"
    local _Base = "Frame"
	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- Model:AdvanceTime() 
		["AdvanceTime"] = function(self, ...)
			return self.__UI:AdvanceTime(...)
		end,
	    -- Model:ClearFog() - Removes all fogging effects currently active in rendering. 
		["ClearFog"] = function(self, ...)
			return self.__UI:ClearFog(...)
		end,
	    -- Model:ClearModel() - Removes all geometry from the Model (i.e. makes it empty) 
		["ClearModel"] = function(self, ...)
			return self.__UI:ClearModel(...)
		end,
	    -- Model:GetFacing() - Returns the direction the model is facing. 
		["GetFacing"] = function(self, ...)
			return self.__UI:GetFacing(...)
		end,
	    -- Model:GetFogColor() - Gets the fog color (r, g, b, a) - New in 1.11. 
		["GetFogColor"] = function(self, ...)
			return self.__UI:GetFogColor(...)
		end,
	    -- Model:GetFogFar() - Gets the fog far distance - New in 1.11. 
		["GetFogFar"] = function(self, ...)
			return self.__UIGetFogFar(...)
		end,
	    -- Model:GetFogNear() - Gets the fog near distance - New in 1.11. 
		["GetFogNear"] = function(self, ...)
			return self.__UI:GetFogNear(...)
		end,
	    -- Model:GetLight() - Gets the light specification for the model, returns a list of results compatible with the SetLight method - New in 1.11. 
		["GetLight"] = function(self, ...)
			return self.__UI:GetLight(...)
		end,
	    -- Model:GetModel() - Gets the model file for this Model - New in 1.11. 
		["GetModel"] = function(self, ...)
			return self.__UI:GetModel(...)
		end,
	    -- Model:GetModelScale() - Returns the current mesh scaling factor. 
		["GetModelScale"] = function(self, ...)
			return self.__UI:GetModelScale(...)
		end,
	    -- Model:GetPosition() - Returns the current position of the mesh as x, y, z 
		["GetPosition"] = function(self, ...)
			return self.__UI:GetPosition(...)
		end,
	    -- Model:ReplaceIconTexture("texture") 
		["ReplaceIconTexture"] = function(self, texture, ...)
			return self.__UI:ReplaceIconTexture(IGAS:GetUI(texture), ...)
		end,
	    -- Model:SetCamera(index) - Select a pre-defined camera. 
		["SetCamera"] = function(self, ...)
			return self.__UI:SetCamera(...)
		end,
	    -- Model:SetFacing(facing) - Set the direction that the model is facing. 
		["SetFacing"] = function(self, ...)
			return self.__UI:SetFacing(...)
		end,
	    -- Model:SetFogColor(r, g, b[, a]) - Set the fog color and enable fogging. 
		["SetFogColor"] = function(self, ...)
			return self.__UI:SetFogColor(...)
		end,
	    -- Model:SetFogFar(value) - Set the far-clipping plane distance for fogging. 
		["SetFogFar"] = function(self, ...)
			return self.__UI:SetFogFar(...)
		end,
	    -- Model:SetFogNear(value) - Set the near-clipping plane distance for fogging. 
		["SetFogNear"] = function(self, ...)
			return self.__UI:SetFogNear(...)
		end,
	    -- Model:SetGlow(...) - 
		["SetGlow"] = function(self, ...)
			return self.__UI:SetGlow(...)
		end,

	    -- Model:SetLight(enabled[, omni, dirX, dirY, dirZ, ambIntensity[, ambR, ambG, ambB[, dirIntensity[, dirR, dirG, dirB]]]]) - Place the light source used for rendering 
		["SetLight"] = function(self, ...)
			return self.__UI:SetLight(...)
		end,
	    -- Model:SetModel("file") - Set the mesh that is displayed in the frame. 
		["SetModel"] = function(self, ...)
			return self.__UI:SetModel(...)
		end,
	    -- Model:SetModelScale(scale) - Sets the scale factor for the mesh before rendering. 
		["SetModelScale"] = function(self, ...)
			return self.__UI:SetModelScale(...)
		end,
	    -- Model:SetPosition(x, y, z) - Set the position of the mesh inside the frame's coordinate system. 
		["SetPosition"] = function(self, ...)
			return self.__UI:SetPosition(...)
		end,
	    -- Model:SetSequence(sequence) - Set the animation to be played. 
		["SetSequence"] = function(self, ...)
			return self.__UI:SetSequence(...)
		end,
	    -- Model:SetSequenceTime(sequence, time) 
		["SetSequenceTime"] = function(self, ...)
			return self.__UI:SetSequenceTime(...)
		end,
    }
    
	-- Property
	local _Property = {	
    	-- FogColor
		["FogColor"] = {
			["Set"] = function(self, colorTable)
				self.__UI:SetFogColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetFogColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
    
    	-- FogFar
		["FogFar"] = {
			["Set"] = function(self, value)
				self.__UI:SetFogFar(value)
			end,
			
			["Get"] = function(self)
				return self.__UI:GetFogFar() 
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
   
    	-- FogNear
		["FogNear"] = {
			["Set"] = function(self, value)
				self.__UI:SetFogNear(value)
			end,
			
			["Get"] = function(self)
				return self.__UI:GetFogNear() 
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},   
         
    	-- ModelScale
		["ModelScale"] = {
			["Set"] = function(self, scale)
				self.__UI:SetModelScale(scale)
			end,
			
			["Get"] = function(self)
				return self.__UI:GetModelScale() 
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},   
		
    	-- Model
		["Model"] = {
			["Set"] = function(self, file)
				if file and type(file) == "string" and file ~= "" then
					self.__UI:SetModel(file)
				else
					self.__UI:ClearModel()
				end
			end,
			
			["Get"] = function(self)
				return self.__UI:GetModel() 
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
    	-- Position
		["Position"] = {
			["Set"] = function(self, table)
				self.__UI:SetPosition(table.x, table.y, table.z)
			end,
			
			["Get"] = function(self)
				local table = {}
				table.x, table.y, table.z = self.__UI:GetPosition()
				return table 
			end,
			
			["Type"] = IGAS:GetPropertyType("Position"),
		},
		
    	-- Light
		["Light"] = {
			["Set"] = function(self, set)
				self.__UI:SetLight(set.enabled, set.omni, set.dirX, set.dirY, set.dirZ, set.ambIntensity, set.ambR, set.ambG, set.ambB, set.dirIntensity, set.dirR, set.dirG, set.dirB)
			end,
			
			["Get"] = function(self)
				local set = {}
				set.enabled, set.omni, set.dirX, set.dirY, set.dirZ, set.ambIntensity, set.ambR, set.ambG, set.ambB, set.dirIntensity, set.dirR, set.dirG, set.dirB = self.__UI:GetLight()
				return set 
			end,
			
			["Type"] = IGAS:GetPropertyType("Light"),
		},
   }
	
    -- New  
    local _New = function(_parent,inheritFrame)
        -- New Frame
		return CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end