﻿-- Author      : Kurapica
-- Create Date : 6/12/2008 1:13:21 AM

-- Region
do
	-- Check Version
	local version = 3
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Region", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Region"
    local _Base = "UIObject"
    
    -- FuncProxy
	local _FuncProxy = {
        --- Original Functions
        -- ClearAllPoints() - Clear all attachment points for this object. 
        ["ClearAllPoints"] = function(self)
            self.__UI:ClearAllPoints()
        end,
		
        -- Region:CreateAnimationGroup(["name"[,"inheritsFrom"]]) - Create and return a new AnimationGroup as a child of this Region. - New in 3.1.0 
		["CreateAnimationGroup"] = function(self, name, inheritsFrom)
			return IGAS:NewFrame("AnimationGroup", name, self, inheritsFrom)
		end,
		
		-- Region:GetAnimationGroups() - Returns all AnimationGroups that are children of this Region. - New in 3.1.0 
		-- Not include, this method will be added when scan function works.
		--[[["GetAnimationGroups"] = function(self)
			return self.__UI:GetAnimationGroups()
		end,--]]
		
        -- GetBottom() - Get the y location of the bottom edge of this frame
        ["GetBottom"] = function(self)
            return self.__UI:GetBottom()
        end,
        
        -- GetCenter() - Get the coordinates of the center of this frame
        ["GetCenter"] = function(self)
            return self.__UI:GetCenter()
        end,
        
        -- GetHeight() - Get the height of this object. 
        ["GetHeight"] = function(self)
            return self.__UI:GetHeight()
        end,
        
        -- SetHeight(height) - Set the height of the object. 
        ["SetHeight"] = function(self, height)
            self.__UI:SetHeight(height)
        end,
        
        -- GetLeft() - Get the x location of the left edge of this frame
        ["GetLeft"] = function(self)
            return self.__UI:GetLeft()
        end,
        
        -- GetNumPoints() - Get the number of anchor points for this frame
        ["GetNumPoints"] = function(self)
            return self.__UI:GetNumPoints()
        end,
        
        -- GetPoint(pointNum) - Get details for an anchor point for this frame (point, relativeTo, relativePoint, xofs, yofs)
        ["GetPoint"] = function(self, pointNum)
            point, frame, relativePoint, x, y = self.__UI:GetPoint(pointNum)
            frame = IGAS:GetWrapper(frame)
            return point, frame, relativePoint, x, y
        end,
        
        -- GetRight() - Get the x location of the right edge of this frame
        ["GetRight"] = function(self)
            return self.__UI:GetRight()
        end,
        
        -- GetTop() - Get the y location of the top edge of this frame
        ["GetTop"] = function(self)
            return self.__UI:GetTop()
        end,
        
        -- GetWidth() - Get the width of this object. 
        ["GetWidth"] = function(self)
            return self.__UI:GetWidth()
        end,
        
        -- SetWidth(width) - Set the width of the object. 
        ["SetWidth"] = function(self, width)
            self.__UI:SetWidth(width)
        end,
        
        -- Hide() - Set this object to hidden (it and all of its children will disappear). 
        ["Hide"] = function(self)
            return self.__UI:Hide()
        end,
        
        -- Show() - Set this object to shown (it will appear if its parent is visible). 
        ["Show"] = function(self)
            self.__UI:Show()
        end,
        
        -- IsProtected() - Determine if this object can be manipulated in certain ways by tainted code in combat or not 
        ["IsProtected"] = function(self)
            return self.__UI:IsProtected()
        end,
        
        -- IsShown() - Determine if this object is shown (would be visible if its parent was visible). 
        ["IsShown"] = function(self)
            return self.__UI:IsShown()
        end,
        
        -- IsVisible() - Get whether the object is visible on screen (logically (IsShown() and GetParent():IsVisible())); 
        ["IsVisible"] = function(self)
            return self.__UI:IsVisible()
        end,
        
        -- SetAllPoints(frame or "frameName") - Set all anchors to match edges of specified frame
        ["SetAllPoints"] = function(self, frame)            
            if frame and type(frame) == "string" then
                frame = _G[frame]
            end
            
            if not frame or type(frame) ~= "table" then
                frame = self.Parent
            end
            
            frame = IGAS:GetUI(frame)
            
            self.__UI:SetAllPoints(frame)
        end,
        
        -- SetPoint("point","relativeFrame" or relativeObject,"relativePoint"[, xOfs, yOfs]) - Set an attachment point of this object - Updated in 1.10. Since 2.2 cooordinates are now relative to the closest anchor point 
        ["SetPoint"] = function(self, point, relativeObject, relativePoint, xOfs, yOfs)            
            if relativeObject then
                if type(relativeObject) == "string" then
                    relativeObject = _G[relativeObject]
                end
                
                relativeObject = IGAS:GetUI(relativeObject)
                
                self.__UI:SetPoint(point, relativeObject, relativePoint, xOfs, yOfs)
            end
        end,
        
        -- Dispose, release resource
        ["Dispose"] = function(self)
            self:ClearAllPoints()
            self:Hide()
            
            -- Call super's dispose
			self:CallMethod(_Base, "Dispose")
        end,

		--Region:GetRect() - Get frame's left, bottom, width, height. 
		["GetRect"] = function(self)
			return self.__UI:GetRect()
		end,
		
	    --Region:IsDragging() - True if this Region or its Parent is being dragged. - New in 3.1.0 
		["IsDragging"] = function(self)
			return self.__UI:IsDragging()
		end,
		
	    --Region:StopAnimating() - Stops any active animations on the Region and its children. - New in 3.1.0 
		["StopAnimating"] = function(self)
			return self.__UI:StopAnimating()
		end,
    }
    
    -- Property
	local _Property = {
		--- Height
		["Height"] = {
			["Set"] = function(self, height)
				self:SetHeight(height)
			end,

			["Get"] = function(self)
				return self:GetHeight()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		},
		--- Width
		["Width"] = {
			["Set"] = function(self, width)
				self:SetWidth(width)
			end,

			["Get"] = function(self)
				return self:GetWidth()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		},
		--- Visible
		["Visible"] = {
			["Set"] = function(self, visible)
				if not self.InDesignMode then
					if visible then
						self:Show()
					else
						self:Hide()
					end
				else
					self.__Visible = (visible and true) or false
				end
			end,

			["Get"] = function(self)
				if not self.InDesignMode then
					return (self:IsShown() and true ) or false
				else
					return (self.__Visible and true) or false
				end
			end,

			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
    }
		
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
    }
    
    IGAS:NewWidget(_WidgetInfo, true)
end