﻿-- Author      : Kurapica
-- Create Date : 8/18/2009

-- Rotation
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Rotation", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Rotation"
    local _Base = "Animation"
		
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- Rotation:SetDegrees(degrees) - Sets the amount of degrees that the animation's parent Region would rotate. 
		["SetDegrees"] = function(self, degrees)
			return self.__UI:SetDegrees(degrees)
		end,
	    -- Rotation:GetDegrees() - Gets the amount of degrees that the animation's parent Region would rotate. 
		["GetDegrees"] = function(self)
			return self.__UI:GetDegrees()
		end,
	    -- Rotation:SetRadians(radians) - Sets the amount of radians that the animation's parent Region would travel. 
		["SetRadians"] = function(self, radians)
			return self.__UI:SetRadians(radians)
		end,
	    -- Rotation:GetRadians() - Sets the amount of radians that the animation's parent Region would travel. 
		["GetRadians"] = function(self)
			return self.__UI:GetRadians()
		end,
	    -- Rotation:SetOrigin(point, offsetX, offsetY) - Sets the animation's origin of rotation for its parent Region. 
		["SetOrigin"] = function(self, point, offsetX, offsetY))
			return self.__UI:SetOrigin()
		end,
	    -- Rotation:GetOrigin() - Gets the point, X offset, and Y offset of the animation's origin of rotation for its parent Region.
		["GetOrigin"] = function(self)
			return self.__UI:GetOrigin()
		end,
	}
	
	-- Property
	local _Property = {
        -- Degrees
        ["Degrees"] = {
			["Set"] = function(self, degrees)
				self:SetDegrees(degrees)
			end,
			
			["Get"] = function(self)
				return self:GetDegrees()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Radians
        ["Radians"] = {
			["Set"] = function(self, radians)
				self:SetRadians(radians)
			end,
			
			["Get"] = function(self)
				return self:GetRadians()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Origin
        ["Origin"] = {
			["Set"] = function(self, origin)
				if origin and type(origin) == "table" then
					self:SetOrigin(origin.point, origin.x, origin.y)
				end
			end,
			["Get"] = function(self)
				local origin = {}
				origin.point, origin.x, origin.y = self:GetOrigin()
				return origin
			end,
			
			["Type"] = IGAS:GetPropertyType("Origin"),
		},
	}
	    
    -- New      
    local _New = function(_parent,inheritsFrom)
		if _parent and _parent["CreateAnimation"] then
			return IGAS:GetUI(_parent):CreateAnimation(_WidgetName, nil, inheritsFrom)
		end
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
	local _ScanFrameParent = IGAS:NewFrame("AnimationGroup")
	
    IGAS:NewWidget(_WidgetInfo, true, true, _ScanFrameParent)
	
	_ScanFrameParent:Dispose()
end