﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 14:39
     
-- Slider
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Slider", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Slider"
    local _Base = "Frame"
	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
	    -- Slider:Disable() - Disables the slider 
		["Disable"] = function(self, ...)
			return self.__UI:Disable(...)
		end,
	    -- Slider:Enable() - Enables the slider 
		["Enable"] = function(self, ...)
			return self.__UI:Enable(...)
		end,
	    -- Slider:GetMinMaxValues() - Get the current bounds of the slider. 
		["GetMinMaxValues"] = function(self, ...)
			return self.__UI:GetMinMaxValues(...)
		end,
	    -- Slider:GetOrientation() - Returns "HORIZONTAL" or "VERTICAL". 
		["GetOrientation"] = function(self, ...)
			return self.__UI:GetOrientation(...)
		end,
	    -- Slider:GetThumbTexture() - Get the texture for this slider's thumb - New in 1.11. 
		["GetThumbTexture"] = function(self, ...)
			return IGAS:GetWrapper(self.__UI:GetThumbTexture(...))
		end,
	    -- Slider:GetValue() - Get the current value of the slider. 
		["GetValue"] = function(self, ...)
			return self.__UI:GetValue(...)
		end,
	    -- Slider:GetValueStep() - Get the current step size of the slider. 
		["GetValueStep"] = function(self, ...)
			return self.__UI:GetValueStep(...)
		end,
	    -- Slider:IsEnabled() - Returns enabled status of the slider. 
		["IsEnabled"] = function(self, ...)
			return self.__UI:IsEnabled(...)
		end,
	    -- Slider:SetMinMaxValues(min, max) - Set the bounds of the slider. 
		["SetMinMaxValues"] = function(self, ...)
			return self.__UI:SetMinMaxValues(...)
		end,
	    -- Slider:SetOrientation("orientation") - "HORIZONTAL" or "VERTICAL". 
		["SetOrientation"] = function(self, ...)
			return self.__UI:SetOrientation(...)
		end,
	    -- Slider:SetThumbTexture(texture or "texturePath") 
		["SetThumbTexture"] = function(self, texture, ...)
			return self.__UI:SetThumbTexture(IGAS:GetUI(texture), ...)
		end,
	    -- Slider:SetValue(value) - Set the value of the slider. Also causes the thumb to show on the first call. 
		["SetValue"] = function(self, ...)
			return self.__UI:SetValue(...)
		end,
	    -- Slider:SetValueStep(value) - Set the step size of the slider. 
		["SetValueStep"] = function(self, ...)
			return self.__UI:SetValueStep(...)
		end,
    }
    
	-- Property
	local _Property = {		
        -- Orientation
        ["Orientation"] = {
			["Set"] = function(self, orientation)
				self.__UI:SetOrientation(orientation)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetOrientation()
			end,
			
			["Type"] = IGAS:GetPropertyType("Orientation"),
		},
    
        -- ThumbTexture
        ["ThumbTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetThumbTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetThumbTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
		
        -- Value
        ["Value"] = {
			["Set"] = function(self, value)
				self.__UI:SetValue(value)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetValue()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
    
        -- ValueStep
        ["ValueStep"] = {
			["Set"] = function(self, value)
				self.__UI:SetValueStep(value)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetValueStep()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Enabled
        ["Enabled"] = {
			["Set"] = function(self, enabled)
                if enabled then
				    self:Enable()
                else
				    self:Disable()
                end
			end,
			
			["Get"] = function(self)
				return (self:IsEnabled() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
	}
	
    -- New
    local _New = function(_parent,inheritFrame)
        -- New Frame
		return CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end