﻿-- Author      : Kurapica
-- Create Date : 7/16/2008 14:51
     
-- StatusBar
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.StatusBar", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "StatusBar"
    local _Base = "Frame"
	
	-- ScriptType
	local _ScriptType = {
	}
    
	-- FuncProxy
	local _FuncProxy = {
		-- StatusBar:GetMinMaxValues() - Get the current bounds of the bar. 
		["GetMinMaxValues"] = function(self, ...)
			return self.__UI:GetMinMaxValues(...)
		end,
	    -- StatusBar:GetOrientation 
		["GetOrientation"] = function(self, ...)
			return self.__UI:GetOrientation(...)
		end,
	    -- StatusBar:GetStatusBarColor 
		["GetStatusBarColor"] = function(self, ...)
			return self.__UI:GetStatusBarColor(...)
		end,
	    -- StatusBar:GetStatusBarTexture() - Returns the texture object for the bar - Before 1.11 it returned the filename. 
		["GetStatusBarTexture"] = function(self, ...)
			return IGAS:GetWrapper(self.__UI:GetStatusBarTexture(...))
		end,
	    -- StatusBar:GetValue() - Get the current value of the bar. 
		["GetValue"] = function(self, ...)
			return self.__UI:GetValue(...)
		end,
	    -- StatusBar:SetMinMaxValues(min, max) - Set the bounds of the bar. 
		["SetMinMaxValues"] = function(self, ...)
			return self.__UI:SetMinMaxValues(...)
		end,
	    -- StatusBar:SetOrientation("orientation") - "HORIZONTAL" or "VERTICAL". 
		["SetOrientation"] = function(self, ...)
			return self.__UI:SetOrientation(...)
		end,
	    -- StatusBar:SetStatusBarColor(r, g, b[, alpha]) - Set the color of the bar. 
		["SetStatusBarColor"] = function(self, ...)
			return self.__UI:SetStatusBarColor(...)
		end,
	    -- StatusBar:SetStatusBarTexture("file" or texture[,"layer"]) - Sets the texture of the bar - Added texture as valid arg in 1.11. 
		["SetStatusBarTexture"] = function(self, texture, ...)
			return self.__UI:SetStatusBarTexture(IGAS:GetUI(texture), ...)
		end,
	    -- StatusBar:SetValue(value) - Set the value of the bar. 
		["SetValue"] = function(self, ...)
			return self.__UI:SetValue(...)
		end,
    }
    
	-- Property
	local _Property = {
        -- MinMaxValue
        ["MinMaxValue"] = {
			["Set"] = function(self, set)
				self.__UI:SetMinMaxValues(set.min, set.max)
			end,
			
			["Get"] = function(self)
				local set = {}
				set.min, set.max = self.__UI:GetMinMaxValues()
				return set
			end,
			
			["Type"] = IGAS:GetPropertyType("MinMaxValue"),
		},
	
        -- Orientation
        ["Orientation"] = {
			["Set"] = function(self, orientation)
				self.__UI:SetOrientation(orientation)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetOrientation()
			end,
			
			["Type"] = IGAS:GetPropertyType("Orientation"),
		},
	
		-- StatusBarColor
		["StatusBarColor"] = {
			["Set"] = function(self, colorTable)
				self.__UI:SetStatusBarColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetStatusBarColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
    
        -- StatusBarTexture
        ["StatusBarTexture"] = {
			["Set"] = function(self, texture)
				self.__UI:SetStatusBarTexture(texture)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetStatusBarTexture()
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
		
		-- Layer
		["Layer"] = {
			["Set"] = function(self, layer)
				self.__Layer = layer
				self.__UI:SetStatusBarTexture(self.__UI:GetStatusBarTexture(), layer)
			end,
			
			["Get"] = function(self)				
				return self.__Layer
			end,
			
			["Type"] = IGAS:GetPropertyType("DrawLayer"),
		},
		
        -- Value
        ["Value"] = {
			["Set"] = function(self, value)
				self.__UI:SetValue(value)
			end,
			
			["Get"] = function(self)				
				return self.__UI:GetValue()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}
	
    -- New    
    local _New = function(_parent,inheritFrame)
        -- New Frame
		return CreateFrame(_WidgetName,nil,IGAS:GetUI(_parent),inheritFrame)
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end