﻿-- Author      : Kurapica
-- Create Date : 6/13/2008 9:27:45 PM
   
-- Texture
do
	-- Check Version
	local version = 2
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Texture", version) then
		return
	end

    local CreateFrame = CreateFrame
    
	local _WidgetName = "Texture"
    local _Base = "LayeredRegion"
   
	
	-- FuncProxy
	local _FuncProxy = {
		
        -- SetParent(parent or "parentName") - Set the parent for this frame
        ["SetParent"] = function(self, parent)
			-- Get frame from name
			if parent and type(parent) == "string" then
				parent = _G[parent]
				
				if not parent then
					error("No such frame", 2)
				end
			end
			
			parent = IGAS:GetWrapper(parent)
			
			-- Check parent
			if parent and (type(parent) ~= "table" or not parent["IsIGASFrame"] or not parent["AddChild"]) then 
				error("This object can't add child.", 2)
			end
			
			-- Set parent
			if parent then
				if self.__Name then
	                -- Check
	                if parent.__Childs[self.__Name] then
	                    if parent.__Childs[self.__Name] ~= self then
	                        error("The parent has a child with the same name")
	                    else
	                        return
	                    end
	                end
	                
	                -- Remove ob from it's old parent
					self:SetParent(nil)
	                
	                -- Add ob to new parent
	                self.__Parent = parent
	                parent.__Childs[self.__Name] = self
					
					-- SetParent
					if IGAS:GetUI(self) ~= self and IGAS:GetUI(self)["SetParent"] then
						IGAS:GetUI(self):SetParent(IGAS:GetUI(parent))
					end
	            else
	                error("can't add this child", 2)
	            end
			else				
				if self.__Name and self.__Parent then
					self.__Parent.__Childs[self.__Name] = nil
				end
				self.__Parent = nil
			end
        end,
		
		-- Texture:GetBlendMode() - Return the blend mode set by SetBlendMode() 
		["GetBlendMode"] = function(self, ...)
			return self.__UI:GetBlendMode(...)
		end,
	    -- Texture:GetTexCoord() - Gets the 8 texture coordinates that map to the Texture's corners - New in 1.11. 
		["GetTexCoord"] = function(self, ...)
			return self.__UI:GetTexCoord(...)
		end,
	    -- Texture:GetTexCoordModifiesRect() - Get the SetTexCoordModifiesRect setting - New in 1.11 
		["GetTexCoordModifiesRect"] = function(self, ...)
			return self.__UI:GetTexCoordModifiesRect(...)
		end,
	    -- Texture:GetTexture() - Gets this texture's current texture path. 
		["GetTexture"] = function(self, ...)
			return self.__UI:GetTexture(...)
		end,
	    -- Texture:GetVertexColor() - Gets the vertex color for the Texture. 
		["GetVertexColor"] = function(self, ...)
			return self.__UI:GetVertexColor(...)
		end,
	    -- Texture:IsDesaturated() - Gets the desaturation state of this Texture. - New in 1.11 
		["IsDesaturated"] = function(self, ...)
			return self.__UI:IsDesaturated(...)
		end,
	    -- Texture:SetBlendMode("mode") - Set the alphaMode of the texture. 
		["SetBlendMode"] = function(self, ...)
			return self.__UI:SetBlendMode(...)
		end,
	    -- Texture:SetDesaturated(flag) - Set whether this texture should be displayed with no saturation (Note: This has a return value) 
		["SetDesaturated"] = function(self, ...)
			return self.__UI:SetDesaturated(...)
		end,
	    -- Texture:SetGradient("orientation", minR, minG, minB, maxR, maxG, maxB) 
		["SetGradient"] = function(self, ...)
			return self.__UI:SetGradient(...)
		end,
	    -- Texture:SetGradientAlpha("orientation", minR, minG, minB, minA, maxR, maxG, maxB, maxA) 
		["SetGradientAlpha"] = function(self, ...)
			return self.__UI:SetGradientAlpha(...)
		end,
	    -- Texture:SetRotation(angle, [,cx, cy]) - This is simply a shorthand for the appropriate 8 argument SetTexCoord rotation (in C++ so it's fast) 
		["SetRotation"] = function(self, ...)
			return self.__UI:SetRotation(...)
		end,
	    -- Texture:SetTexCoord(minX, maxX, minY, maxY or ULx, ULy, LLx, LLy, URx, URy, LRx, LRy) - Set the corner coordinates for texture display. 
		["SetTexCoord"] = function(self, ...)
			return self.__UI:SetTexCoord(...)
		end,
	    -- Texture:SetTexCoordModifiesRect(enableFlag) - Set whether future SetTexCoord operations should modify the display rectangle rather than stretch the texture. - New in 1.11 
		["SetTexCoordModifiesRect"] = function(self, ...)
			return self.__UI:SetTexCoordModifiesRect(...)
		end,
	    -- Texture:SetTexture("texturePath" or r, g, b[, a]) - Sets the texture to be displayed from a file or to a solid color. 
		["SetTexture"] = function(self, ...)
			return self.__UI:SetTexture(...)
		end,

	    -- SetPortraitTexture(texture,"unit") - Paint a Texture object with the specified unit's portrait. 
		["SetPortraitTexture"] = function(self, ...)
			return SetPortraitTexture(self.__UI, ...)
		end,
	    -- SetPortraitToTexture(texture or "texture", "texturePath") - Sets the texture to be displayed from a file applying circular opacity mask making it look round like portraits. 
		["SetPortraitToTexture"] = function(self, ...)
			return SetPortraitToTexture(self.__UI, ...)
		end,
	}
	
	-- Property
	local _Property = {
		--- BlendMode
		["BlendMode"] = {
			["Set"] = function(self, mode)
				self.__UI:SetBlendMode(mode)
			end,

			["Get"] = function(self)
				return self.__UI:GetBlendMode()
			end,

			["Type"] = IGAS:GetPropertyType("AlphaMode"),
		},
		--- Desaturated
		["Desaturated"] = {		
			["Set"] = function(self, desaturation)
				local shaderSupported = self.__UI:SetDesaturated(desaturation);
				if ( not shaderSupported ) then
					if ( desaturation ) then
						self.__UI:SetVertexColor(0.5, 0.5, 0.5);
					else
						self.__UI:SetVertexColor(1.0, 1.0, 1.0);
					end				
				end
			end,

			["Get"] = function(self)
				return (self.__UI:IsDesaturated() and true) or false
			end,

			["Type"] = IGAS:GetPropertyType("Boolean"), 
		},
		--- Texture
		["Texture"] = {
			["Set"] = function(self, texture)
				if type(texture) == "string" then
					self.__UI:SetTexture(texture)
				elseif type(texture) == "table" then
					self.__UI:SetTexture(texture.red,texture.green,texture.blue,texture.alpha)
				end
			end,

			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetTexture()
				
				if colorTable.green then
					return colorTable
				else
					return colorTable.red
				end
			end,

			["Type"] = IGAS:GetPropertyType("Texture"), 
		},
		--- VertexColor
		["VertexColor"] = {
			["Set"] = function(self, color)
				self:SetVertexColor(color.red, color,green, color.blue, color.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self.__UI:GetVertexColor()
				
				return colorTable
			end,

			["Type"] = IGAS:GetPropertyType("Color"),		
		},
	}

    local _New = function(_parent, layer, inheritsFrom)
		if _parent and _parent["CreateTexture"] then
			return IGAS:GetUI(_parent):CreateTexture(nil, layer, inheritsFrom)
		end
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true, true)
end																 