﻿-- Author      : Kurapica

--------------------------
-- CheckBox		        --
--------------------------
do
	-- Check Version
	local version = 3

	if not IGAS:RegisterFile("IGAS.GUI.Widget.CheckBox", version) then
		return
	end

	local CreateFrame = IGAS.GUI.CreateFrame

	local _WidgetName = "CheckBox"
	local _Base = "Button"

	-- Script
    local function OnClick(self, ...)		
        self:GetChild("ChkBtn").Checked = not self:GetChild("ChkBtn").Checked
		if self:GetChild("ChkBtn").Checked and self.TrueText then
			self.Text = self.TrueText
		elseif (not self:GetChild("ChkBtn").Checked) and self.FalseText then
			self.Text = self.FalseText
		end
        return self:Fire("OnValueChanged", self:GetChild("ChkBtn").Checked)
    end
    
    local function OnEnter(self)
        self.Parent:GetHighlightTexture():Show()
    end
    
    local function OnLeave(self)
        self.Parent:GetHighlightTexture():Hide()
    end
    
	-- ScriptType
	local _ScriptType = {
        ["OnValueChanged"] = true,
	}

	--	FuncProxy
	local _FuncProxy = {
        -- Checked
        ["SetChecked"] = function(self, flag)
            self:GetChild("ChkBtn").Checked = (flag and true) or false
			if self:GetChild("ChkBtn").Checked and self.TrueText then
				self.Text = self.TrueText
			elseif (not self:GetChild("ChkBtn").Checked) and self.FalseText then
				self.Text = self.FalseText
			end
        end,
        ["GetChecked"] = function(self)
            return self:GetChild("ChkBtn").Checked or false
        end,
        -- Text
        ["SetText"] = function(self, text)
            self:GetChild("Text").Text = text
        end,
        ["GetText"] = function(self)
            return self:GetChild("Text").Text
        end,
        ["Enable"] = function(self)
            self.__UI:Enable()
            self:GetChild("ChkBtn"):Enable()
        end,
        ["Disable"] = function(self)
            self.__UI:Disable()
            self:GetChild("ChkBtn"):Disable()
        end,
	}

	--	Property
	local _Property = {
        -- Checked
        ["Checked"] = {
			["Set"] = function(self, value)
				self:SetChecked(value)
			end,
			
			["Get"] = function(self)				
				return (self:GetChecked() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		-- Text
		["Text"] = {
			["Set"] = function(self, text)
				self:SetText(text)
                self.Width = self:GetChild("Text").Width + 26
			end,

			["Get"] = function(self)
				return self:GetText()
			end,

			["Type"] = IGAS:GetPropertyType("String"),
		},
		-- TrueText
		["TrueText"] = {
			["Set"] = function(self, text)
				self.__TrueText = text and tostring(text)
			end,

			["Get"] = function(self)
				return self.__TrueText
			end,

			["Type"] = IGAS:GetPropertyType("String"),
		},
		-- FalseText
		["FalseText"] = {
			["Set"] = function(self, text)
				self.__FalseText = text and tostring(text)
			end,

			["Get"] = function(self)
				return self.__FalseText
			end,

			["Type"] = IGAS:GetPropertyType("String"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("Button",nil,parent)
        
        frame.Height = 24
        frame.Width = 100
        frame.MouseEnabled = true
        
        local chkBtn = CreateFrame("CheckButton", "ChkBtn", frame)
        chkBtn:SetPoint("TOPLEFT", frame, "TOPLEFT")
        chkBtn.MouseEnabled = false
        
        local text = CreateFrame("FontString","Text",frame,"OVERLAY","GameFontNormal")
        text.JustifyH = "LEFT"
        text:SetPoint("TOPLEFT", chkBtn, "TOPRIGHT")
        text:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT")
        text.Text = ""
        
        frame:SetHighlightTexture("Interface\\PaperDollInfoFrame\\UI-Character-Tab-Highlight")
        local t = frame:GetHighlightTexture()
        t:SetBlendMode("ADD")
        t:SetAllPoints(text)
        
        chkBtn.OnEnter = OnEnter
        chkBtn.OnLeave = OnLeave        
        frame:StackScript("OnClick", OnClick)
		
		return frame
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
