﻿--[[
	Widget name	  	ComboBox
	Author     	 	  	Kurapica
	Create Date	 	   	8/01/2008 23:46
	Change Log
		11/25/2008	Combine List and ComboBox Widgets.
		10/16/2009	Only one List will be used for all ComboBoxs
		12/10/2009	Remove List widget, it's public now.
		2010.01.10	Add icons and frames settings
  		2010.01.13	Change the List's Parent to WorldFrame
		2010/02/08  Make the list FrameStrata to TOOLTIP\
		2010/02/23  Add some properties
--]]

-- ComboBox
do
	-- Check Version
	local version = 9
	if not IGAS:RegisterFile("IGAS.GUI.Widget.ComboBox", version) then
		return
	end
    
    local CreateFrame = IGAS.GUI.CreateFrame
	local wipe = table.wipe
	local tinsert = table.insert
	local tremove = table.remove
	
    ------------------------------------------------------
	---------------------  List   ----------------------
	------------------------------------------------------
	local _List = nil
    
    do
        local _ComboBoxListContainer = CreateFrame("Frame", "IGASGUI_ListContainer", WorldFrame)
        _ComboBoxListContainer.__ShowList = _ComboBoxListContainer.__ShowList or nil
		if not _ComboBoxListContainer.__Timer then
			local timer = CreateFrame("Timer", "Timer", _ComboBoxListContainer)
			timer.Interval = 0
			timer.OnTimer = function(self)
				self.Interval = 0
				if self.Parent.__ShowList then
					self.Parent.__ShowList.Visible = false
				end
			end
			_ComboBoxListContainer.__Timer = timer
		end
    
		-- Create List
		_List = CreateFrame("List", nil, _ComboBoxListContainer)
		_List.FrameStrata = "TOOLTIP"
		_List.Visible = false
	
		-- Scripts
        function _List:OnShow()
            if _ComboBoxListContainer.__ShowList and _ComboBoxListContainer.__ShowList ~= self then
                _ComboBoxListContainer.__ShowList.Visible = false
            end
			
			_ComboBoxListContainer.__Timer.Interval = 2
            
            _ComboBoxListContainer.__ShowList = self
        end
        
        function _List:OnHide()
            if _ComboBoxListContainer.__ShowList and _ComboBoxListContainer.__ShowList == self then
                _ComboBoxListContainer.__ShowList = nil
            end
			_ComboBoxListContainer.__Timer.Interval = 0
        end
        
		function _List:OnEnter()
			_ComboBoxListContainer.__Timer.Interval = 0
		end
		
		function _List:OnLeave(self)
			_ComboBoxListContainer.__Timer.Interval = 2
		end
		
		function _List:OnItemChoosed(key, text, icon, frame)
			local parent = self.__ComboBox
			
			if not parent then return end
			
			if parent.__Value ~= key then
				parent.__Value = key
				parent:GetChild("Text"):SetText(text)
				if icon then
					parent:GetChild("Icon"):SetTexture(icon)
					parent:GetChild("Icon").Width = 18
				else
					parent:GetChild("Icon"):SetTexture(nil)
					parent:GetChild("Icon").Width = 1
				end
				
				parent:Fire("OnValueChanged", key)
				parent:Fire("OnTextChanged", text)
			end
			self.Visible = false
		end
						
		_List.__ComboBox = nil
    end

    ------------------------------------------------------
	---------------------   ComboBox   -------------------
	------------------------------------------------------
	local _WidgetName = "ComboBox"
    local _Base = "Frame"

    -- Scripts
	local _FrameBackdrop = {
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        tile = true, tileSize = 16, edgeSize = 16,
        insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
        
    local function OnClick(self)
		if self.Parent.InDesignMode then
			return
		end
		if _List.Visible and _List.__ComboBox == self.Parent then
			_List.Visible = false
		else
			_List.__ComboBox = self.Parent
			_List:ClearAllPoints()
			if self.Parent:GetBottom() - _List.Height > 0 then
				_List:SetPoint("TOPLEFT", self.Parent, "BOTTOMLEFT")
				_List:SetPoint("TOPRIGHT", self.Parent, "BOTTOMRIGHT")
			else
				_List:SetPoint("BOTTOMLEFT", self.Parent, "TOPLEFT")
				_List:SetPoint("BOTTOMRIGHT", self.Parent, "TOPRIGHT")
			end
			_List.Keys = self.Parent.Keys
			_List.Items = self.Parent.Items
			_List.Icons = self.Parent.Icons
			_List.Frames = self.Parent.Frames
			_List.DisplayItemCount = self.__DisplayItemCount
			_List.Visible = true
			_List:Refresh()
			_List:SelectItemByValue(self.Parent.__Value)
		end
    end
    
    local function OnHide(self, ...)
		if _List.__ComboBox == self then
			_List.Visible = false
		end
    end
    
	local function OnCharComposition(self)
	    local value
		local parent = self.Parent
	    local list = parent.Items or {}
	    local text = self:GetText();
		local textlen = strlen(text);
		local i, name;

		if self.Parent.HideDropDownButton then
			return
		end
		
		if not text or text == "" then
			return
		end
		
		i, name = next(list)
		while i do
			if strfind(strupper(name), strupper(text), 1, 1) == 1 then
				self:SetText(name);
				if ( self:IsInIMECompositionMode() ) then
					self:HighlightText(textlen - strlen(arg1), -1);
				else
					self:HighlightText(textlen, -1);
				end
				
				value = parent.Keys[i]
            
	            if value then
	                if value ~= parent.__Value then
	                    parent.__Value = value
						if parent.Icons[i] then
							parent:GetChild("Icon"):SetTexture(parent.Icons[i])
							parent:GetChild("Icon").Width = 18
						else
							parent:GetChild("Icon"):SetTexture(nil)
							parent:GetChild("Icon").Width = 1
						end
						if _List.Visible and _List.__ComboBox == parent then							
							_List:SelectItemByIndex(i)
						end
	                    parent:Fire("OnValueChanged", value)
	                    parent:Fire("OnTextChanged", text)
	                end
	            else
	                parent.__Value = nil
					parent:GetChild("Icon"):SetTexture(nil)
					parent:GetChild("Icon").Width = 1
	                parent:Fire("OnTextChanged", text)
	            end
				return;
			end
			i, name = next(list, i)
		end
	end

	local function OnChar(self)
		if ( not self:IsInIMECompositionMode() ) then
			OnCharComposition(self)
		end
	end
	
	local function OnEditFocusGained(self)
        self:HighlightText()
		self.Parent:Fire("OnEditFocusGained")
	end
	
	local function OnEditFocusLost(self)
        self:HighlightText(0, 0)
		self.Parent:Fire("OnEditFocusLost")
	end
	
	local function OnEnterPressed(self)
		self:ClearFocus()
		self.Parent:Fire("OnEnterPressed")
	end
	
	local function OnTabPressed(self)
		self:ClearFocus()
		self.Parent:Fire("OnTabPressed")
	end
	
	local function OnEscapePressed(self)
		self:ClearFocus()
		self.Parent:Fire("OnEscapePressed")
	end

	local function pairsByKeys (t)
		local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then 
				return nil
			else 
				return a[i], t[a[i]]
			end
		end
		return iter
	end
	
	local function remove(t, idx)
		for i, v in pairsByKeys(t) do
			if i > idx then
				t[i-1] = v
				t[i] = nil
			elseif i == idx then
				t[i] = nil
			end
		end
	end
	
	-- ScriptType
	local _ScriptType = {
        ["OnValueChanged"] = true,
        ["OnTextChanged"] = true,
		["OnEditFocusGained"] = true,
		["OnEditFocusLost"] = true,
		["OnEnterPressed"] = true,
		["OnEscapePressed"] = true,
		["OnTabPressed"] = true,
	}
	
	-- FuncProxy
	local _FuncProxy = {
        -- Dispose, release resource
        ["Dispose"] = function(self)
            if _List.__ComboBox == self then
				_List.Visible = false
				_List:ClearAllPoints()
				_List.__ComboBox = nil
			end
            
            -- Call super's dispose
			self:CallMethod(_Base, "Dispose")
        end,
		-- Clear
		["Clear"] = function(self)
			wipe(self.Keys)
			wipe(self.Items)
			wipe(self.Icons)
			wipe(self.Frames)
			self.__Value = nil
			self:GetChild("Icon"):SetTexture(nil)
			self:GetChild("Icon").Width = 1
			
			if _List.Visible and _List.__ComboBox == self then
				_List:SelectItemByValue(self.__Value)
				_List:Refresh()
			end
		end,
        -- AddItem
        ["AddItem"] = function(self, key, text, icon, frame)
			if key ~= nil then
				tinsert(self.Keys, key)
				if text and type(text) == "string" then
					tinsert(self.Items, text)
				else
					tinsert(self.Items, tostring(key))
				end
				self.Icons[getn(self.Keys)] = icon
				self.Frames[getn(self.Keys)] = frame
			end
			
			if _List.Visible and _List.__ComboBox == self then
				_List:Refresh()
			end
        end,
        -- SetItem
        ["SetItem"] = function(self, key, text, icon, frame)            
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			self.Keys[idx] = key
			self.Items[idx] = text
			self.Icons[idx] = icon
			self.Frames[idx] = frame
			
			if self.__Value == key then
				self:GetChild("Text"):SetText(text)
			end
			
			if _List.Visible and _List.__ComboBox == self then
				_List:Refresh()
			end
        end,
		-- GetItem
		["GetItem"] = function(self, key)
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			if self.Keys[idx] then
				return self.Keys[idx], self.Items[idx], self.Icons[idx], self.Frames[idx]
			end
		end,
		-- GetItemByIndex
		["GetItemByIndex"] = function(self, idx)
			if self.Keys[idx] then
				return self.Keys[idx], self.Items[idx], self.Icons[idx], self.Frames[idx]
			end
		end,
        -- RemoveItem
        ["RemoveItem"] = function(self, key)
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			if self.Keys[idx] and self.Keys[idx] == key then
				tremove(self.Keys, idx)
				tremove(self.Items, idx)
				remove(self.Icons, idx)
				remove(self.Frames, idx)
				
				if self.__Value == key then
					self:SetValue(nil)
				elseif _List.Visible and _List.__ComboBox == self then
					_List:Refresh()
				end
			end
        end,
		-- SetList
		["SetList"] = function(self, list)
			self:Clear()
			if type(list) == "table" then
				for k, v in pairs(list) do
					self:AddItem(k, v)
				end
			else
				error("The parameter must be a table")
			end
		end,
		-- GetDisplayItemCount
		["GetDisplayItemCount"] = function(self)
			return self.__DisplayItemCount
		end,
		-- SetDisplayItemCount	
		["SetDisplayItemCount"] = function(self, cnt)
			self.__DisplayItemCount = cnt
			if _List.Visible and _List.__ComboBox == self then
				_List:Refresh()
			end
		end,
        -- GetValue
        ["GetValue"] = function(self)
            return self.__Value
        end,
        -- SetValue
        ["SetValue"] = function(self, value)
            local text, icon
			if self.__Value == value then
				return
			end
            if value then
				for i,k in ipairs(self.Keys) do
					if k == value then
						text = self.Items[i]
						icon = self.Icons[i]
						break
					end
				end
				
                if text then
                    self.__Value = value
                    self:GetChild("Text"):SetText(text)
					if icon then
						self:GetChild("Icon"):SetTexture(icon)
						self:GetChild("Icon").Width = 18
					else
						self:GetChild("Icon"):SetTexture(nil)
						self:GetChild("Icon").Width = 1
					end
                    
                    self:Fire("OnValueChanged", value)
                    self:Fire("OnTextChanged", text)
					if _List.Visible and _List.__ComboBox == self then
						_List:SelectItemByValue(self.__Value)
					end
                end
			else
				self.__Value = nil
				self:GetChild("Text"):SetText("")
				self:GetChild("Icon"):SetTexture(nil)
				self:GetChild("Icon").Width = 1
				
				self:Fire("OnValueChanged", nil)
				self:Fire("OnTextChanged", "")
				if _List.Visible and _List.__ComboBox == self then
					_List:SelectItemByValue(nil)
				end
            end
        end,
        -- GetText
        ["GetText"] = function(self)
            return self:GetChild("Text"):GetText()
        end,
        -- SetText
        ["SetText"] = function(self, text)
			local value, icon
            if text and type(text) == "string" and self:GetChild("Text").Text ~= text then
                for i,t in ipairs(self.Items) do
					if t == text then
						value = self.Keys[i]
						icon = self.Icons[i]
						break
					end
				end
                if value then
                    self.__Value = value
                    self:GetChild("Text"):SetText(text)
					if icon then
						self:GetChild("Icon"):SetTexture(icon)
						self:GetChild("Icon").Width = 18
					else
						self:GetChild("Icon"):SetTexture(nil)
						self:GetChild("Icon").Width = 1
					end
                    
                    self:Fire("OnValueChanged", value)
                    self:Fire("OnTextChanged", text)
					if _List.Visible and _List.__ComboBox == self then
						_List:SelectItemByValue(self.__Value)
					end
                else
                    self.__Value = nil
                    self:GetChild("Text"):SetText(text)
					self:GetChild("Icon"):SetTexture(nil)
					self:GetChild("Icon").Width = 1
					
					self:Fire("OnValueChanged", nil)
                    self:Fire("OnTextChanged", text)
					if _List.Visible and _List.__ComboBox == self then
						_List:SelectItemByValue(nil)
					end
                end
            end
        end,
        -- GetEditable
		["GetEditable"] = function(self)
			return self:GetChild("Text").MouseEnabled
		end,
        -- SetEditable
		["SetEditable"] = function(self, flag)
			self:GetChild("Text").MouseEnabled = flag
            if not flag then
                self:GetChild("Text").AutoFocus = false
				if self:GetChild("Text"):HasFocus() then
					self:GetChild("Text"):ClearFocus()
				end
            end
		end,
		-- GetTextObject
		["GetTextObject"] = function(self)
			return self:GetChild("Text")
		end,
        -- SetAutoFocus
        ["SetAutoFocus"] = function(self, flag)
            if flag and self:GetChild("Text").MouseEnabled then
                self:GetChild("Text").AutoFocus = true
            else
                self:GetChild("Text").AutoFocus = false
            end
        end,
        -- GetAutoFocus
        ["IsAutoFocus"] = function(self)
            return self:GetChild("Text").AutoFocus
        end,
        -- ClearFocus
        ["ClearFocus"] = function(self)
            self:GetChild("Text"):ClearFocus()
        end,
		-- HasFocus
		["HasFocus"] = function(self)
			return self:GetChild("Text"):HasFocus()
		end,
		-- SetFocus
		["SetFocus"] = function(self)
			if self:GetChild("Text").MouseEnabled then
				self:GetChild("Text"):SetFocus()
			end
		end,
    }
    
	-- Property
	local _Property = {
        -- AutoFocus
        ["AutoFocus"] = {
			["Set"] = function(self, state)
				self:SetAutoFocus(state)
			end,
			
			["Get"] = function(self)
				return (self:IsAutoFocus() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
        -- DisplayItemCount
		["DisplayItemCount"] = {
			["Set"] = function(self, cnt)
				self:SetDisplayItemCount(cnt)
			end,
			
			["Get"] = function(self)
				return self:GetDisplayItemCount()
			end,
            
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- Value
		["Value"] = {
			["Set"] = function(self, value)
				self:SetValue(value)
			end,
			
			["Get"] = function(self)
				return self:GetValue()
			end,
            
			["Type"] = IGAS:GetPropertyType("Object"),
		},        
        -- Value
		["Text"] = {
			["Set"] = function(self, text)
				self:SetText(text)
			end,
			
			["Get"] = function(self)
				return self:GetText()
			end,
            
			["Type"] = IGAS:GetPropertyType("String"),
		},
        -- Editable
        ["Editable"] = {
			["Set"] = function(self, flag)
				self:SetEditable(flag)
			end,
			
			["Get"] = function(self)
				return (self:GetEditable() and true) or false
			end,
            
			["Type"] = IGAS:GetPropertyType("Boolean"),
        },
		-- HideDropDownButton
		["HideDropDownButton"] = {
			["Set"] = function(self, flag)
				self:GetChild("DropdownBtn").Visible = not flag
			end,
			
			["Get"] = function(self)
				return not self:GetChild("DropdownBtn").Visible
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		-- Keys
		["Keys"] = {
			["Set"] = function(self, keys)
				if keys and type(keys) == "table" then
					self.__Keys = keys
				else
					self.__Keys = {}
				end
			end,
			["Get"] = function(self)
				self.__Keys = self.__Keys or {}
				return self.__Keys
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Items
		["Items"] = {
			["Set"] = function(self, items)
				if items and type(items) == "table" then
					self.__Items = items
				else
					self.__Items = {}
				end
			end,
			["Get"] = function(self)
				self.__Items = self.__Items or {}
				return self.__Items
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Icons
		["Icons"] = {
			["Set"] = function(self, icons)
				if icons and type(icons) == "table" then
					self.__Icons = icons
				else
					self.__Icons = {}
				end
			end,
			["Get"] = function(self)
				self.__Icons = self.__Icons or {}
				return self.__Icons
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Frames
		["Frames"] = {
			["Set"] = function(self, frames)
				if frames and type(frames) == "table" then
					self.__Frames = frames
				else
					self.__Frames = {}
				end
			end,
			["Get"] = function(self)
				self.__Frames = self.__Frames or {}
				return self.__Frames
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- ItemCount
		["ItemCount"] = {			
			["Get"] = function(self)
				return getn(self.Keys)
			end,
            
			["Type"] = IGAS:GetPropertyType("Number"),
		},
    }
    
    -- New    
    local _New = function(parent)
        -- EditBox
		local frame = CreateFrame("Frame",nil,parent)
		
        frame:SetWidth(180)
        frame:SetHeight(26)
		frame:SetBackdrop(_FrameBackdrop)
		frame:SetBackdropColor(0,0,0,1)

        local btnDropdown = CreateFrame("Button", "DropdownBtn", frame)
        btnDropdown:SetWidth(32)
        btnDropdown:SetHeight(32)
        btnDropdown:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 3, 3)
        btnDropdown:SetNormalTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Up.blp")
        btnDropdown:SetPushedTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Down.blp")
        btnDropdown:SetDisabledTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Disabled.blp")
        btnDropdown:SetHighlightTexture("Interface\\Buttons\\UI-ScrollBar-ScrollDownButton-Highlight.blp", "ADD")
        btnDropdown:SetHitRectInsets(6, 7, 7, 8)

		icon = CreateFrame("Texture", "Icon", frame, "OVERLAY")
		icon:SetWidth(18)
		icon:SetHeight(18)
		icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
		icon:SetPoint("LEFT", frame, "LEFT", 4, 0)

        local edit = CreateFrame("EditBox", "Text", frame)
        edit:SetPoint("TOP", frame, "TOP", 0, -4)
        edit:SetPoint("LEFT", icon, "RIGHT")
        edit:SetPoint("RIGHT", btnDropdown, "LEFT", 6, 0)
        edit:SetPoint("BOTTOM", frame, "BOTTOM", 0, 4)
        edit:SetAutoFocus(false)
        edit:SetJustifyH("CENTER")
        edit:SetJustifyV("MIDDLE")
        edit:SetFontObject("GameFontNormalSmall")
		edit.MouseEnabled = false
        edit.AutoFocus = false
		
        -- Event Handle
        btnDropdown.OnClick = OnClick
		
        edit.OnTabPressed = OnTabPressed
        edit.OnEnterPressed = OnEnterPressed
        edit.OnEscapePressed = OnEscapePressed
        edit.OnEditFocusLost = OnEditFocusLost
        edit.OnEditFocusGained = OnEditFocusGained
		edit.OnCharComposition = OnCharComposition
		edit.OnChar = OnChar
		
		frame:StackScript("OnHide", OnHide)
        
        return frame
    end
	
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true)
end