﻿-- Author      : Kurapica

--------------------------
-- DataGrid		        --
--------------------------
do
	-- Check Version
	local version = 2

	if not IGAS:RegisterFile("IGAS.GUI.Widget.DataGrid", version) then
		return
	end

	local CreateFrame = IGAS.GUI.CreateFrame

    ------------------------------------------------------
	---------------------  Cell   ----------------------
	------------------------------------------------------
    local Cell = (IGAS.GUI.Widget.DataGrid and IGAS.GUI.Widget.DataGrid.Cell) or nil
	
	do
		local _WidgetName = "Cell"
        local _Base = "ComboBox"
		
		-- Define Block
		local _CellTypeEnum = IGAS:NewEnum("CellType", {
			"Text",
			"Label",
			"ComboBox",
			"Boolean",
		})

		local _CellType = IGAS:NewPropertyType("CellType", {
			["Type"] = "enum",
			["EnumType"] = _CellTypeEnum,
		})

		-- Scripts
		local _FrameBackdrop = {
	        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
	        --edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
	        tile = true, tileSize = 16, edgeSize = 16,
	        insets = { left = 3, right = 3, top = 3, bottom = 3 }
		}
		-- ScriptType
		local _ScriptType = {
		}

		--	FuncProxy
		local _FuncProxy = {
			-- CellType
			["SetCellType"] = function(self, cellType)				
	            -- Check Style
	            if not cellType or type(cellType) ~= "string" then
	                return
	            end
	                        
	            if (not _CellTypeEnum[cellType]) or cellType == self.__CellType then
	                return
	            end
				
	            self.__CellType = cellType
				
				self:Clear()
				if cellType == "Text" then
					self.Editable = true
					self.HideDropDownButton = true
				elseif cellType == "Label" then
					self.Editable = false
					self.HideDropDownButton = true
				elseif cellType == "ComboBox" then
					self.Editable = false
					self.HideDropDownButton = false
				elseif cellType == "Boolean" then
					self.Editable = false
					self.HideDropDownButton = false
					self:SetList{
						[true] = ["True"],
						[false] = ["False"],
					}
				end
			end,
			["GetCellType"] = function(self)
				return self.__CellType
			end,
		}

		--	Property
		local _Property = {
			["CellType"] = {
				["Set"] = function(self, cellType)
	                self:SetCellType(cellType)
				end,
				
				["Get"] = function(self)
					return self:GetCellType()
				end,
				
				["Type"] = _CellType,
			},
		}

		local function _New(parent)		
			-- New Frame
			local frame = CreateFrame(_Base,nil,parent)
			
			frame:SetBackdrop(_FrameBackdrop)
			frame.__CellType = "ComboBox"
			
			return frame
		end

		-- Register
		local _WidgetInfo = {
			["WidgetName"] = _WidgetName,
			["Base"] = _Base,
			["FuncProxy"] = _FuncProxy,
			["Property"] = _Property,
			["ScriptType"] = _ScriptType,
			["New"] = _New,
		}

		Cell = IGAS:NewWidget(_WidgetInfo, Cell)
	end
	
	local _WidgetName = "DataGrid"
	local _Base = "Frame"

	-- Script
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
    
	-- ScriptType
	local _ScriptType = {
	}

	--	FuncProxy
	local _FuncProxy = {
	}

	--	Property
	local _Property = {
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame(_Base,nil,parent)
        
		frame.Width = 700
		frame.Height = 500
		frame.FrameStrata = "DIALOG"
		frame.Toplevel = true

		frame:SetBackdrop(_FrameBackdrop)
		frame:SetBackdropColor(0,0,0,1)
		
		-- Scroll Bar
		local scrollBar = CreateFrame("ScrollBar", "ScrollBar", frame)
		scrollBar:Hide()
		
		-- Event Handle
		scrollBar.OnValueChanged = OnValueChanged
		
		return frame
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
