﻿-- Author      : Kurapica
-- ChangreLog  :
--				2010.01.13	Change the DropDownList's Parent to WorldFrame

--------------------------
-- DropDownList	        --
--------------------------
--[[
    
--]]
do
	-- Check Version
	local version = 7

	if not IGAS:RegisterFile("IGAS.GUI.Widget.DropDownList", version) then
		return
	end

	local CreateFrame = IGAS.GUI.CreateFrame

    ------------------------------------------------------
	------------------- DropDownMenuButton----------------
	------------------------------------------------------
    local DropDownMenuButton = (IGAS.GUI.Widget.DropDownList and IGAS.GUI.Widget.DropDownList.DropDownMenuButton) or nil
    local itemHeight = 16
    
    do
        local _WidgetName = "DropDownMenuButton"
        local _Base = "Button"
        
        -- Scripts
        --[[
        local function OpenColorPicker(self)
            ColorPickerFrame.func = info.swatchFunc
            ColorPickerFrame.hasOpacity = info.hasOpacity
            ColorPickerFrame.opacityFunc = info.opacityFunc
            ColorPickerFrame.opacity = info.opacity
            ColorPickerFrame.previousValues = {r = info.r, g = info.g, b = info.b, opacity = info.opacity}
            ColorPickerFrame.cancelFunc = info.cancelFunc
            ColorPickerFrame.extraInfo = info.extraInfo
            -- This must come last, since it triggers a call to ColorPickerFrame.func()
            ColorPickerFrame:SetColorRGB(info.r, info.g, info.b)
            ShowUIPanel(ColorPickerFrame)
        end
        
        --- colorBack
        local function colorBack_OnClick(self)
            self.Parent:Fire("OnClick")
			OpenColorPicker(self:GetParent())
        end
        
        local function colorBack_OnEnter(self)
            self.Parent.Parent:CloseDropDownMenus()
            self:GetChild("Bg"):SetVertexColor(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			self.Parent.Parent:Fire("OnEnter")
        end
        
        local function colorBack_OnLeave(self)
            self:GetChild("Bg"):SetVertexColor(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			self.Parent.Parent:Fire("OnLeave")
        end
        --]]
        local function hideDropList(self)
			self = self.__DdList or self
            if self.__Childs and type(self.__Childs) == "table" then
                for _, v in pairs(self.__Childs) do
                    if type(v) == "table" and v["IsObjectType"] and v:IsObjectType(DropDownMenuButton) then
                        if v.__DropDownList then
                            hideDropList(v.__DropDownList)
                        end
                    end
                end
            end
            self.Visible = false
        end
    
        local function hideDropDownList(self)
            if self.__DropDownList then
                hideDropList(self.__DropDownList)
            end
        end
        
        local function showDropDownList(self)
            local offsetx, offsety, menu
            
            -- Hide other sub dropdownlist
            if self.Parent.__Childs and type(self.Parent.__Childs) == "table" then
                for _,v in pairs(self.Parent.__Childs) do
                    if type(v) == "table" then
                        hideDropDownList(v)
                    end
                end
            end
            
            -- Show self's dropdownlist
            menu = self.__DropDownList
            
            if menu then
                menu.Visible = true
                
                -- Offset-X
                if self.Parent:GetRight() + menu.Width > GetScreenWidth() then
                    offsetx = - (self.Parent.Width + menu.Width)
                else
                    offsetx = 0
                end
                -- Offset-Y
                if self:GetTop() < menu.Height then
                    offsety = menu.Height - self:GetTop()
                else
                    offsety = 0
                end
				menu:ClearAllPoints()
                menu:SetPoint("TOPLEFT", self, "TOPRIGHT", offsetx, offsety)
            end
        end
        
        local function toggleDropDownList(self)
            if self.__DropDownList then
                if self.__DropDownList.Visible then
                    hideDropDownList(self)
                else
                    showDropDownList(self)
                end
            end
        end
        
        --- expandArrow
        local function expandArrow_OnClick(self)
            toggleDropDownList(self.Parent)
        end
        
        local function expandArrow_OnEnter(self)
            showDropDownList(self.Parent)
			self.Parent.Parent:Fire("OnEnter")
        end
        
        local function expandArrow_OnLeave(self)
            self.Parent.Parent:Fire("OnLeave")
        end
        
        -- Item        
        local function item_OnEnter(self)
            showDropDownList(self)
            self:GetChild("HighLight"):Show()
            self.Parent:Fire("OnEnter")
        end
        
        local function item_OnLeave(self)
            self:GetChild("HighLight"):Hide()
            self.Parent:Fire("OnLeave")
        end
        
        local function updateWidth(self)
            local maxW = 0
            
            if self.__Childs and type(self.__Childs) == "table" then
                for i,v in pairs(self.__Childs) do
                    if type(v) == "table" and v["IsObjectType"] and v:IsObjectType(DropDownMenuButton) then
                        if v:GetChild("Text").Width > maxW then
                            maxW = v:GetChild("Text").Width
                        end
                    end
                end
                
                maxW = maxW + 40
                
                self.Width = maxW
            end
        end
        
        -- OnClick Script
        local function OnClick(self, ...)            
            -- if this is checkButton
            if self.__IsCheckButton then
                if self.Parent.__MultiSelect then
                    self.Checked = not self.Checked
                    return true
                else
                    if self.Checked then
                        -- No need to action
                        hideDropList(self.Parent)
                        return true
                    end
                    self.Checked = true
                    for i, v in pairs(self.Parent.__Childs) do
                        if type(v) == "table" and v["IsObjectType"] and v:IsObjectType(DropDownMenuButton) and v.__IsCheckButton and v ~= self and v.Checked then
                            v.Checked = false
                        end
                    end
                    hideDropList(self.Parent)
                end
                return
            end
            
            local root = self.Parent
                        
            while root.__MenuBase do
                root = root.__MenuBase
            end
            
            hideDropList(root)
		end
        
        -- ScriptType
        local _ScriptType = {
            ["OnCheckChanged"] = true,
        }
        
        -- FuncProxy
        local _FuncProxy = {
            -- Dispose, release resource
            ["Dispose"] = function(self)
                local id = self.ID
                local _id
                
                if self.Parent.__Childs and type(self.Parent.__Childs) == "table" then
                    for i,v in pairs(self.Parent.__Childs) do
                        if type(v) == "table" then
                            _id = v.ID
                            if _id and type(_id) == "number" and _id > id then
                                _id = _id - 1
                                v.ID = _id
                                v:SetPoint("TOP", self.Parent, "TOP", 0, -(16 + itemHeight * (_id - 1)))
                            end
                        end
                    end
                end
                
                self.Parent.__ItemCount = self.Parent.__ItemCount - 1
                self.Parent.Height = self.Parent.Height - itemHeight
                
	            -- Call super's dispose
				self:CallMethod(_Base, "Dispose")
            end,
            -- SetText
            ["SetText"] = function(self, text)
                if self.__TextColor then
                    self:GetChild("Text").Text = self.__TextColor..text.."|r"
                else
                    self:GetChild("Text").Text = text
                end
                self.__Text = text
                updateWidth(self.Parent)
            end,
            -- GetText
            ["GetText"] = function(self)
                return self.__Text or ""
            end,
			-- SetDropDownList
			["SetDropDownList"] = function(self, list)
				self.__DropDownList = list
				if self.__DropDownList then
					self:GetChild("ExpandArrow"):Show()
					if list["IsObjectType"] and list:IsObjectType("DropDownList") then
						list.__DdList.__MenuLevel = self.Parent.__MenuLevel + 1
						list.__DdList.__MenuBase = self.Parent
					end
				else
					self:GetChild("ExpandArrow"):Hide()
				end
			end,
			-- GetDropDownList
			["GetDropDownList"] = function(self)
				return self.__DropDownList
			end,
			-- SetIcon
			["SetIcon"] = function(self, texture)
				self:GetChild("Icon").Visible = (texture and true) or false
				self:GetChild("Icon"):SetTexture(texture)
			end,
            -- SetTextColor
            ["SetTextColor"] = function(self, r, g, b)
                if r and g and b then
                    self.__TextColor = string.format("|cFF%02x%02x%02x", r, g, b)
                    
                    if self.__Text then
                        self:GetChild("Text").Text = self.__TextColor..self.__Text.."|r"
                    end
                else
                    self.__TextColor = nil
                    self:GetChild("Text").Text = self.__Text
                end
            end,
        }
        
        -- Property
        local _Property = {
			-- DropDownList
            ["DropDownList"] = {
                ["Set"] = function(self, list)
                    self:SetDropDownList(list)
                end,
    
                ["Get"] = function(self)
                    return self:GetDropDownList()
                end,
    
                ["Type"] = IGAS:GetPropertyType("Object"),
            },
            -- Text
            ["Text"] = {
                ["Set"] = function(self, text)
                    self:SetText(text)
                end,
                
                ["Get"] = function(self)
                    return self:GetText()
                end,
                
                ["Type"] = IGAS:GetPropertyType("String"),
            },
            -- IsCheckButton
            ["IsCheckButton"] = {                
                ["Set"] = function(self, flag)
                    self.__IsCheckButton = (flag and true) or false
                end,
                
                ["Get"] = function(self)
                    return (self.__IsCheckButton and true) or false
                end,
                
                ["Type"] = IGAS:GetPropertyType("Boolean"),
            },
            -- Checked
            ["Checked"] = {
                ["Set"] = function(self, flag)
                    self:GetChild("Check").Visible = (flag and true) or false
                    self:Fire("OnCheckChanged")
                end,
                
                ["Get"] = function(self)
                    return self:GetChild("Check").Visible
                end,
                
                ["Type"] = IGAS:GetPropertyType("Boolean"),
            },
        }
        
        -- New
        local _New = function(parent)
			local btn = CreateFrame("Button", nil, parent.__DdList or parent)
            local maxID = parent.__ItemCount or 0
            
            btn.Width = 100
            btn.Height = itemHeight
            
            maxID = maxID + 1
            btn.ID = maxID
            parent.__ItemCount = maxID
            
            -- Anchor
            btn:SetPoint("LEFT", parent, "LEFT", 4, 0)
            btn:SetPoint("RIGHT", parent, "RIGHT", -4, 0)
            btn:SetPoint("TOP", parent, "TOP", 0, - (16 + itemHeight * (maxID - 1)))
            parent.Height = 32 + itemHeight * maxID
            
            -- HighLightTexture
            local highLight = CreateFrame("Texture", "HighLight", btn, "BACKGROUND")
            highLight:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
            highLight:SetBlendMode("ADD")
            highLight:SetAllPoints(btn)
            highLight.Visible = false
            
            -- CheckTexture
            local check = CreateFrame("Texture", "Check", btn, "ARTWORK")
            check:SetTexture("Interface\\Buttons\\UI-CheckBox-Check")
            check.Height = 18
            check.Width = 18
			check.Visible = false
            check:SetPoint("LEFT", btn, "LEFT")
            
            -- IconTexture
            local icon = CreateFrame("Texture", "Icon", btn, "ARTWORK")
            icon.Height = 16
            icon.Width = 16
            icon.Visible = false
            icon:SetPoint("LEFT", btn, "LEFT")     
            
            --[[
            -- ColorSwatch
            local colorsWatch = CreateFrame("Button", "ColorSwatch", btn)
            colorsWatch.Height = 16
            colorsWatch.Width = 16
            colorsWatch.Visible = false
            colorsWatch:SetPoint("RIGHT", btn, "RIGHT", -6, 0)
            colorsWatch:SetNormalTexture("Interface\\ChatFrame\\ChatFrameColorSwatch")
            
            local colorBack = CreateFrame("Texture", "Bg", colorsWatch, "BACKGROUND")
            colorBack.Height = 14
            colorBack.Width = 14
            colorBack:SetPoint("CENTER", colorsWatch, "CENTER")
            colorBack:SetVertexColor(1.0, 1.0, 1.0)
            
            colorsWatch.OnClick = colorBack_OnClick
            colorsWatch.OnEnter = colorBack_OnEnter
            colorsWatch.OnLeave = colorBack_OnLeave
            --]]
            
            -- ExpandArrow
            local expandArrow = CreateFrame("Button", "ExpandArrow", btn)
            expandArrow.Height = 16
            expandArrow.Width = 16
            expandArrow.Visible = false
            expandArrow:SetPoint("RIGHT", btn, "RIGHT", 0, 0)
            expandArrow:SetNormalTexture("Interface\\ChatFrame\\ChatFrameExpandArrow")
            
            expandArrow.OnClick = expandArrow_OnClick
            expandArrow.OnEnter = expandArrow_OnEnter
            expandArrow.OnLeave = expandArrow_OnLeave
            
            -- FontString
            local text = CreateFrame("FontString","Text",btn,"OVERLAY","GameFontNormal")
            text.JustifyH = "LEFT"
            text:SetPoint("LEFT", btn, "LEFT", 18, 0)
            text:SetHeight(16)
            btn:SetFontString(text)
            
            -- Script
            btn:StackScript("OnEnter", item_OnEnter)
            btn:StackScript("OnLeave", item_OnLeave)
            btn:StackScript("OnClick", OnClick)
            
            --- Font
            btn:SetNormalFontObject(GameFontHighlightSmallLeft)
            btn:SetDisabledFontObject(GameFontDisableSmallLeft)
            btn:SetHighlightFontObject(GameFontHighlightSmallLeft)
            
            return btn
        end
        
        local _WidgetInfo = {
            ["WidgetName"] = _WidgetName,
            ["Base"] = _Base,
            ["FuncProxy"] = _FuncProxy,
            ["Property"] = _Property,
            ["ScriptType"] = _ScriptType,
            ["New"] = _New,
        }
        
        DropDownMenuButton = IGAS:NewWidget(_WidgetInfo, DropDownMenuButton)
    end
    
    ------------------------------------------------------
	--------------------- DropDownList ----------------------
	------------------------------------------------------    
	local _WidgetName = "DropDownList"
	local _Base = "VirtualUIObject"
    
    local _DropDownListContainer = CreateFrame("Frame", "IGASGUI_ListContainer", WorldFrame)
    _DropDownListContainer.__ShowList = _DropDownListContainer.__ShowList or nil

	-- Script
	local _FrameBackdrop = {
        bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
        edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
        tile = true, tileSize = 16, edgeSize = 16,
        insets = { left = 5, right = 5, top = 5, bottom = 5 }
	}
    	
    local function hideDropList(self)
		self = self.__DdList or self
        if self.__Childs and type(self.__Childs) == "table" then
            for i,v in pairs(self.__Childs) do
                if type(v) == "table" and v["IsObjectType"] and v:IsObjectType(DropDownMenuButton) then
                    if v.__DropDownList then
                        hideDropList(v.__DropDownList)
                    end
                end
            end
        end
        self.Visible = false
    end
    
	local function OnTimer(self)
        hideDropList(self.Parent)
        self.Interval = 0
	end
	
	local function OnEnter(self)
		self:GetChild("Timer").Interval = 0
        
        if self.__MenuBase then
            self.__MenuBase:Fire("OnEnter")
        end
		
		return self.__Mask:Fire("OnEnter")
	end
	
	local function OnLeave(self)
		self:GetChild("Timer").Interval = 2
        if self.__MenuBase then
            self.__MenuBase:Fire("OnLeave")
        end
		
		return self.__Mask:Fire("OnLeave")
	end
	
    local function OnShow(self, ...)        
        self:GetChild("Timer").Interval = 2
		
        -- Set the dropdownframe scale
		local uiScale
		local uiParentScale = UIParent:GetScale()
		if ( GetCVar("useUIScale") == "1" ) then
			uiScale = tonumber(GetCVar("uiscale"))
			if ( uiParentScale < uiScale ) then
				uiScale = uiParentScale
			end
		else
			uiScale = uiParentScale
		end
        
		self:SetScale(uiScale)
		
        if self.__MenuLevel > 1 then
            return self.__Mask:Fire("OnShow")
        end
        
        if _DropDownListContainer.__ShowList and _DropDownListContainer.__ShowList ~= self then
            _DropDownListContainer.__ShowList.Visible = false
        end
        
        _DropDownListContainer.__ShowList = self

		if self.__ShowOnCursor then
            local cursorX, cursorY = GetCursorPosition()
            cursorX = cursorX/uiScale
            cursorY =  cursorY/uiScale
            
            local offsetX, offsetY
            
            offsetX = cursorX
            offsetY = cursorY
            
            self:ClearAllPoints()
            self:SetPoint("TOPLEFT", UIParent, "BOTTOMLEFT", offsetX, offsetY)
            
            local x, y = self:GetCenter()
            
            if not (x and y) then
                self:Hide()
                return
            end
            
            -- Determine whether the menu is off the screen or not
            local offscreenY, offscreenX;
            if ( (y - self:GetHeight()/2) < 0 ) then
                offscreenY = 1
            end
            if ( self:GetRight() > GetScreenWidth() ) then
                offscreenX = 1
            end
            
            self:ClearAllPoints()
            if offscreenX and offscreenY then
                self:SetPoint("BOTTOMRIGHT", UIParent, "BOTTOMLEFT", offsetX, offsetY)            
            elseif offscreenX then
                self:SetPoint("TOPRIGHT", UIParent, "BOTTOMLEFT", offsetX, offsetY)  
            elseif offscreenY then
                self:SetPoint("BOTTOMLEFT", UIParent, "BOTTOMLEFT", offsetX, offsetY)  
            end
        end
		
		return self.__Mask:Fire("OnShow")
    end
    
    local function OnHide(self, ...)        
		self:GetChild("Timer").Interval = 0
        
		hideDropList(self)
        
        if self.__MenuLevel > 1 then
            return self.__Mask:Fire("OnHide")
        end
        
        if _DropDownListContainer.__ShowList and _DropDownListContainer.__ShowList == self then
            _DropDownListContainer.__ShowList = nil
        end
		
		return self.__Mask:Fire("OnHide")
    end
    
	-- ScriptType
	local _ScriptType = {
        ["OnShow"] = true,
        ["OnHide"] = true,
        ["OnEnter"] = true,
        ["OnLeave"] = true,
	}
	
	--	FuncProxy
	local _FuncProxy = {
        ["AddMenuButton"] = function(self, btnName)
			self = self.__DdList
            if not btnName or type(btnName) ~= "string" or btnName == "" then
                error("the parameter must be a string.")
            end
            
            local btn = CreateFrame(DropDownMenuButton, btnName, self)
            btn.Text = btnName
            
            return btn
        end,
        
        ["GetMenuButton"] = function(self, btnName)
			self = self.__DdList
            if not btnName or type(btnName) ~= "string" or btnName == "" then
                error("the parameter must be a string.")
            end
            return self:GetChild(btnName)
        end,
        
        ["RemoveMenuButton"] = function(self, btnName)
			self = self.__DdList
            if not btnName or type(btnName) ~= "string" or btnName == "" then
                error("the parameter must be a string.")
            end
            if self:GetChild(btnName) then
                self:GetChild(btnName):Dispose()
            end
        end,

        --- Original Functions
        -- ClearAllPoints() - Clear all attachment points for this object. 
        ["ClearAllPoints"] = function(self)
            self.__DdList:ClearAllPoints()
        end,
				
        -- GetBottom() - Get the y location of the bottom edge of this frame
        ["GetBottom"] = function(self)
            return self.__DdList:GetBottom()
        end,
        
        -- GetCenter() - Get the coordinates of the center of this frame
        ["GetCenter"] = function(self)
            return self.__DdList:GetCenter()
        end,
        
        -- GetHeight() - Get the height of this object. 
        ["GetHeight"] = function(self)
            return self.__DdList:GetHeight()
        end,
                
        -- GetLeft() - Get the x location of the left edge of this frame
        ["GetLeft"] = function(self)
            return self.__DdList:GetLeft()
        end,
        
        -- GetNumPoints() - Get the number of anchor points for this frame
        ["GetNumPoints"] = function(self)
            return self.__DdList:GetNumPoints()
        end,
        
        -- GetPoint(pointNum) - Get details for an anchor point for this frame (point, relativeTo, relativePoint, xofs, yofs)
        ["GetPoint"] = function(self, pointNum)
            return self.__DdList:GetPoint(pointNum)
        end,
        
        -- GetRight() - Get the x location of the right edge of this frame
        ["GetRight"] = function(self)
            return self.__DdList:GetRight()
        end,
        
        -- GetTop() - Get the y location of the top edge of this frame
        ["GetTop"] = function(self)
            return self.__DdList:GetTop()
        end,
        
        -- GetWidth() - Get the width of this object. 
        ["GetWidth"] = function(self)
            return self.__DdList:GetWidth()
        end,
                
        -- Hide() - Set this object to hidden (it and all of its children will disappear). 
        ["Hide"] = function(self)
            return self.__DdList:Hide()
        end,
        
        -- Show() - Set this object to shown (it will appear if its parent is visible). 
        ["Show"] = function(self)
            self.__DdList:Show()
        end,
                
        -- IsShown() - Determine if this object is shown (would be visible if its parent was visible). 
        ["IsShown"] = function(self)
            return self.__DdList:IsShown()
        end,
        
        -- IsVisible() - Get whether the object is visible on screen (logically (IsShown() and GetParent():IsVisible())); 
        ["IsVisible"] = function(self)
            return self.__DdList:IsVisible()
        end,
        
        -- SetAllPoints(frame or "frameName") - Set all anchors to match edges of specified frame
        ["SetAllPoints"] = function(self, frame)
            if frame and type(frame) == "string" then
                frame = _G[frame]
            end
            
            if not frame or type(frame) ~= "table" then
                frame = self.Parent
            end
                        
            self.__DdList:SetAllPoints(frame)
        end,
        
        -- SetPoint("point","relativeFrame" or relativeObject,"relativePoint"[, xOfs, yOfs]) - Set an attachment point of this object - Updated in 1.10. Since 2.2 cooordinates are now relative to the closest anchor point 
        ["SetPoint"] = function(self, point, relativeObject, relativePoint, xOfs, yOfs)
			self.__DdList:SetPoint(point, relativeObject, relativePoint, xOfs, yOfs)
        end,
        
        -- Dispose, release resource
        ["Dispose"] = function(self)
			self.__DdList:Dispose()
            
            -- Call super's dispose
			self:CallMethod(_Base, "Dispose")
        end,

		--Region:GetRect() - Get frame's left, bottom, width, height. 
		["GetRect"] = function(self)
			return self.__DdList:GetRect()
		end,
		
	    --Region:IsDragging() - True if this Region or its Parent is being dragged. - New in 3.1.0 
		["IsDragging"] = function(self)
			return self.__DdList:IsDragging()
		end,		
		
	    -- Frame:DisableDrawLayer("layer") - Disable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
        ["DisableDrawLayer"] = function(self, ...)
            return self.__DdList:DisableDrawLayer(...)
        end,
		
	    -- Frame:EnableDrawLayer("layer") - Enable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
        ["EnableDrawLayer"] = function(self, ...)
            return self.__DdList:EnableDrawLayer(...)
        end,
				
	    -- Frame:GetBackdrop() - Creates and returns a backdrop table suitable for use in SetBackdrop - New in 1.11. 
        ["GetBackdrop"] = function(self, ...)
            return self.__DdList:GetBackdrop(...)
        end,
		
	    -- Frame:SetBackdrop([backdropTable]) - Set the backdrop of the frame according to the specification provided. 
		["SetBackdrop"] = function(self, backdropTable)
			return self.__DdList:SetBackdrop(backdropTable or nil)
		end,
		
	    -- Frame:GetBackdropBorderColor() - Gets the frame's backdrop border color (r, g, b, a)- New in 1.11. 
        ["GetBackdropBorderColor"] = function(self, ...)
            return self.__DdList:GetBackdropBorderColor(...)
        end,
		
	    -- Frame:SetBackdropBorderColor(r, g, b[, a]) - Set the frame's backdrop's border's color. 
        ["SetBackdropBorderColor"] = function(self, ...)
            return self.__DdList:SetBackdropBorderColor(...)
        end,
		
	    -- Frame:GetBackdropColor() - Gets the frame's backdrop color (r, g, b, a)- New in 1.11. 
        ["GetBackdropColor"] = function(self, ...)
            return self.__DdList:GetBackdropColor(...)
        end,
		
	    -- Frame:SetBackdropColor(r, g, b[, a]) - Set the frame's backdrop color. 
        ["SetBackdropColor"] = function(self, ...)
            return self.__DdList:SetBackdropColor(...)
        end,
		
	    -- Frame:GetClampRectInsets() - Gets the modifiers to the frame's rectangle used for clamping the frame to screen. 
        ["GetClampRectInsets"] = function(self, ...)
            return self.__DdList:GetClampRectInsets(...)
        end,
		
	    -- Frame:SetClampRectInsets(left, right, top, bottom) - Modify the frame's rectangle used to prevent dragging offscreen. 
        ["SetClampRectInsets"] = function(self, ...)
            return self.__DdList:SetClampRectInsets(...)
        end,
		
	    -- Frame:GetDepth() - New in 3.0.8 
        ["GetDepth"] = function(self, ...)
            return self.__DdList:GetDepth(...)
        end,
		
	    -- Frame:SetDepth(depth) - New in 3.0.8 
        ["SetDepth"] = function(self, ...)
            return self.__DdList:SetDepth(...)
        end,
		
	    -- Frame:GetEffectiveAlpha() - Returns the effective alpha of a frame. - Since 2.1. 
        ["GetEffectiveAlpha"] = function(self, ...)
            return self.__DdList:GetEffectiveAlpha(...)
        end,
		
	    -- Frame:GetEffectiveDepth() - New in 3.0.8 
        ["GetEffectiveDepth"] = function(self, ...)
            return self.__DdList:GetEffectiveDepth(...)
        end,
		
	    -- Frame:GetEffectiveScale() - Get the scale factor of this object relative to the root window. 
        ["GetEffectiveScale"] = function(self, ...)
            return self.__DdList:GetEffectiveScale(...)
        end,
		
	    -- Frame:GetHitRectInsets() - Gets the frame's hit rectangle inset distances (l, r, t, b) - new in 1.11. 
        ["GetHitRectInsets"] = function(self, ...)
            return self.__DdList:GetHitRectInsets(...)
        end,
		
	    -- Frame:SetHitRectInsets(left, right, top, bottom) - Set the inset distances for the frame's hit rectangle - New in 1.11. 
        ["SetHitRectInsets"] = function(self, ...)
            return self.__DdList:SetHitRectInsets(...)
        end,
		
	    -- Frame:GetID() - Get the ID of this frame. 
        ["GetID"] = function(self, ...)
            return self.__DdList:GetID(...)
        end,

	    -- Frame:SetID(id) - Set the ID of this frame. 
        ["SetID"] = function(self, ...)
            return self.__DdList:SetID(...)
        end,
		
	    -- Frame:IgnoreDepth(ignoreFlag) - New in 3.0.8 
        ["IgnoreDepth"] = function(self, ...)
            return self.__DdList:IgnoreDepth(...)
        end,
		
	    -- Frame:IsIgnoringDepth() - New in 3.0.8 
        ["IsIgnoringDepth"] = function(self, ...)
            return self.__DdList:IsIgnoringDepth(...)
        end,
		
	    -- Frame:GetChildren() - Get the list of "children" (frames and things derived from frames) of this frame. 
        ["GetChildren"] = function(self, ...)
            return self.__DdList:GetChilds()
        end,
		
	    -- Frame:GetNumChildren() - Get the number of "children" (frames and things derived from frames) this frame has. 
        ["GetNumChildren"] = function(self, ...)
            return self.__DdList:GetNumChildren(...)
        end,
	}

	--	Property
	local _Property = {
		["ShowOnCursor"] = {
			["Set"] = function(self, flag)
				self = self.__DdList
				self.__ShowOnCursor = (flag and true) or false
			end,
			
			["Get"] = function(self)
				self = self.__DdList
				return (self.__ShowOnCursor and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		["MultiSelect"] = {
			["Set"] = function(self, flag)
				self = self.__DdList
				self.__MultiSelect = (flag and true) or false
			end,
			
			["Get"] = function(self)
				self = self.__DdList
				return (self.__MultiSelect and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		--- Height
		["Height"] = {
			["Get"] = function(self)
				self = self.__DdList
				return self:GetHeight()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		},
		--- Width
		["Width"] = {
			["Get"] = function(self)
				self = self.__DdList
				return self:GetWidth()
			end,

			["Type"] = IGAS:GetPropertyType("Number"),
		},
		--- Visible
		["Visible"] = {
			["Set"] = function(self, visible)
				self.__DdList.Visible = visible
			end,

			["Get"] = function(self)
				return self.__DdList.Visible
			end,

			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
				                  
		["Backdrop"] = {
			["Set"] = function(self, backdropTable)
				self:SetBackdrop(backdropTable)
			end,
			
			["Get"] = function(self)
				return self:GetBackdrop()
			end,
			
			["Type"] = IGAS:GetPropertyType("Backdrop"),
		},

		["BackdropBorderColor"] = {
			["Set"] = function(self, colorTable)
				self:SetBackdropBorderColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropBorderColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		["BackdropColor"] = {
			["Set"] = function(self, colorTable)
				self:SetBackdropColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
			end,
			
			["Get"] = function(self)
				local colorTable = {}
				colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropColor()
				return colorTable 
			end,
			
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		    		     
		["ClampRectInsets"] = {
			["Set"] = function(self, RectInset)
				self:SetClampRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,
			
			["Get"] = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetClampRectInsets()
				return RectInset 
			end,
			
			["Type"] = IGAS:GetPropertyType("RectInset"),
		},
		       		
		["HitRectInsets"] = {
			["Set"] = function(self, RectInset)
				self:SetHitRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,
			
			["Get"] = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetHitRectInsets()
				return RectInset 
			end,
			
			["Type"] = IGAS:GetPropertyType("RectInset"),
		},
		                
		["ID"] = {
			["Set"] = function(self, id)
				self:SetID(id) 
			end,
			
			["Get"] = function(self)
				return self:GetID()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
				
		["Depth"] = {
			["Set"] = function(self, depth)
				self:SetDepth(depth) 
			end,
			
			["Get"] = function(self)
				return self:GetDepth()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}

	local function _New(parent)
		local mask = CreateFrame(_Base, nil, parent)
        local frame = CreateFrame("Button", nil, _DropDownListContainer)
		frame.__Mask = mask
		mask.__DdList = frame
        
        frame.FrameStrata = "TOOLTIP"
        frame.MouseWheelEnabled = true
        frame.Visible = false
        frame:ClearAllPoints()
        frame:SetBackdrop(_FrameBackdrop)
        frame:SetBackdropBorderColor(1, 1, 1);
		frame:SetBackdropColor(0.09, 0.09, 0.19);
        frame.Height = 8
        frame.Width = 100
		frame.__ShowOnCursor = true
		frame.__MultiSelect = true
        
        frame.OnShow = OnShow
        frame.OnHide = OnHide
        frame.OnEnter = OnEnter
        frame.OnLeave = OnLeave
        
		-- Timer
		local timer = CreateFrame("Timer", "Timer", frame)
		timer.Interval = 0
		timer.OnTimer = OnTimer
        
        -- MenuLevel
        frame.__MenuLevel = 1
        
		return mask
	end

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
		-- Special Item
		["DropDownMenuButton"] = DropDownMenuButton,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
