﻿-- Author      : Kurapica,zys924

--------------------------
-- Form		        	--
--------------------------
do
	-- Check Version
	local version = 3
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Form", version) then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "Form"
	local _Base = "Frame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function _UpdateBackdrop(self)
		_FrameBackdrop.bgFile = self.__BackTexture;
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			self.__BorderWidth = 2;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			self.__BorderWidth = 2;
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			self.__BorderWidth = 2;
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:GetChild("Title"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
		self:GetChild("Title"):SetPoint("TOPRIGHT",self,"TOPRIGHT",-self.__BorderWidth,-self.__BorderWidth);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
	end

	-- Event Handlers
	local function frameOnMouseDown(self)
		if not self.Parent.InDesignMode then
			self.Parent:StartMoving();
			self.Parent:Fire("OnBeginMove");
		end
	end

	local function frameOnMouseUp(self)
		self.Parent:StopMovingOrSizing()
		self.Parent:Fire("OnEndMove");
	end

	local function sizerseOnMouseDown(self)
		if not self.Parent.InDesignMode and self:GetParent().Resizable then
			self.Parent:StartSizing("BOTTOMRIGHT")
			self.Parent:Fire("OnBeginResize");
		end
	end

	local function sizersOnMouseDown(self)
		if not self.Parent.InDesignMode and self:GetParent().Resizable then
			self.Parent:StartSizing("BOTTOM")
			self.Parent:Fire("OnBeginResize");
		end
	end

	local function sizereOnMouseDown(self)
		if not self.Parent.InDesignMode and self.Parent.Resizable then
			self.Parent:StartSizing("RIGHT")
			self.Parent:Fire("OnBeginResize");
		end
	end

	local function sizerOnMouseUp(self)
		self.Parent:StopMovingOrSizing()
		self.Parent:Fire("OnEndResize");
	end

	-- ScriptType
	local _ScriptType = {
		["OnBeginMove"] = true,
		["OnBeginResize"] = true,
		["OnEndMove"] = true,
		["OnEndResize"] = true,
	}

	--	FuncProxy
	local _FuncProxy = {	
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			
			if borderstyle == self.__BorderStyle then
				return
			end
			
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be not less than 0.")
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0
			end
			self.__BorderWidth = borderwidth
			_UpdateBackdrop(self)
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderTexture
		["GetBorderTexture"] = function(self)
			return self.__BorderTexture
		end,
		["SetBorderTexture"] = function(self, bordertexture)
			self.__BorderTexture = bordertexture
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BackTexture
		["GetBackTexture"] = function(self)
			return self.__BackTexture
		end,
		["SetBackTexture"] = function(self, backtexture)
			self.__BackTexture = backtexture or ""
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set HasTitleBar
		["GetHasTitleBar"] = function(self)
			return self:GetChild("Title"):GetChild("Back").Visible
		end,
		["SetHasTitleBar"] = function(self, hastitlebar)
			self:GetChild("Title"):GetChild("Back").Visible = (hastitlebar and true) or false
		end,
		
		-- Get/Set TitleBarColor
		["GetTitleBarColor"] = function(self)
			return self.__TitleBarColor
		end,
		["SetTitleBarColor"] = function(self, titlebarcolor)
			self.__TitleBarColor = titlebarcolor
			self:GetChild("Title"):GetChild("Back"):SetTexture(titlebarcolor.red,titlebarcolor.green,titlebarcolor.blue,titlebarcolor.alpha)
		end,
		
		-- Get/Set TitleBarHeight
		["GetTitleBarHeight"] = function(self)
			return self:GetChild("Title").Height
		end,
		["SetTitleBarHeight"] = function(self, TitleBarHeight)
			if not TitleBarHeight or type(TitleBarHeight) ~= "number" or TitleBarHeight < 10 then
				return
			end
			self:GetChild("Title"):SetHeight(TitleBarHeight)
		end,
		
		-- Get/Set TitleBarAlign
		["GetTitleBarAlign"] = function(self)
			return self:GetChild("Title"):GetChild("Text").JustifyH
		end,
		["SetTitleBarAlign"] = function(self, align)
			self:GetChild("Title"):GetChild("Text").JustifyH = align
		end,
		
		-- Get/Set FontObject
		["GetFontObject"] = function(self)
			return self:GetChild("Title"):GetChild("Text").FontObject
		end,
		["SetFontObject"] = function(self, font)
			self:GetChild("Title"):GetChild("Text").FontObject = font
		end,
		
		-- Get/Set FontColor
		["GetFontColor"] = function(self)
			return self:GetChild("Title"):GetChild("Text").TextColor
		end,
		["SetFontColor"] = function(self, fontcolor)
			self:GetChild("Title"):GetChild("Text").TextColor = fontcolor
		end,
		
		-- Get/Set HasCloseButton
		["GetHasCloseButton"] = function(self)
			return self:GetChild("Close").Visible
		end,
		["SetHasCloseButton"] = function(self, hasclosebutton)
			self:GetChild("Close").Visible = (hasclosebutton and true) or false
			if self:GetChild("Close").Visible then
				self:GetChild("Title"):GetChild("Text"):SetPoint("RIGHT",self:GetChild("Close"),"LEFT",0,0);
			else
				self:GetChild("Title"):GetChild("Text"):SetPoint("RIGHT",self:GetChild("Title"),"RIGHT",0,0);
			end
		end,
		
		-- GetCaption
		["GetCaption"] = function(self)
			return self:GetChild("Title"):GetChild("Text"):GetText();
		end,
		-- SetCaption		
		["SetCaption"] = function(self, title)
			self:GetChild("Title"):GetChild("Text"):SetText(title)
		end,
		
        -- GetMessage
        ["GetMessage"] = function(self)
            return self:GetChild("StatusBar"):GetChild("Text"):GetText() or ""
        end,
		-- SetMessage
		["SetMessage"] = function(self, mes)
			self:GetChild("StatusBar"):GetChild("Text"):SetText(mes)
		end,
		
		-- GetContentPane
		["GetContentPane"] = function(self)
			return self:GetChild("ContentPane");
		end,
	}


	--	Property
	local _Property = {
	
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BackTexture(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["BackTexture"]={
			["Get"] = function(self)
				return self:GetBackTexture();
			end,
			["Set"] = function(self, value)
				self:SetBackTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- HasTitleBar
		["HasTitleBar"]={
			["Get"] = function(self)
				return self:GetHasTitleBar();
			end,
			["Set"] = function(self, value)
				self:SetHasTitleBar(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- TitleBarAlign
		["TitleBarAlign"]={
			["Get"] = function(self)
				return self:GetTitleBarAlign();
			end,
			["Set"] = function(self, value)
				self:SetTitleBarAlign(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- TitleBarColor(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["TitleBarColor"]={
			["Get"] = function(self)
				return self:GetTitleBarColor();
			end,
			["Set"] = function(self, value)
				self:SetTitleBarColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- TitleBarHeight(default:24)
		["TitleBarHeight"]={
			["Get"] = function(self)
				return self:GetTitleBarHeight();
			end,
			["Set"] = function(self, value)
				self:SetTitleBarHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- FontObject(default:"GameFontHighlight")
		["FontObject"]={
			["Get"] = function(self)
				return self:GetFontObject();
			end,
			["Set"] = function(self, value)
				self:SetFontObject(value);
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		
		-- FontColor{red=1,green=1,blue=1,alpha=1}
		["FontColor"]={
			["Get"] = function(self)
				return self:GetFontColor();
			end,
			["Set"] = function(self, value)
				self:SetFontColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- HasCloseButton
		["HasCloseButton"]={
			["Get"] = function(self)
				return self:GetHasCloseButton();
			end,
			["Set"] = function(self, value)
				self:SetHasCloseButton(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- Caption
		["Caption"] = {
			["Set"] = function(self, title)
				self:SetCaption(title)
			end,
			["Get"] = function(self)
				return self:GetCaption()
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- Message
		["Message"] = {
			["Set"] = function(self, mes)
				self:SetMessage(mes)
			end,
			["Get"] = function(self)
				return self:GetMessage()
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("Frame",nil,parent)

		frame.Width = 400
		frame.Height = 300
		frame.Movable = true
		frame.Resizable = true
		frame.Toplevel = true

		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		frame:SetMinResize(300,200);
		
		local title = CreateFrame("Frame","Title",frame)
		title.MouseEnabled = true
		title.Height = 24
		title:SetPoint("TOPLEFT",frame,"TOPLEFT",2,-2);
		title:SetPoint("TOPRIGHT",frame,"TOPRIGHT",-2,-2);
		title.OnMouseDown = frameOnMouseDown
		title.OnMouseUp = frameOnMouseUp
		
		frame.__BorderStyle = "TOOLTIP"
		frame.__BorderWidth = 2
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8}
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND"
		frame.__BackTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND"
		frame.__BackColor = {["red"]=0,["green"]=0,["blue"]=0,["alpha"]=0.6}

		_UpdateBackdrop(frame);

		local CloseButton = CreateFrame("Button","Close",frame)
		CloseButton:SetPoint("TOPRIGHT",title,"TOPRIGHT",4,4);
		CloseButton:SetFrameLevel(title:GetFrameLevel()+1);
        CloseButton.Style = "CLOSE"
		local titletext = CreateFrame("FontString","Text",title,"OVERLAY","GameFontHighlight");
		titletext:SetPoint("LEFT",title,"LEFT",2,0);
		titletext:SetPoint("RIGHT",CloseButton,"LEFT",0,0);
		titletext:SetText("Form");
		local titleback = title:CreateTexture("Back");
		titleback:SetAllPoints();
		titleback:SetDrawLayer("ARTWORK");
		titleback:SetTexture(0.2, 0.3, 0.9, 0.9);
		
        local status = CreateFrame("Frame","StatusBar",frame)
        status.Height = 24
        status:SetPoint("BOTTOMLEFT",frame,"BOTTOMLEFT", 6, 0)
        status:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT")
        
        local statusText = CreateFrame("FontString","Text",status,"OVERLAY","GameFontNormal");
        statusText.JustifyH = "LEFT";
        statusText:SetAllPoints(status);
        statusText:SetText("");

		local sizer_se = CreateFrame("Frame","Sizer_se",frame)
		sizer_se.Width = 25
		sizer_se.Height = 25
		sizer_se.MouseEnabled = true
		sizer_se:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0)
		sizer_se.OnMouseDown = sizerseOnMouseDown
		sizer_se.OnMouseUp = sizerOnMouseUp

		local sizer_s = CreateFrame("Frame","Sizer_s",frame)
		sizer_s:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",-25,0)
		sizer_s:SetPoint("BOTTOMLEFT",frame,"BOTTOMLEFT",0,0)
		sizer_s.Height = 25
		sizer_s.MouseEnabled = true
		sizer_s.OnMouseDown = sizersOnMouseDown
		sizer_s.OnMouseUp = sizerOnMouseUp

		local sizer_e = CreateFrame("Frame","Sizer_e",frame)
		sizer_e:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,25)
		sizer_e:SetPoint("TOPRIGHT",frame,"TOPRIGHT",0,0)
		sizer_e.Width = 25
		sizer_e.MouseEnabled = true
		sizer_e.OnMouseDown = sizereOnMouseDown
		sizer_e.OnMouseUp = sizerOnMouseUp

		frame.MouseWheelEnabled = true
		frame.MouseEnabled = true

		return frame
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
