﻿-- Author      : zys924

--------------------------
-- Form		        	--
--------------------------
do
	-- Check Version
	local version = 1
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Image", version) then
		return;
	end
	local _, clientversion = GetBuildInfo();
	if tonumber(clientversion)<11159 then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "Image"
	local _Base = "ScrollFrame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function _UpdateBackdrop(self)
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
	end
	local function _UpdateImage(self)
		self:GetChild("Container"):GetChild("Image"):ClearAllPoints();
		self:GetChild("Container"):GetChild("Image"):SetTexCoordModifiesRect(not self.__Stretch);
		self:GetChild("Container"):GetChild("Image"):SetTexCoord(self.__TexCoord.left,self.__TexCoord.right,self.__TexCoord.top,self.__TexCoord.bottom);
		if self.__Stretch then
			self:GetChild("Container"):GetChild("Image"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
			self:GetChild("Container"):GetChild("Image"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth,self.__BorderWidth);
			self.__UI:SetWidth(self.__Width);
			self.__UI:SetHeight(self.__Height);
		elseif self.__AutoSize then
			self:GetChild("Container"):GetChild("Image"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
			self:GetChild("Container"):GetChild("Image"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth,self.__BorderWidth);
			self.__UI:SetWidth(self:GetChild("Container"):GetChild("Image"):GetFileWidth()+2*self.__BorderWidth);
			self.__UI:SetHeight(self:GetChild("Container"):GetChild("Image"):GetFileHeight()+2*self.__BorderWidth);
		else
			self:GetChild("Container"):GetChild("Image"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
			self:GetChild("Container"):GetChild("Image"):SetWidth(self:GetChild("Container"):GetChild("Image").__UI:GetFileWidth()+2*self.__BorderWidth);
			self:GetChild("Container"):GetChild("Image"):SetHeight(self:GetChild("Container"):GetChild("Image").__UI:GetFileHeight()+2*self.__BorderWidth);
			self:GetChild("Container"):SetWidth(self.__Width);
			self:GetChild("Container"):SetWidth(self.__Height);
		end
	end
	
	-- Event Handlers

	-- ScriptType
	local _ScriptType = {
	}

	--	FuncProxy
	local _FuncProxy = {
		
		-- Get/Set AutoSize
		["GetAutoSize"] = function(self)
			return self.__AutoSize;
		end,
		["SetAutoSize"] = function(self, autosize)
			self.__AutoSize = autosize;
			if autosize then
				self.__Stretch = false;
			end
			_UpdateImage(self);
		end,
	
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			if borderstyle == self.__BorderStyle then
				return
			end
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be no less than 0.");
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0;
			end
			self.__BorderWidth = borderwidth;
			_UpdateBackdrop(self);
			_UpdateImage(self);
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor;
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set Stretch
		["GetStretch"] = function(self)
			return self.__Stretch;
		end,
		["SetStretch"] = function(self, stretch)
			self.__Stretch = stretch;
			if stretch then
				self.__AutoSize = false;
			end
			_UpdateImage(self);
		end,
		
		-- Get/Set TexCoord
		["GetTexCoord"] = function(self)
			return self.__TexCoord;
		end,
		["SetTexCoord"] = function(self, texcoord)
			self.__TexCoord = texcoord;
			_UpdateImage(self);
		end,
		
		-- Get/Set Texture
		["GetTexture"] = function(self)
			return self:GetChild("Container"):GetChild("Image"):GetTexture();
		end,
		["SetTexture"] = function(self, texture)
			return self:GetChild("Container"):GetChild("Image"):SetTexture(texture);
		end,
		
		-- [OVERRIDE]Get/Set Width
		["GetWidth"] = function(self)
			return self.__Width;
		end,	
		["SetWidth"] = function(self, width)
			self.__Width = width;
			_UpdateImage(self);
		end,
		
		-- [OVERRIDE]Get/Set Height
		["GetHeight"] = function(self)
			return self.__Height;
		end,	
		["SetHeight"] = function(self, height)
			self.__Height = height;
			_UpdateImage(self);
		end,
	}


	--	Property
	local _Property = {
		-- AutoSize(default:true)
		["AutoSize"]={
			["Get"] = function(self)
				return self:GetAutoSize();
			end,
			["Set"] = function(self, value)
				self:SetAutoSize(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- Stretch(default:true)
		["Stretch"]={
			["Get"] = function(self)
				return self:GetStretch();
			end,
			["Set"] = function(self, value)
				self:SetStretch(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- TexCoord(default:{0,1,0,1})
		["TexCoord"]={
			["Get"] = function(self)
				return self:GetTexCoord();
			end,
			["Set"] = function(self, value)
				self:SetTexCoord(value);
			end,
			["Type"] = IGAS:GetPropertyType("Table"),
		},
		
		-- TexCoord(default:"")
		["Texture"]={
			["Get"] = function(self)
				return self:GetTexture();
			end,
			["Set"] = function(self, value)
				self:SetTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- Width
		["Width"] = {
			["Get"] = function(self)
				return self:GetWidth();
			end,
			["Set"] = function(self, value)
				self:SetWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Height
		["Height"] = {
			["Get"] = function(self)
				return self:GetHeight();
			end,
			["Set"] = function(self, value)
				self:SetHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("ScrollFrame",nil,parent);
		frame.Movable = false
		frame.Resizable = false
		frame.MouseEnabled = true
		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		frame.__AutoSize = false;
		frame.__BorderStyle = "SINGLE";
		frame.__BorderWidth = 1;
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8};
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackColor = {["red"]=0.3,["green"]=0.3,["blue"]=0.3,["alpha"]=0.6};
		frame.__Stretch = true;
		frame.__TexCoord = {["left"]=0,["right"]=1,["top"]=0,["bottom"]=1};
		frame.__Width = 100;
		frame.__Height = 25;
		
		local container = CreateFrame("Frame","Container",frame);
		frame:SetScrollChild(container);
		container:EnableMouse(nil);
		
		local texture = container:CreateTexture("Image");
		texture:SetDrawLayer("ARTWORK");
	
		_UpdateBackdrop(frame);
		_UpdateImage(frame);
		
		return frame;
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
