﻿-- Author      : Kurapica
-- ChangreLog  :
--				2010.01.13	Change the ItemList's Parent to WorldFrame

--------------------------
-- ItemButton	        --
--------------------------
do
	-- Check Version
	local version = 5

	if not IGAS:RegisterFile("IGAS.GUI.Widget.ItemButton", version) then
		return
	end

	local CreateFrame = IGAS.GUI.CreateFrame
		
	local GameTooltip = IGAS:GetWrapper(GameTooltip)
	
	-- ItemListDirection
	local _EnumDir = IGAS:NewEnum("ItemListDirection", {
		"TOP",
		"BOTTOM",
		"LEFT",
		"RIGHT",
	})
	
	local _PropDir = IGAS:NewPropertyType("ItemListDirection", {
		["Type"] = "enum",
		["EnumType"] = _EnumDir,
	})
	
	------------------------------------------------------
	---------------       ItemList           ---------------
	------------------------------------------------------
	local ItemList = (IGAS.GUI.Widget.ItemButton and IGAS.GUI.Widget.ItemButton.ItemList) or nil
	local ItemSize = 37
	do		
		local _WidgetName = "ItemList"
		local _Base = "VirtualUIObject"
		local _ItemListContainer = CreateFrame("Frame", "IGASGUI_ListContainer", WorldFrame)
		
		_ItemListContainer.__ShowList = _ItemListContainer.__ShowList or nil
		if not _ItemListContainer.__Timer then			
			local timer = CreateFrame("Timer", "Timer", _ItemListContainer)
			timer.Interval = 0
			timer.OnTimer = function(self)
				self.Interval = 0
				if self.Parent.__ShowList then
					self.Parent.__ShowList.Visible = false
				end
			end
			_ItemListContainer.__Timer = timer
		end
		
		-- Scripts		
		local function OnShow(self)			
			if _ItemListContainer.__ShowList and _ItemListContainer.__ShowList ~= self then
				_ItemListContainer.__ShowList.Visible = false
			end
			
			_ItemListContainer.__Timer.Interval = 2
			
			_ItemListContainer.__ShowList = self
		end
		
		local function OnHide(self)
            if _ItemListContainer.__ShowList and _ItemListContainer.__ShowList == self then
				_ItemListContainer.__ShowList = nil
			end
			
			_ItemListContainer.__Timer.Interval = 0
		end
				
		local function OnEnter(self)
			_ItemListContainer.__Timer.Interval = 0
		end
		
		local function OnLeave(self)
			_ItemListContainer.__Timer.Interval = 2
		end
		
		local function Item_OnEnter(self)
			self.Parent:Fire("OnEnter")
		end
		
		local function Item_OnLeave(self)
			self.Parent:Fire("OnLeave")
		end
		
		local function Item_OnClick(self)
            self.Parent.__Mask:Fire("OnClick", self.Item)
			self.Parent.Visible = false
		end
		
		local function makePosition(self, start, count)
			local btn, locX, locY
			local i, maxC = start, start + (count or 0)
			
			btn = self:GetChild("ItemButton"..i)
			
			while (not count or i < maxC) and btn do
				btn = self:GetChild("ItemButton"..i)
				if btn then
					if self.__Direction == "TOP" then
						locX = 0
						locY = ItemSize * i
					elseif self.__Direction == "BOTTOM" then
						locX = 0
						locY = - ItemSize * i
					elseif self.__Direction == "LEFT" then
						locX = - ItemSize * i
						locY = 0
					elseif self.__Direction == "RIGHT" then
						locX = ItemSize * i
						locY = 0
					end
					btn:ClearAllPoints()
					btn:SetPoint(self.__Direction, self, self.__Direction, locX, locY)
				end
				i = i + 1
				btn = self:GetChild("ItemButton"..i)
			end
		end
		
		local function setItem(self, index, link)
			local btn
			btn = self:GetChild("ItemButton"..index)
			
			if not btn then
				btn = CreateFrame("ItemButton", "ItemButton"..index, self)
				
				btn.ShowItemCount = false
				btn.ShowItemName = false
				
				btn.OnEnter = Item_OnEnter
				btn.OnLeave = Item_OnLeave
				btn.OnClick = Item_OnClick
				
				makePosition(self, index, 1)
			end
			btn.Item = link
			btn.Visible = true
		end
		
		local function makeList(self)
			local index = 1
			local item
			
			self.__Mask:Clear()
			
			if self.__FilterFunc then
				-- Set List
				--- Check Inventory Item
				for slot = 0, 19 do
					item = GetInventoryItemLink(self.__Unit, slot)
					if item and (self.__AllowSameItem or (not self.__Mask:HasItem(item))) and self.__FilterFunc(self.__Mask.Parent, item, -1, slot) then
						setItem(self, index, item)
						index = index + 1
					end
				end
				
				-- Check Container Item
				if self.__Unit == "player" then
					for i=0,NUM_BAG_FRAMES do
						for j=1,GetContainerNumSlots(i) do
							item = GetContainerItemLink(i,j)
							if item and (self.__AllowSameItem or (not self.__Mask:HasItem(item))) and self.__FilterFunc(self.__Mask.Parent, item, -1, slot) then
								setItem(self, index, item)
								index = index + 1
							end
						end
					end
				end
				
				-- A nil ItemButton
				if self.__ShowNilButton then
					setItem(self, index, nil)
					index = index + 1
				end
				
				-- Hide no-use ItemButton
				while self:GetChild("ItemButton"..index) do
					self:GetChild("ItemButton"..index).Item = nil
					self:GetChild("ItemButton"..index).Visible = false
					index = index + 1
				end
			else
				-- Hide List
				if self.__ShowNilButton then
					setItem(self, index, nil)
					index = index + 1
				end				
				
				-- Hide no-use ItemButton
				while self:GetChild("ItemButton"..index) do
					self:GetChild("ItemButton"..index).Item = nil
					self:GetChild("ItemButton"..index).Visible = false
					index = index + 1
				end
			end
		end
				
		-- ScriptType
		local _ScriptType = {
            ["OnClick"] = true,
		}
	    
		-- FuncProxy
		local _FuncProxy = {
			-- Refresh
			["Refresh"] = function(self)
				self = self.__ItemList
				makeList(self)
			end,
			-- AddItem
			["Add"] = function(self, item)
				local index = 1
				local btn
				
				self = self.__ItemList
				btn = self:GetChild("ItemButton"..index)
				while btn and btn.Visible and btn.Item do
					index = index + 1
					btn = self:GetChild("ItemButton"..index)
				end
				
				setItem(self, index, item)
				
				-- A nil ItemButton
				if self.__ShowNilButton then
					setItem(self, index, nil)
					index = index + 1
				end
			end,
			-- RemoveItem
			["Remove"] = function(self, item)
				local index = 1
				local btn, nextBtn
				
				self = self.__ItemList
				if not item then
					return
				end
				
				btn = self:GetChild("ItemButton"..index)
				while btn and btn.Visible and btn.Item ~= item do
					index = index + 1
					btn = self:GetChild("ItemButton"..index)
				end
				
				if btn and btn.Item == item then
					index = index + 1
					nextBtn = self:GetChild("ItemButton"..index)
					while nextBtn and nextBtn.Visible do
						btn.Item = nextBtn.Item
						btn = nextBtn
						index = index + 1
						nextBtn = self:GetChild("ItemButton"..index)
					end
					btn.Item = nil
					btn.Visible = false
				end
			end,
			-- ClearItem
			["Clear"] = function(self)
				local index = 1
				local btn
				
				self = self.__ItemList
				btn = self:GetChild("ItemButton"..index)
				while btn and btn.Visible do
					btn.Item = nil
					btn.Visible = false
					
					index = index + 1
					btn = self:GetChild("ItemButton"..index)
				end
				
				-- A nil ItemButton
				if self.__ShowNilButton then
					setItem(self, 1, nil)
				end
			end,
			-- HasItem
			["HasItem"] = function(self, item)
				local index = 1
				local btn, nextBtn
				
				self = self.__ItemList
				if not item then
					return
				end
				
				btn = self:GetChild("ItemButton"..index)
				while btn and btn.Visible and btn.Item ~= item do
					index = index + 1
					btn = self:GetChild("ItemButton"..index)
				end
				
				if btn and btn.Item == item then
					return true
				else
					return false
				end
			end,
			
			--- Original Functions
			-- ClearAllPoints() - Clear all attachment points for this object. 
			["ClearAllPoints"] = function(self)
				self.__ItemList:ClearAllPoints()
			end,

			-- GetBottom() - Get the y location of the bottom edge of this frame
			["GetBottom"] = function(self)
				return self.__ItemList:GetBottom()
			end,
			
			-- GetCenter() - Get the coordinates of the center of this frame
			["GetCenter"] = function(self)
				return self.__ItemList:GetCenter()
			end,
			
			-- GetHeight() - Get the height of this object. 
			["GetHeight"] = function(self)
				return self.__ItemList:GetHeight()
			end,
					
			-- GetLeft() - Get the x location of the left edge of this frame
			["GetLeft"] = function(self)
				return self.__ItemList:GetLeft()
			end,
			
			-- GetNumPoints() - Get the number of anchor points for this frame
			["GetNumPoints"] = function(self)
				return self.__ItemList:GetNumPoints()
			end,
			
			-- GetPoint(pointNum) - Get details for an anchor point for this frame (point, relativeTo, relativePoint, xofs, yofs)
			["GetPoint"] = function(self, pointNum)
				return self.__ItemList:GetPoint(pointNum)
			end,
			
			-- GetRight() - Get the x location of the right edge of this frame
			["GetRight"] = function(self)
				return self.__ItemList:GetRight()
			end,
			
			-- GetTop() - Get the y location of the top edge of this frame
			["GetTop"] = function(self)
				return self.__ItemList:GetTop()
			end,
			
			-- GetWidth() - Get the width of this object. 
			["GetWidth"] = function(self)
				return self.__ItemList:GetWidth()
			end,
					
			-- Hide() - Set this object to hidden (it and all of its children will disappear). 
			["Hide"] = function(self)
				return self.__ItemList:Hide()
			end,
			
			-- Show() - Set this object to shown (it will appear if its parent is visible). 
			["Show"] = function(self)
				self.__ItemList:Show()
			end,
					
			-- IsShown() - Determine if this object is shown (would be visible if its parent was visible). 
			["IsShown"] = function(self)
				return self.__ItemList:IsShown()
			end,
			
			-- IsVisible() - Get whether the object is visible on screen (logically (IsShown() and GetParent():IsVisible())); 
			["IsVisible"] = function(self)
				return self.__ItemList:IsVisible()
			end,
			
			-- SetAllPoints(frame or "frameName") - Set all anchors to match edges of specified frame
			["SetAllPoints"] = function(self, frame)
				if frame and type(frame) == "string" then
					frame = _G[frame]
				end
				
				if not frame or type(frame) ~= "table" then
					frame = self.Parent
				end
							
				self.__ItemList:SetAllPoints(frame)
			end,
			
			-- SetPoint("point","relativeFrame" or relativeObject,"relativePoint"[, xOfs, yOfs]) - Set an attachment point of this object - Updated in 1.10. Since 2.2 cooordinates are now relative to the closest anchor point 
			["SetPoint"] = function(self, point, relativeObject, relativePoint, xOfs, yOfs)
				self.__ItemList:SetPoint(point, relativeObject, relativePoint, xOfs, yOfs)
			end,
			
			-- Dispose, release resource
			["Dispose"] = function(self)
				self.__ItemList:Dispose()
				
				-- Call super's dispose
				self:CallMethod(_Base, "Dispose")
			end,

			--Region:GetRect() - Get frame's left, bottom, width, height. 
			["GetRect"] = function(self)
				return self.__ItemList:GetRect()
			end,
			
			--Region:IsDragging() - True if this Region or its Parent is being dragged. - New in 3.1.0 
			["IsDragging"] = function(self)
				return self.__ItemList:IsDragging()
			end,		
			
			-- Frame:DisableDrawLayer("layer") - Disable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
			["DisableDrawLayer"] = function(self, ...)
				return self.__ItemList:DisableDrawLayer(...)
			end,
			
			-- Frame:EnableDrawLayer("layer") - Enable rendering of "regions" (fontstrings, textures) in the specified draw layer. 
			["EnableDrawLayer"] = function(self, ...)
				return self.__ItemList:EnableDrawLayer(...)
			end,
					
			-- Frame:GetBackdrop() - Creates and returns a backdrop table suitable for use in SetBackdrop - New in 1.11. 
			["GetBackdrop"] = function(self, ...)
				return self.__ItemList:GetBackdrop(...)
			end,
			
			-- Frame:SetBackdrop([backdropTable]) - Set the backdrop of the frame according to the specification provided. 
			["SetBackdrop"] = function(self, backdropTable)
				return self.__ItemList:SetBackdrop(backdropTable or nil)
			end,
			
			-- Frame:GetBackdropBorderColor() - Gets the frame's backdrop border color (r, g, b, a)- New in 1.11. 
			["GetBackdropBorderColor"] = function(self, ...)
				return self.__ItemList:GetBackdropBorderColor(...)
			end,
			
			-- Frame:SetBackdropBorderColor(r, g, b[, a]) - Set the frame's backdrop's border's color. 
			["SetBackdropBorderColor"] = function(self, ...)
				return self.__ItemList:SetBackdropBorderColor(...)
			end,
			
			-- Frame:GetBackdropColor() - Gets the frame's backdrop color (r, g, b, a)- New in 1.11. 
			["GetBackdropColor"] = function(self, ...)
				return self.__ItemList:GetBackdropColor(...)
			end,
			
			-- Frame:SetBackdropColor(r, g, b[, a]) - Set the frame's backdrop color. 
			["SetBackdropColor"] = function(self, ...)
				return self.__ItemList:SetBackdropColor(...)
			end,
			
			-- Frame:GetClampRectInsets() - Gets the modifiers to the frame's rectangle used for clamping the frame to screen. 
			["GetClampRectInsets"] = function(self, ...)
				return self.__ItemList:GetClampRectInsets(...)
			end,
			
			-- Frame:SetClampRectInsets(left, right, top, bottom) - Modify the frame's rectangle used to prevent dragging offscreen. 
			["SetClampRectInsets"] = function(self, ...)
				return self.__ItemList:SetClampRectInsets(...)
			end,
			
			-- Frame:GetDepth() - New in 3.0.8 
			["GetDepth"] = function(self, ...)
				return self.__ItemList:GetDepth(...)
			end,
			
			-- Frame:SetDepth(depth) - New in 3.0.8 
			["SetDepth"] = function(self, ...)
				return self.__ItemList:SetDepth(...)
			end,
			
			-- Frame:GetEffectiveAlpha() - Returns the effective alpha of a frame. - Since 2.1. 
			["GetEffectiveAlpha"] = function(self, ...)
				return self.__ItemList:GetEffectiveAlpha(...)
			end,
			
			-- Frame:GetEffectiveDepth() - New in 3.0.8 
			["GetEffectiveDepth"] = function(self, ...)
				return self.__ItemList:GetEffectiveDepth(...)
			end,
			
			-- Frame:GetEffectiveScale() - Get the scale factor of this object relative to the root window. 
			["GetEffectiveScale"] = function(self, ...)
				return self.__ItemList:GetEffectiveScale(...)
			end,
			
			-- Frame:GetHitRectInsets() - Gets the frame's hit rectangle inset distances (l, r, t, b) - new in 1.11. 
			["GetHitRectInsets"] = function(self, ...)
				return self.__ItemList:GetHitRectInsets(...)
			end,
			
			-- Frame:SetHitRectInsets(left, right, top, bottom) - Set the inset distances for the frame's hit rectangle - New in 1.11. 
			["SetHitRectInsets"] = function(self, ...)
				return self.__ItemList:SetHitRectInsets(...)
			end,
			
			-- Frame:GetID() - Get the ID of this frame. 
			["GetID"] = function(self, ...)
				return self.__ItemList:GetID(...)
			end,

			-- Frame:SetID(id) - Set the ID of this frame. 
			["SetID"] = function(self, ...)
				return self.__ItemList:SetID(...)
			end,
			
			-- Frame:IgnoreDepth(ignoreFlag) - New in 3.0.8 
			["IgnoreDepth"] = function(self, ...)
				return self.__ItemList:IgnoreDepth(...)
			end,
			
			-- Frame:IsIgnoringDepth() - New in 3.0.8 
			["IsIgnoringDepth"] = function(self, ...)
				return self.__ItemList:IsIgnoringDepth(...)
			end,
			
			-- Frame:GetChildren() - Get the list of "children" (frames and things derived from frames) of this frame. 
			["GetChildren"] = function(self, ...)
				return self.__ItemList:GetChilds()
			end,
			
			-- Frame:GetNumChildren() - Get the number of "children" (frames and things derived from frames) this frame has. 
			["GetNumChildren"] = function(self, ...)
				return self.__ItemList:GetNumChildren(...)
			end,
	    }
	    
		-- Property
		local _Property = {
			-- Filter
			["Filter"] = {
				["Set"] = function(self, func)
					self = self.__ItemList
                    if func and type(func) ~= "function" then
						error("The parameter must be a function.", 2)
					end
					self.__FilterFunc = func
                end,
                
                ["Get"] = function(self)
					self = self.__ItemList
                    return self.__FilterFunc
                end,
                
                ["Type"] = IGAS:GetPropertyType("Function"),
			},
			-- Direction
			["Direction"] = {
				["Set"] = function(self, dir)					
					self = self.__ItemList
					
					if not _EnumDir[dir] then
						error("The Direction is incorrect."..dir, 2)
					end
					self.__Direction = dir
					
					makePosition(self, 1)
                end,
                
                ["Get"] = function(self)
					self = self.__ItemList
                    return self.__Direction
                end,
                
                ["Type"] = _PropDir,
			},
			-- Unit
			["Unit"] = {
				["Set"] = function(self, unit)
					self = self.__ItemList
					self.__Unit = unit
                end,
                
                ["Get"] = function(self)
					self = self.__ItemList
					return self.__Unit
                end,
                
                ["Type"] = IGAS:GetPropertyType("String"),
			},
			-- ShowNilButton
			["ShowNilButton"] = {
				["Set"] = function(self, flag)
					self = self.__ItemList
					self.__ShowNilButton = (flag and true) or false
                end,
                
                ["Get"] = function(self)
					self = self.__ItemList
					return (self.__ShowNilButton and true) or false
                end,
                
                ["Type"] = IGAS:GetPropertyType("Boolean"),
			},
			-- AllowSameItem
			["AllowSameItem"] = {
				["Set"] = function(self, flag)
					self = self.__ItemList
					self.__AllowSameItem = (flag and true) or false
                end,
                
                ["Get"] = function(self)
					self = self.__ItemList
					return (self.__AllowSameItem and true) or false
                end,
                
                ["Type"] = IGAS:GetPropertyType("Boolean"),
			},
			--- Visible
			["Visible"] = {
				["Set"] = function(self, visible)
					self.__ItemList.Visible = visible
				end,

				["Get"] = function(self)
					return self.__ItemList.Visible
				end,

				["Type"] = IGAS:GetPropertyType("Boolean"),
			},

			["Backdrop"] = {
				["Set"] = function(self, backdropTable)
					self:SetBackdrop(backdropTable)
				end,
				
				["Get"] = function(self)
					return self:GetBackdrop()
				end,
				
				["Type"] = IGAS:GetPropertyType("Backdrop"),
			},

			["BackdropBorderColor"] = {
				["Set"] = function(self, colorTable)
					self:SetBackdropBorderColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
				end,
				
				["Get"] = function(self)
					local colorTable = {}
					colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropBorderColor()
					return colorTable 
				end,
				
				["Type"] = IGAS:GetPropertyType("Color"),
			},
			
			["BackdropColor"] = {
				["Set"] = function(self, colorTable)
					self:SetBackdropColor(colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha)
				end,
				
				["Get"] = function(self)
					local colorTable = {}
					colorTable.red, colorTable.green, colorTable.blue, colorTable.alpha = self:GetBackdropColor()
					return colorTable 
				end,
				
				["Type"] = IGAS:GetPropertyType("Color"),
			},

			["ClampRectInsets"] = {
				["Set"] = function(self, RectInset)
					self:SetClampRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
				end,
				
				["Get"] = function(self)
					local RectInset = {}
					RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetClampRectInsets()
					return RectInset 
				end,
				
				["Type"] = IGAS:GetPropertyType("RectInset"),
			},

			["HitRectInsets"] = {
				["Set"] = function(self, RectInset)
					self:SetHitRectInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
				end,
				
				["Get"] = function(self)
					local RectInset = {}
					RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetHitRectInsets()
					return RectInset 
				end,
				
				["Type"] = IGAS:GetPropertyType("RectInset"),
			},
			
			["Depth"] = {
				["Set"] = function(self, depth)
					self:SetDepth(depth) 
				end,
				
				["Get"] = function(self)
					return self:GetDepth()
				end,
				
				["Type"] = IGAS:GetPropertyType("Number"),
			},
		}
		
	    -- New      
	    local _New = function(parent)
			local mask = CreateFrame(_Base, nil, parent)
			
			local frame = CreateFrame("Frame", nil, _ItemListContainer)
			mask.__ItemList = frame
			frame.__Mask = mask
						
			frame.FrameStrata = "TOOLTIP"
			frame.Height = ItemSize
			frame.Width = ItemSize
            frame.Visible = false
			
			frame.__Direction = "BOTTOM"
			frame.__Unit = "player"
			frame.__ShowNilButton = true
						
			frame.OnShow = OnShow
			frame.OnHide = OnHide
			frame.OnEnter = OnEnter
			frame.OnLeave = OnLeave
			
	        return mask
	    end
		
	    local _WidgetInfo = {
	        ["WidgetName"] = _WidgetName,
	        ["Base"] = _Base,
	        ["FuncProxy"] = _FuncProxy,
	        ["Property"] = _Property,
	        ["ScriptType"] = _ScriptType,
	        ["New"] = _New,
	    }
    
		ItemList = IGAS:NewWidget(_WidgetInfo, ItemList)
	end
	
	------------------------------------------------------
	---------------       ItemButton          ------------
	------------------------------------------------------
	local _WidgetName = "ItemButton"
	local _Base = "Button"

	-- Script
	-- OnClick Script
	local function OnClick(self, ...)		
		if self.__ItemLink then
			if ( IsModifiedClick() ) then
				return HandleModifiedItemClick(self.__ItemLink)
			end
		end
		
		if self:GetChild("ItemList") then
			self:GetChild("ItemList").Visible = not self:GetChild("ItemList").Visible
			return
		end		
	end
		
	local function OnEnter(self, ...)		
		if self.__ItemLink then
			GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
			GameTooltip:SetHyperlink(self.__ItemLink)
			GameTooltip:Show()
		end
	end
		
	local function OnLeave(self, ...)		
		GameTooltip:ClearLines()
		GameTooltip:Hide()
	end
    
	local function OnHide(self, ...)		
		if self:GetChild("ItemList") and self:GetChild("ItemList").Visible then
			self:GetChild("ItemList"):Hide()
		end
	end
	
	local function SetItemTexture(self, texture)
		if ( texture ) then
			self:GetChild("Icon"):Show()
		else
			self:GetChild("Icon"):Hide()
		end
		self:GetChild("Icon"):SetTexture(texture)
	end
	
	local function SetItemName(self, name)
		self:GetChild("ItemName"):SetText(name)
	end
	
	local function SetItemCount(self, count)			
		if ( not count ) then
			count = 0
		end

		self.__Count = count
		if ( count > 1  ) then
			if ( count > 999 ) then
				count = "*"
			end
			self:GetChild("Count"):SetText(count)
			if self.__ShowItemCount then
				self:GetChild("Count"):Show()
			end
		else
			self:GetChild("Count"):Hide()
		end
	end
	
    local function list_OnClick(self, item)
        self.Parent.Item = item
    end
    
	local function GetItemList(self)
		if self:GetChild("ItemList") then
			return self:GetChild("ItemList")
		end
		local itemL = CreateFrame(ItemList, "ItemList", self)
		itemL:SetPoint("CENTER", self, "CENTER")
        itemL.OnClick = list_OnClick
        return itemL
	end
	
	-- ScriptType
	local _ScriptType = {
		["OnItemChanged"] = true,
	}	
	
	--	FuncProxy
	local _FuncProxy = {
		-- SetItemCount
		["SetItemCount"] = function(self, count)			
			SetItemCount(self, count)
		end,
		-- SetItemStock
		["SetItemStock"] = function(self, numInStock)			
			if ( not numInStock ) then
				numInStock = 0
			end

			self.__Stock = numInStock;
			if ( numInStock > 0 ) then
				self:GetChild("Stock"):SetFormattedText(MERCHANT_STOCK, numInStock);
				self:GetChild("Stock"):Show()
			else
				self:GetChild("Stock"):Hide()
			end
		end,
		-- SetItemTexture
		["SetItemTexture"] = function(self, texture)
			SetItemTexture(self, texture)
		end,
		-- SetItemTextureVertexColor
		["SetItemTextureVertexColor"] = function(self, r, g, b)			
			self:GetChild("Icon"):SetVertexColor(r, g, b)
		end,
		-- SetItemDesaturated
		["SetItemDesaturated"] = function(self, desaturated, r, g, b)
			local icon = self:GetChild("Icon")
			if ( not icon ) then
				return
			end
			local shaderSupported = icon:SetDesaturated(desaturated);

			if ( not desaturated ) then
				r = 1.0;
				g = 1.0;
				b = 1.0;
			elseif ( not r or not shaderSupported ) then
				r = 0.5;
				g = 0.5;
				b = 0.5;
			end
			
			icon:SetVertexColor(r, g, b)
		end,
		-- SetItemNormalTextureVertexColor
		["SetItemNormalTextureVertexColor"] = function(self, r, g, b)			
			self:GetChild("NormalTexture"):SetVertexColor(r, g, b);
		end,
		-- SetItemName
		["SetItemName"] = function(self, name)
			SetItemName(self, name)
		end,
		
		--- Special Item Set
		-- SetContainerItem
		["SetItem"] = function(self, link)
			local itemName, itemTexture, itemCount
			if not link then
				-- Clear
				self.__ItemLink = nil
				SetItemName(self, "")
				SetItemTexture(self, nil)
				if self.__ShowItemCount then
					SetItemCount(self, 0)
				end
				self:Fire("OnItemChanged")
				return
			end
			-- Set
			itemName, _, _, _, _, _, _, _, _, itemTexture = GetItemInfo(link)
			if itemName then
				if link ~= self.__ItemLink then
					self.__ItemLink = link
					SetItemName(self, itemName)
					SetItemTexture(self, itemTexture)
					if self.__ShowItemCount then
						itemCount = GetItemCount(link, self.__BankItemIncluded)
						SetItemCount(self, itemCount)
					end
					self:Fire("OnItemChanged")
				end
			else
				error("The parameter you passed in is not an item.", 2)
			end
		end,
		-- GetItemLink
		["GetItem"] = function(self)
			return self.__ItemLink
		end,
		--- Item List
		-- RefreshList
		["RefreshList"] = function(self, filterFunc)
			local list = GetItemList(self)
			
			if filterFunc and type(filterFunc) == "function" then
				list.Filter = filterFunc
			end
			list:Refresh()
		end,
		-- AddListItem
		["AddListItem"] = function(self, item)
			local list = GetItemList(self)
			
			list:Add(item)
		end,
		-- RemoveListItem
		["RemoveListItem"] = function(self, item)			
			local list = GetItemList(self)
			
			list:Remove(item)
		end,
		-- ClearList
		["ClearList"] = function(self)					
			local list = GetItemList(self)
			
			list:Clear()
		end,
		-- RemoveList
		["RemoveList"] = function(self)
			if self:GetChild("ItemList") then
				self:GetChild("ItemList"):Dispose()
			end
		end,
	}

	--	Property
	local _Property = {
		-- ShowItemCount
		["ShowItemCount"] = {
			["Set"] = function(self, flag)
				self.__ShowItemCount = (flag and true) or false
			end,
			
			["Get"] = function(self)
				return (self.__ShowItemCount and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		-- BankItemIncluded
		["BankItemIncluded"] = {
			["Set"] = function(self, flag)
				self.__BankItemIncluded = (flag and true) or false
			end,
			
			["Get"] = function(self)
				return (self.__BankItemIncluded and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		-- ItemCount
		["ItemCount"] = {
			["Set"] = function(self, count)
				self:SetItemCount(count)
			end,
			
			["Get"] = function(self)
				return self.__Count or 0
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		-- ItemStock
		["ItemStock"] = {
			["Set"] = function(self, numInStock)
				self:SetItemStock(numInStock)
			end,
			
			["Get"] = function(self)
				return self.__Stock or 0
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		-- ItemTexture
		["ItemTexture"] = {
			["Set"] = function(self, texture)				
				self:SetItemTexture(texture)
			end,
			
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
		-- ShowItemName
		["ShowItemName"] = {
			["Set"] = function(self, flag)
				self.__ShowItemName = (flag and true) or false
				self:GetChild("ItemName").Visible = self.__ShowItemName
			end,
			
			["Get"] = function(self)
				return (self.__ShowItemName and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		-- ItemName
		["ItemName"] = {
			["Set"] = function(self, name)				
				self:SetItemName(name)
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
		-- Item
		["Item"] = {
			["Set"] = function(self, link)
				self:SetItem(link)
			end,
			
			["Get"] = function(self)
				return self:GetItem()
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
		--- Item List
		-- Direction
		["ListDirection"] = {
			["Set"] = function(self, dir)
				local list = GetItemList(self)
				
				list.Direction = dir
			end,
			
			["Get"] = function(self)
				local list = GetItemList(self)
				
				return list.Direction
			end,
			
			["Type"] = _PropDir,
		},
		-- Unit
		["ListUnit"] = {
			["Set"] = function(self, unit)
				local list = GetItemList(self)
				
				list.Unit = unit
			end,
			
			["Get"] = function(self)
				local list = GetItemList(self)
				return list.Unit
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
		-- ListShowNilButton
		["ListShowNilButton"] = {
			["Set"] = function(self, flag)
				local list = GetItemList(self)
				list.ShowNilButton = flag
			end,
			
			["Get"] = function(self)
				local list = GetItemList(self)
				return list.ShowNilButton
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
	}

	local function _New(parent)		
        local btn = CreateFrame("Button", nil, parent)
        
        btn.Width = 37
        btn.Height = 37
        
		-- Texture
        local t = CreateFrame("Texture", "NormalTexture", btn)
        t:SetTexture("Interface\\Buttons\\UI-Quickslot2")
        t.Height = 64
        t.Width = 64
        t:SetPoint("CENTER", btn, "CENTER", 0, -1)
        btn:SetNormalTexture(t)
        btn:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
        btn:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square")
        t = btn:GetHighlightTexture()
        t:SetBlendMode("ADD")
        
		-- Icon
        local icon = CreateFrame("Texture", "Icon", btn, "BORDER")
        icon:SetAllPoints(btn)
        
		-- Count
        local count = CreateFrame("FontString", "Count", btn, "BORDER", "NumberFontNormal")
        count.JustifyH = "RIGHT"
        count.Visible = false
        count:SetPoint("BOTTOMRIGHT", btn, "BOTTOMRIGHT", -5, 2)
        
        local stock = CreateFrame("FontString", "Stock", btn, "BORDER", "NumberFontNormalYellow")
        stock.JustifyH = "LEFT"
        stock.Visible = false
        stock:SetPoint("TOPLEFT", btn, "TOPLEFT", 0, -2)
        
		-- ItemName
		local itemName = CreateFrame("FontString", "ItemName", btn, "OVERLAY", "GameFontHighlightSmallOutline")
		itemName.Width = 36
		itemName.Height = 10
		itemName.Visible = false
        itemName:SetPoint("BOTTOM", btn, "BOTTOM", 0, 2)
		
		btn.__ShowItemCount = true
		btn.__ShowItemName = false
		
		btn:StackScript("OnClick", OnClick)
		btn:StackScript("OnEnter", OnEnter)
		btn:StackScript("OnLeave", OnLeave)
        btn:StackScript("OnHide", OnHide)
		
		return btn		
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
		-- Special Item
		["ItemList"] = ItemList,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
