﻿-- Author      : zys924

--------------------------
-- Label	        	--
--------------------------
do
	-- Check Version
	local version = 3
	if not IGAS:RegisterFile("IGAS.GUI.Widget.Label", version) then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "Label"
	local _Base = "ScrollFrame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function _UpdateBackdrop(self)
		_FrameBackdrop.bgFile = self.__BackTexture;
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
	end

	local function _UpdateLabel(self)
		self:GetChild("Container"):GetChild("Text"):ClearAllPoints();
		if self.__AutoSize then
			if self.__AutoWrap then
				self.__UI:SetWidth(self.__Width);
				self:GetChild("Container"):GetChild("Text"):SetNonSpaceWrap(true);
				self:GetChild("Container"):GetChild("Text"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
				self:GetChild("Container"):GetChild("Text"):SetWidth(self.__Width - self.__BorderWidth*2);
				self:GetChild("Container"):GetChild("Text"):SetHeight(5000);
				self:GetChild("Container"):GetChild("Text"):SetJustifyH(self.__JustifyH);
				self:GetChild("Container"):GetChild("Text"):SetJustifyV("TOP");
				self.__Height = self:GetChild("Container"):GetChild("Text"):GetStringHeight() + 2*self.__BorderWidth;
				self.__UI:SetHeight(self.__Height);
			else
				self:GetChild("Container"):GetChild("Text"):SetNonSpaceWrap(false);
				self:GetChild("Container"):GetChild("Text"):SetPoint("CENTER",self,"CENTER",0,0);
				self:GetChild("Container"):GetChild("Text"):SetJustifyH("CENTER");
				self:GetChild("Container"):GetChild("Text"):SetJustifyV("MIDDLE");
				self:GetChild("Container"):GetChild("Text"):SetWidth(5000);
				self:GetChild("Container"):GetChild("Text"):SetHeight(self:GetChild("Container"):GetChild("Text"):GetStringHeight());
				self.__Width = self:GetChild("Container"):GetChild("Text"):GetStringWidth() + 2*self.__BorderWidth;
				self.__Height = self:GetChild("Container"):GetChild("Text"):GetStringHeight() + 2*self.__BorderWidth;
				self.__UI:SetWidth(self.__Width);
				self.__UI:SetHeight(self.__Height);
			end
		else
			self:GetChild("Container"):GetChild("Text"):SetJustifyH(self.__JustifyH);
			self:GetChild("Container"):GetChild("Text"):SetJustifyV(self.__JustifyV);
			if self.__JustifyH == "LEFT" then
				if self.__JustifyV == "TOP" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
				elseif self.__JustifyV == "MIDDLE" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("LEFT",self,"LEFT",self.__BorderWidth,0);
				else
					self:GetChild("Container"):GetChild("Text"):SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT",self.__BorderWidth,self.__BorderWidth);
				end
			elseif self.__JustifyH == "CENTER" then
				if self.__JustifyV == "TOP" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("TOP",self,"TOP",0,-self.__BorderWidth);
				elseif self.__JustifyV == "MIDDLE" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("CENTER",self,"CENTER",0,0);
				else
					self:GetChild("Container"):GetChild("Text"):SetPoint("BOTTOM",self,"BOTTOM",0,self.__BorderWidth);
				end
			else
				if self.__JustifyV == "TOP" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("TOPRIGHT",self,"TOPRIGHT",-self.__BorderWidth,-self.__BorderWidth);
				elseif self.__JustifyV == "MIDDLE" then
					self:GetChild("Container"):GetChild("Text"):SetPoint("RIGHT",self,"RIGHT",-self.__BorderWidth,0);
				else
					self:GetChild("Container"):GetChild("Text"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth,self.__BorderWidth);
				end
			end
			self:GetChild("Container"):GetChild("Text"):SetNonSpaceWrap(self.__AutoWrap);
			if self.__AutoWrap then
				self:GetChild("Container"):GetChild("Text"):SetWidth(self.__Width - 2*self.__BorderWidth);
			else
				self:GetChild("Container"):GetChild("Text"):SetWidth(65535);
			end
			self:GetChild("Container"):GetChild("Text"):SetHeight(65535);
			self.__UI:SetWidth(self.__Width);
			self.__UI:SetHeight(self.__Height);
		end
	end
	-- Event Handlers

	-- ScriptType
	local _ScriptType = {
	}

	--	FuncProxy
	local _FuncProxy = {
		
		-- Get/Set AutoSize
		["GetAutoSize"] = function(self)
			return self.__AutoSize;
		end,
		["SetAutoSize"] = function(self, autosize)
			self.__AutoSize = autosize;
			_UpdateLabel(self);
		end,
		
		-- Get/Set AutoWrap
		["GetAutoWrap"] = function(self)
			return self.__AutoWrap;
		end,
		["SetAutoWrap"] = function(self, autowrap)
			self.__AutoWrap = autowrap;
			_UpdateLabel(self);
		end,
	
		-- Get/Set BackTexture
		["GetBackTexture"] = function(self)
			return self.__BackTexture;
		end,
		["SetBackTexture"] = function(self, backtexture)
			self.__BackTexture = backtexture;
			_UpdateBackdrop(self);
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			if borderstyle == self.__BorderStyle then
				return
			end
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be no less than 0.");
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0;
			end
			self.__BorderWidth = borderwidth;
			_UpdateBackdrop(self);
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor;
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderTexture
		["GetBorderTexture"] = function(self)
			return self.__BorderTexture
		end,
		["SetBorderTexture"] = function(self, bordertexture)
			self.__BorderTexture = bordertexture
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set FontObject
		["GetFontObject"] = function(self)
			return self:GetChild("Title"):GetChild("Text").FontObject
		end,
		["SetFontObject"] = function(self, font)
			self:GetChild("Title"):GetChild("Text").FontObject = font
		end,
		
		-- Get/Set FontColor
		["GetFontColor"] = function(self)
			return self:GetChild("Title"):GetChild("Text").TextColor
		end,
		["SetFontColor"] = function(self, fontcolor)
			self:GetChild("Title"):GetChild("Text").TextColor = fontcolor
		end,
		
		-- Get/Set Caption
		["GetCaption"] = function(self)
			return self:GetChild("Container"):GetChild("Text"):GetText();
		end,	
		["SetCaption"] = function(self, title)
			self:GetChild("Container"):GetChild("Text"):SetText(title);
			_UpdateLabel(self);
		end,
		
		-- Get/Set JustifyH
		["GetJustifyH"] = function(self)
			return self.__JustifyH;
		end,
		["SetJustifyH"] = function(self, justifyh)
			self.__JustifyH = justifyh;
			_UpdateLabel(self);
		end,
		
		-- Get/Set JustifyV
		["GetJustifyV"] = function(self)
			return self.__JustifyV;
		end,
		["SetJustifyV"] = function(self, justifyv)
			self.__JustifyV = justifyv;
			_UpdateLabel(self);
		end,
		
		-- [OVERRIDE]Get/Set Width
		["GetWidth"] = function(self)
			return self.__Width;
		end,	
		["SetWidth"] = function(self, width)
			self.__Width = width;
			_UpdateLabel(self);
		end,
		
		-- [OVERRIDE]Get/Set Height
		["GetHeight"] = function(self)
			return self.__Height;
		end,	
		["SetHeight"] = function(self, height)
			self.__Height = height;
			_UpdateLabel(self);
		end,
	}


	--	Property
	local _Property = {
		-- AutoSize(default:true)
		["AutoSize"]={
			["Get"] = function(self)
				return self:GetAutoSize();
			end,
			["Set"] = function(self, value)
				self:SetAutoSize(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- AutoWrap(default:false)
		["AutoWrap"]={
			["Get"] = function(self)
				return self:GetAutoWrap();
			end,
			["Set"] = function(self, value)
				self:SetAutoWrap(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- BackTexture(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["BackTexture"]={
			["Get"] = function(self)
				return self:GetBackTexture();
			end,
			["Set"] = function(self, value)
				self:SetBackTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},

		-- FontObject(default:"GameFontHighlight")
		["FontObject"]={
			["Get"] = function(self)
				return self:GetFontObject();
			end,
			["Set"] = function(self, value)
				self:SetFontObject(value);
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		
		-- FontColor{red=1,green=1,blue=1,alpha=1}
		["FontColor"]={
			["Get"] = function(self)
				return self:GetFontColor();
			end,
			["Set"] = function(self, value)
				self:SetFontColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- Caption
		["Caption"] = {
			["Get"] = function(self)
				return self:GetCaption()
			end,
			["Set"] = function(self, title)
				self:SetCaption(title)
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},

		-- JustifyH
		["JustifyH"] = {
			["Get"] = function(self)
				return self:GetJustifyH()
			end,
			["Set"] = function(self, value)
				self:SetJustifyH(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyH"),
		},
		
		-- JustifyV
		["JustifyV"] = {
			["Get"] = function(self)
				return self:GetJustifyV()
			end,
			["Set"] = function(self, value)
				self:SetJustifyV(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyV"),
		},
		
		-- Width
		["Width"] = {
			["Get"] = function(self)
				return self:GetWidth();
			end,
			["Set"] = function(self, value)
				self:SetWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Height
		["Height"] = {
			["Get"] = function(self)
				return self:GetHeight();
			end,
			["Set"] = function(self, value)
				self:SetHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("ScrollFrame",nil,parent);
		frame.Movable = false
		frame.Resizable = false
		frame.MouseEnabled = true
		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		frame.__JustifyH = "LEFT";
		frame.__JustifyV = "MIDDLE";
		frame.__AutoSize = true;
		frame.__AutoWrap = false;
		frame.__BorderStyle = "SINGLE";
		frame.__BorderWidth = 1;
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8};
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackColor = {["red"]=0.3,["green"]=0.3,["blue"]=0.3,["alpha"]=0.6};
		frame.__Width = 100;
		frame.__Height = 25;
		
		local container = CreateFrame("Frame","Container",frame);
		frame:SetScrollChild(container);
		container:EnableMouse(nil);
		
		local titletext = CreateFrame("FontString","Text",container,"OVERLAY","GameFontHighlight");
		titletext:SetDrawLayer("ARTWORK");
		titletext:SetText("label1");
	
		_UpdateBackdrop(frame);
		_UpdateLabel(frame);
		
		return frame;
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
