--[[
	Widget name	  	List
	Author     	 	Kurapica
	Create Date	 	12/07/2009
	ChangeLog
		2010.01.09	Add icons and frames settings
		2010/02/08  remove the list FrameStrata settings
		2010/02/12  Add OnSizeChanged script
		2010/03/03	Add OnItemDoubleClick script
--]]

-- List

do
	-- Check Version
	local version = 5
	if not IGAS:RegisterFile("IGAS.GUI.Widget.List", version) then
		return
	end
    
    local CreateFrame = IGAS.GUI.CreateFrame
	
	local _WidgetName = "List"
	local _Base = "Frame"
	local _Height = 26

	-- Script
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function Item_OnEnter(self)
		self.Parent:Fire("OnEnter")
	end
	
	local function Item_OnLeave(self)
		self.Parent:Fire("OnLeave")
	end
	
	local function Item_OnClick(self)
		local parent = self.Parent
		parent.__ChooseItem = self.ID
		for i = 1, parent.__DisplayItemCount do
			parent:GetChild("ListBtn_"..i).HighlightLocked = false
		end
		self.HighlightLocked = true
		return parent:Fire("OnItemChoosed", parent.Keys[self.ID], parent.Items[self.ID], parent.Icons[self.ID], parent.Frames[self.ID])
	end
	
	local function Item_OnDoubleClick(self)
		local parent = self.Parent
		return parent:Fire("OnItemDoubleClick", parent.Keys[self.ID], parent.Items[self.ID], parent.Icons[self.ID], parent.Frames[self.ID])
	end
	
	local function RefreshItem(self, btnIdx, itemIdx)
		local btn = self:GetChild("ListBtn_"..btnIdx)
		
		if not btn then return end
		
		if itemIdx and self.Keys[itemIdx] ~= nil then
			btn:GetChild("Text"):SetText(self.Items[itemIdx] or "")
			if self.Icons[itemIdx] then
				btn:GetChild("Icon").Width = 18
				btn:GetChild("Icon"):SetTexture(self.Icons[itemIdx])
			else
				btn:GetChild("Icon").Width = 1
				btn:GetChild("Icon"):SetTexture(nil)
			end
			if self.Frames[itemIdx] and type(self.Frames[itemIdx]) == "table" and self.Frames[itemIdx].SetAllPoints then
				btn:GetChild("Text").Visible = false
				btn:GetChild("Icon").Visible = false
				if btn.__ShowF and btn.__ShowF.Parent == btn then
					btn.__ShowF.Visible = false
					btn.__ShowF:ClearAllPoints()
					btn.__ShowF.Parent = nil
				end
				btn.__ShowF = self.Frames[itemIdx]
				self.Frames[itemIdx].Parent = btn
				self.Frames[itemIdx]:ClearAllPoints()
				self.Frames[itemIdx]:SetAllPoints(btn)
				self.Frames[itemIdx].FrameLevel = btn.FrameLevel - 1
				self.Frames[itemIdx].Visible = true
			else
				btn:GetChild("Text").Visible = true
				btn:GetChild("Icon").Visible = true
				if btn.__ShowF and btn.__ShowF.Parent == btn then
					btn.__ShowF.Visible = false
					btn.__ShowF:ClearAllPoints()
					btn.__ShowF.Parent = nil
				end
				btn.__ShowF = nil
			end
		else
			btn:GetChild("Text"):SetText("")
			btn:GetChild("Icon"):SetTexture(nil)
			if btn.__ShowF and btn.__ShowF.Parent == btn then
				btn.__ShowF.Visible = false
				btn.__ShowF:ClearAllPoints()
				btn.__ShowF.Parent = nil
			end
			btn.__ShowF = nil
		end
	end
	
	local function OnValueChanged(self)
		local parent = self.Parent
		
		if not parent.Visible then return end
		
		for i = 1, parent.__DisplayItemCount do			
			RefreshItem(parent, i, i + self:GetValue() - 1)
			parent:GetChild("ListBtn_"..i).ID = i + self:GetValue() - 1
			parent:GetChild("ListBtn_"..i).HighlightLocked = false
			if parent:GetChild("ListBtn_"..i).ID == parent.__ChooseItem then
				parent:GetChild("ListBtn_"..i).HighlightLocked = true
			end
		end
	end
	
	local function relayoutFrame(self)
		if not self.__Layout then return end
		
		-- Dropdown List
		local dropCount = self.__DisplayItemCount
		
		-- Scroll Bar
		local scrollBar = self:GetChild("ScrollBar")
		
		local i, btn, btnFontString, texture, icon
		
		-- Add Button
		for i = 1, dropCount do
			if not self:GetChild("ListBtn_"..i) then
				btn = CreateFrame("Button", "ListBtn_"..i, self)
				btn:ClearAllPoints()
				btn:SetPoint("LEFT", self, "LEFT", 0, -4)
				btn:SetPoint("RIGHT", scrollBar, "LEFT", 0, 0)
				btn:SetPoint("TOP", self, "TOP", 0, (i - 1) * -_Height)
				btn:SetHeight(_Height)
				btn:SetHighlightTexture("Interface\\BUTTONS\\UI-Common-MouseHilight.blp", "ADD")
				texture = btn:GetHighlightTexture()
				texture:SetTexCoord(0.125,0.875,0.125,0.875)
				
				icon = CreateFrame("Texture", "Icon", btn, "OVERLAY")
				icon:SetWidth(18)
				icon:SetHeight(18)
				icon:SetTexCoord(0.05, 0.95, 0.05, 0.95)
				icon:SetPoint("LEFT", btn, "LEFT", 4, 0)
				
				btnFontString = CreateFrame("FontString", "Text", btn, "ARTWORK","GameFontNormal")
				--btn:SetFontString(btnFontString)
				btnFontString:SetPoint("LEFT", icon, "RIGHT")
				btnFontString:SetPoint("RIGHT", btn, "RIGHT")
				btnFontString:SetJustifyH("CENTER")
				btnFontString:SetJustifyV("MIDDLE")
				
				btn:SetText("")
				--btn:Hide()
				btn.OnClick = Item_OnClick
				btn.OnDoubleClick = Item_OnDoubleClick
				btn.OnEnter = Item_OnEnter
				btn.OnLeave = Item_OnLeave
			else
				self:GetChild("ListBtn_"..i).Visible = true
			end
		end
		
		-- Hide no use button
		i = dropCount + 1
		while self:GetChild("ListBtn_"..i) do
			self:GetChild("ListBtn_"..i):Hide()
			i = i + 1
		end
		
		local iStrCount = table.getn(self.Keys or {})
		
		if iStrCount == 0 then
			for i = 1, dropCount do
				self:GetChild("ListBtn_"..i):Hide()
				RefreshItem(self, i)
			end
			self:GetChild("ScrollBar"):SetMinMaxValues(1, 1)
			self:GetChild("ScrollBar"):Hide()
		elseif dropCount >= iStrCount then
			for i = 1, iStrCount do
				self:GetChild("ListBtn_"..i):Show()
				RefreshItem(self, i, i)
				self:GetChild("ListBtn_"..i).ID = i
				self:GetChild("ListBtn_"..i).HighlightLocked = false
				if self:GetChild("ListBtn_"..i).ID == self.__ChooseItem then
					self:GetChild("ListBtn_"..i).HighlightLocked = true
				end
			end
			
			if dropCount > iStrCount then
				for i = iStrCount +1, dropCount do
					self:GetChild("ListBtn_"..i):Hide()
					RefreshItem(self, i)
					self:GetChild("ListBtn_"..i).ID = i
					self:GetChild("ListBtn_"..i).HighlightLocked = false
				end
			end
			
			self:GetChild("ScrollBar"):SetMinMaxValues(1, 1)
			self:GetChild("ScrollBar"):Hide()
		else
			for i = 1, dropCount do
				self:GetChild("ListBtn_"..i):Show()
				RefreshItem(self, i, i)
				self:GetChild("ListBtn_"..i).ID = i
				self:GetChild("ListBtn_"..i).HighlightLocked = false
				if self:GetChild("ListBtn_"..i).ID == self.__ChooseItem then
					self:GetChild("ListBtn_"..i).HighlightLocked = true
				end
			end
			self:GetChild("ScrollBar"):SetMinMaxValues(1, iStrCount - dropCount + 1)
			self:GetChild("ScrollBar"):Show()
		end
		
		self:GetChild("ScrollBar"):SetValue(1)
	end
	
	local function OnMouseWheel(self, ...)
		local scrollBar = self:GetChild("ScrollBar")
		local iMin, iMax = scrollBar:GetMinMaxValues()
		local iPos = scrollBar:GetValue()
		local step = scrollBar.ValueStep
		local btnC = self.__DisplayItemCount

		if arg1 > 0 then
			if IsShiftKeyDown() then
				scrollBar.Value = iMin
			elseif IsControlKeyDown() then
				if iPos - btnC > iMin then
					scrollBar.Value = iPos - btnC
				else
					scrollBar.Value = iMin
				end
			else
				if iPos - step > iMin then
					scrollBar.Value = iPos - step
				else
					scrollBar.Value = iMin
				end
			end
		elseif arg1 < 0 then
			if IsShiftKeyDown() then
				scrollBar.Value = iMax
			elseif IsControlKeyDown() then
				if iPos + btnC < iMax then
					scrollBar.Value = iPos + btnC
				else
					scrollBar.Value = iMax
				end
			else
				if iPos + step < iMax then
					scrollBar.Value = iPos + step
				else
					scrollBar.Value = iMax
				end
			end
		end
	end
	
	local function OnSizeChanged(self)
		local cnt = math.ceil((self.Height - 12) / _Height)
		
		if cnt > 0 and cnt ~= self.Parent.__DisplayItemCount then
			self.Parent.__DisplayItemCount = cnt
			if self.Parent.Visible then
				self.Parent:Refresh()
			end
		end
	end
	
	local function pairsByKeys (t)
		local a = {}
		for n in pairs(t) do table.insert(a, n) end
		table.sort(a)
		local i = 0      -- iterator variable
		local iter = function ()   -- iterator function
			i = i + 1
			if a[i] == nil then 
				return nil
			else 
				return a[i], t[a[i]]
			end
		end
		return iter
	end
	
	local function remove(t, idx)
		for i, v in pairsByKeys(t) do
			if i > idx then
				t[i-1] = v
				t[i] = nil
			elseif i == idx then
				t[i] = nil
			end
		end
	end
	
	-- ScriptType
	local _ScriptType = {
		["OnItemChoosed"] = true,
		["OnItemDoubleClick"] = true,
	}
		
	--	FuncProxy
	local _FuncProxy = {		
		-- SelectItemByIndex
		["SelectItemByIndex"] = function(self, index)
			if not self.Visible then return end
			
			local iStrCount = table.getn(self.Keys)
			
			-- Check later, maybe nil
			self.__ChooseItem = index
			
			if not index or type(index) ~= "number" or index < 1 then index = 1 end
			if index > iStrCount then index = iStrCount end
			
			if self.__DisplayItemCount + index > iStrCount then
				self:GetChild("ScrollBar"):SetValue(iStrCount - self.__DisplayItemCount + 1)
			else
				self:GetChild("ScrollBar"):SetValue(index)
			end
			
			for i = 1, self.__DisplayItemCount do
				self:GetChild("ListBtn_"..i).HighlightLocked = false
				if self:GetChild("ListBtn_"..i).ID == self.__ChooseItem then
					self:GetChild("ListBtn_"..i).HighlightLocked = true
				end
			end
		end,
		
		-- SelectItemByText
		["SelectItemByText"] = function(self, text)
			if text and type(text) == "string" then
				for i,k in ipairs(self.Items) do
					if k == text then
						return self:SelectItemByIndex(i)
					end
				end
			end
			return self:SelectItemByIndex(0)
		end,
		
		-- SelectItemByValue
		["SelectItemByValue"] = function(self, val)
			if val ~= nil then	-- val could be false, so check with nil
				for i,k in ipairs(self.Keys) do
					if k == val then
						return self:SelectItemByIndex(i)
					end
				end
			end
			return self:SelectItemByIndex(0)
		end,
		
		-- GetSelectedItemIndex
		["GetSelectedItemIndex"] = function(self)
			return self.__ChooseItem or 0
		end,
		
		-- GetSelectedItemValue
		["GetSelectedItemValue"] = function(self)
			return self.__ChooseItem and self.Keys[self.__ChooseItem]
		end,
		
		-- GetSelectedItemText
		["GetSelectedItemText"] = function(self)
			return self.__ChooseItem and self.Items[self.__ChooseItem]
		end,
		
		-- GetScrollStep
		["GetScrollStep"] = function(self)
			return self:GetChild("ScrollBar").ValueStep
		end,
		
		-- SetScrollStep
		["SetScrollStep"] = function(self, step)
			if step > 0 then
				self:GetChild("ScrollBar").ValueStep = step
			end
		end,
		
		-- Refresh
		["Refresh"] = function(self)
			if not self.Visible then
				return
			end
			
			-- ReDraw
			relayoutFrame(self)
			
			-- Select Item
			self:SelectItemByIndex(self.__ChooseItem)
		end,
		
        -- SuspendLayout
        ["SuspendLayout"] = function(self)
			self.__Layout = false
        end,
		
        -- ResumeLayout 
        ["ResumeLayout"] = function(self)
			self.__Layout = true
			self:Refresh()
        end,
	
		-- Clear
		["Clear"] = function(self)
			wipe(self.Keys)
			wipe(self.Items)
			wipe(self.Icons)
			wipe(self.Frames)
			self.__ChooseItem = nil
			
			if self.Visible then
				self:Refresh()
			end
		end,
        -- AddItem
        ["AddItem"] = function(self, key, text, icon, frame)
			if key ~= nil then
				tinsert(self.Keys, key)
				if text and type(text) == "string" then
					tinsert(self.Items, text)
				else
					tinsert(self.Items, tostring(key))
				end
				self.Icons[getn(self.Keys)] = icon
				self.Frames[getn(self.Keys)] = frame
			end
			
			if self.Visible then
				self:Refresh()
			end
        end,
        -- SetItem
        ["SetItem"] = function(self, key, text, icon, frame)            
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			self.Keys[idx] = key
			if text and type(text) == "string" then
				self.Items[idx] = text
			else
				self.Items[idx] = tostring(key)
			end
			self.Icons[idx] = icon
			self.Frames[idx] = frame
			
			if self.Visible then
				self:Refresh()
			end
        end,
		-- GetItem
		["GetItem"] = function(self, key)
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			if self.Keys[idx] then
				return self.Keys[idx], self.Items[idx], self.Icons[idx], self.Frames[idx]
			end
		end,
		-- GetItemByIndex
		["GetItemByIndex"] = function(self, idx)
			if self.Keys[idx] then
				return self.Keys[idx], self.Items[idx], self.Icons[idx], self.Frames[idx]
			end
		end,
        -- RemoveItem
        ["RemoveItem"] = function(self, key)
			local idx = 1
			if key == nil then
				return
			end
			while self.Keys[idx] and self.Keys[idx] ~= key do
				idx = idx + 1
			end
			if self.Keys[idx] and self.Keys[idx] == key then
				tremove(self.Keys, idx)
				tremove(self.Items, idx)
				remove(self.Icons, idx)
				remove(self.Frames, idx)
				
				if self.Visible then
					self:Refresh()
				end
			end			
        end,
		-- SetList		
		["SetList"] = function(self, list)
			if type(list) == "table" then
				self:SuspendLayout()
				self:Clear()
				for k, v in pairs(list) do
					self:AddItem(k, v)
				end
				self:ResumeLayout()
			else
				error("The parameter must be a table")
			end
		end,
	}

	--	Property
	local _Property = {
		-- ScrollStep
		["ScrollStep"] = {
			["Set"] = function(self, step)
				self:SetScrollStep(step)
			end,
			
			["Get"] = function(self)
				return self:GetScrollStep()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
        -- DisplayItemCount
		["DisplayItemCount"] = {
			["Set"] = function(self, cnt)
				if cnt and type(cnt) == "number" and cnt > 3 then
					self.__DisplayItemCount = cnt
					self.Height = cnt * _Height + 6
					--if self.Visible then
					--	self:Refresh()
					--end
				end
			end,
			
			["Get"] = function(self)
				return self.__DisplayItemCount
			end,
            
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		-- Keys
		["Keys"] = {
			["Set"] = function(self, keys)
				if keys and type(keys) == "table" then
					self.__Keys = keys
				else
					self.__Keys = {}
				end
			end,
			["Get"] = function(self)
				self.__Keys = self.__Keys or {}
				return self.__Keys
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Items
		["Items"] = {
			["Set"] = function(self, items)
				if items and type(items) == "table" then
					self.__Items = items
				else
					self.__Items = {}
				end
			end,
			["Get"] = function(self)
				self.__Items = self.__Items or {}
				return self.__Items
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Icons
		["Icons"] = {
			["Set"] = function(self, icons)
				if icons and type(icons) == "table" then
					self.__Icons = icons
				else
					self.__Icons = {}
				end
			end,
			["Get"] = function(self)
				self.__Icons = self.__Icons or {}
				return self.__Icons
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- Frames
		["Frames"] = {
			["Set"] = function(self, frames)
				if frames and type(frames) == "table" then
					self.__Frames = frames
				else
					self.__Frames = {}
				end
			end,
			["Get"] = function(self)
				self.__Frames = self.__Frames or {}
				return self.__Frames
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		-- ItemCount
		["ItemCount"] = {			
			["Get"] = function(self)
				return getn(self.Keys)
			end,
            
			["Type"] = IGAS:GetPropertyType("Number"),
		},
	}

	local function _New(parent)	
		local frame = CreateFrame(_Base, nil, parent)
				
		frame.MouseWheelEnabled = true
		frame.Visible = true
		frame:ClearAllPoints()
		frame:SetBackdrop(_FrameBackdrop)
		frame:SetBackdropColor(0, 0, 0, 1)
		frame.__DisplayItemCount = 6
		frame.Height = 6 * _Height + 6
		frame.__Layout = true
		
		-- Scroll Bar
		local scrollBar = CreateFrame("ScrollBar", "ScrollBar", frame)
		scrollBar:Hide()
		
		-- Event Handle
		scrollBar.OnValueChanged = OnValueChanged
		scrollBar.OnSizeChanged = OnSizeChanged
		scrollBar.OnEnter = Item_OnEnter
		scrollBar.OnLeave = Item_OnLeave
		
		frame:StackScript("OnMouseWheel", OnMouseWheel)
		
		relayoutFrame(frame)

		return frame
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end