﻿-- Author      : zys924

--------------------------
-- ListBox	        	--
--------------------------
do
	-- Check Version
	local version = 1
	if not IGAS:RegisterFile("IGAS.GUI.Widget.ListBox", version) then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
	local tinsert = table.insert;
	local tremove = table.remove;
	local getn = table.getn;
	local wipe = wipe;
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "ListBox"
	local _Base = "Frame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function _UpdateBackdrop(self)
		_FrameBackdrop.bgFile = self.__BackTexture;
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
		self:GetChild("List"):ClearAllPoints();
		self:GetChild("List"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
		self:GetChild("List"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth,self.__BorderWidth);
		local sliderback = self:GetChild("SliderContainer").Backdrop;
		sliderback.edgeFile = _FrameBackdrop.edgeFile;
		sliderback.edgeSize = _FrameBackdrop.edgeSize;
		self:GetChild("SliderContainer"):SetBackdrop(sliderback);
		self:GetChild("SliderContainer"):SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:GetChild("SliderContainer"):SetWidth(21+self.__BorderWidth);
		self:GetChild("SliderContainer"):GetChild("SliderUp"):SetPoint("TOP",self:GetChild("SliderContainer"),"TOP",0,-self.__BorderWidth);
		self:GetChild("SliderContainer"):GetChild("SliderDown"):SetPoint("BOTTOM",self:GetChild("SliderContainer"),"BOTTOM",0,self.__BorderWidth);
	end

	local function Round(number)
		if number-math.floor(number)<math.ceil(number)-number then
			return math.floor(number);
		else
			return math.ceil(number);
		end
	end
	
	-- used to generate the list shown, filtered by keyword
	local function GenerateCurrentList(self, keyword)
		if not keyword then
			self.__CurrentList = self.__List;
			for i=1, getn(self.__CurrentList) do
				self.__CurrentList[i].Index = i;
			end
		else
			if type(keyword)~="string" then
				self.__CurrentList = self.__List;
				for i=1, getn(self.__CurrentList) do
					self.__CurrentList.Index = i;
				end
				return;
			else
				if not self.__CurrentList or self.__CurrentList==self.__List then
					self.__TemplateList = self.__TemplateList or {};
					self.__CurrentList = self.__TemplateList;
				end
				for i=1, getn(self.__CurrentList) do
					tremove(self.__CurrentList,i);
				end
				for i=1, getn(self.__List) do
					if self.__List[i].Text then
						if strfind(self.__List[i].Text, keyword) then
							tinsert(self.__CurrentList,getn(self.__CurrentList)+1,self.__List[i]);
							self.__CurrentList[getn(self.__CurrentList)].Index = i;
						end
					end
				end
			end
		end
	end
	
	local function _UpdateListBox(self)
		-- Height Adjustment
		local containerheight = self.__Height-2*self.__BorderWidth;
		if containerheight<self.__ItemHeight then
			containerheight = self.__ItemHeight;
		else
			containerheight = Round(containerheight/self.__ItemHeight)*self.__ItemHeight;
		end
		self.__PageCount = Round(containerheight / self.__ItemHeight);
		self.__UI:SetHeight(containerheight+2*self.__BorderWidth);
		-- used to store current item frames
		self.__ItemFrames = self.__ItemFrames or {};	
		for i=1, getn(self.__ItemFrames) do
			if self.__ItemFrames[i].__Entity then
				self.__ItemFrames[i].__Entity:Hide();
			end
			if self.__ItemFrames[i] then
				self.__ItemFrames[i]:Hide();
			end
		end
		-- used to store standard item frames;
		self.__StandardFrames = self.__StandardFrames or {};
		-- Initialize Slider
		local xoffset;
		if self.__PageCount<getn(self.__CurrentList) then
			xoffset = -20 - self.__BorderWidth;
			self:GetChild("SliderContainer"):Show();
			self:GetChild("SliderContainer"):GetChild("Slider"):SetMinMaxValues(1,getn(self.__CurrentList)-self.__PageCount+1);
			self:GetChild("SliderContainer"):GetChild("Slider"):SetValueStep(1);
			local minvalue,maxvalue = self:GetChild("SliderContainer"):GetChild("Slider"):GetMinMaxValues();
			if minvalue == self:GetChild("SliderContainer"):GetChild("Slider"):GetValue() then
				self:GetChild("SliderContainer"):GetChild("SliderUp"):Disable();
				self:GetChild("SliderContainer"):GetChild("SliderDown"):Enable();
			elseif maxvalue == self:GetChild("SliderContainer"):GetChild("Slider"):GetValue() then
				self:GetChild("SliderContainer"):GetChild("SliderUp"):Enable();
				self:GetChild("SliderContainer"):GetChild("SliderDown"):Disable();
			elseif self:GetChild("SliderContainer"):GetChild("Slider"):GetValue()<1 then
				self:GetChild("SliderContainer"):GetChild("Slider"):SetValue(1);
			else
				self:GetChild("SliderContainer"):GetChild("SliderUp"):Enable();
				self:GetChild("SliderContainer"):GetChild("SliderDown"):Enable();
			end
		else
			xoffset = -self.__BorderWidth;
			self.__StartIndex = 1;
			self:GetChild("SliderContainer"):Hide();
		end
		-- Initialize the starting item index
		if not self.__StartIndex or self.__StartIndex<0 then
			if not self.__SelectedIndex or self.__SelectedIndex<1 then
				self.__StartIndex = 1;
			elseif getn(self.__CurrentList)-self.__SelectedIndex+1 < self.__PageCount then
				self.__StartIndex = getn(self.__CurrentList)-self.__PageCount+1;
			else
				self.__StartIndex = self.__SelectedIndex;
			end
			self:GetChild("SliderContainer"):GetChild("Slider"):SetValue(self.__StartIndex);
		end
		for i=1, self.__PageCount do
			-- Initialize ItemFrames
			if not self.__ItemFrames[i] then
				self.__ItemFrames[i] = CreateFrame("Frame","ItemFrame" .. tostring(i),self:GetChild("List"));
				self.__ItemFrames[i].__Cover = self.__ItemFrames[i]:CreateTexture("Cover");
				self.__ItemFrames[i].__Cover:ClearAllPoints();
				self.__ItemFrames[i].__Cover:SetPoint("TOPLEFT",self.__ItemFrames[i],"TOPLEFT",0,0);
				self.__ItemFrames[i].__Cover:SetPoint("BOTTOMRIGHT",self.__ItemFrames[i],"BOTTOMRIGHT",0,0);
				self.__ItemFrames[i].__Cover:SetDrawLayer("BACKGROUND");
				self.__ItemFrames[i].__Cover:SetBlendMode("Add");
				self.__ItemFrames[i].__Cover:SetAlpha(0.8);
				self.__ItemFrames[i].MouseEnabled = true;
				local listbox = self;
				self.__ItemFrames[i]:StackScript("OnEnter",function(self)
					if listbox.__CurrentList[self.__index].Index ~= listbox.__SelectedIndex then
						self.__Cover:SetTexture(listbox.__HoverTexture);
					end
					listbox:Fire("OnItemMouseEnter",listbox.__CurrentList[self.__index].Index);
				end);
				self.__ItemFrames[i]:StackScript("OnLeave",function(self)
					if listbox.__CurrentList[self.__index].Index ~= listbox.__SelectedIndex then
						self.__Cover:SetTexture("");
					end
					listbox:Fire("OnItemMouseLeave",listbox.__CurrentList[self.__index].Index);
				end);
				self.__ItemFrames[i]:StackScript("OnMouseDown",function(self,btn)
					if btn=="LeftButton" then
						if listbox.__SelectedIndex == listbox.__CurrentList[self.__index].Index then
							listbox.__SelectedIndex = -1;
							listbox:Fire("OnItemCanceled",listbox.__CurrentList[self.__index].Index);
						else
							listbox.__SelectedIndex = listbox.__CurrentList[self.__index].Index;
							self.__Cover:SetTexture(listbox.__SelectedTexture);
							listbox:Fire("OnItemSelected",listbox.__CurrentList[self.__index].Index);
						end
						_UpdateListBox(listbox);
					else
						listbox:Fire("OnItemRightClick",listbox.__CurrentList[self.__index].Index);
					end
				end);
			end
			-- Initialize StandardFrames
			if not self.__StandardFrames[i] then
				self.__StandardFrames[i] = CreateFrame("Frame","StandardFrame" .. tostring(i),self.__ItemFrames[i]);
				self.__StandardFrames[i].Icon = self.__StandardFrames[i]:CreateTexture("Icon");
				self.__StandardFrames[i].Icon:ClearAllPoints();
				self.__StandardFrames[i].Icon:SetDrawLayer("ARTWORK");
				self.__StandardFrames[i].Icon:SetPoint("TOPLEFT",self.__StandardFrames[i],"TOPLEFT",1,-1);
				self.__StandardFrames[i].Icon:SetPoint("BOTTOMLEFT",self.__StandardFrames[i],"BOTTOMLEFT",1,1);
				self.__StandardFrames[i].Icon:SetWidth(self.__ItemHeight-2);
				self.__StandardFrames[i].Text = self.__StandardFrames[i]:CreateFontString("Text");
				self.__StandardFrames[i].Text:SetJustifyV("MIDDLE");
				self.__StandardFrames[i].Text:SetDrawLayer("ARTWORK");
				self.__StandardFrames[i].Text:ClearAllPoints();
			end
			-- Set the current items page
			if self.__CurrentList[self.__StartIndex+i-1] then
				local itemframe;
				if self.__CurrentList[self.__StartIndex+i-1].Frame then
					self.__ItemFrames[i].__Entity = self.__CurrentList[self.__StartIndex+i-1].Frame;
					itemframe = self.__ItemFrames[i].__Entity;
					itemframe:ClearAllPoints();
					itemframe:SetParent(self.__ItemFrames[i]);
					itemframe:SetPoint("TOPLEFT",self.__ItemFrames[i],"TOPLEFT",0,0);
					itemframe:SetPoint("BOTTOMRIGHT",self.__ItemFrames[i],"BOTTOMRIGHT",0,0);
				else
					self.__ItemFrames[i].__Entity = self.__StandardFrames[i];
					itemframe = self.__ItemFrames[i].__Entity;
					itemframe:ClearAllPoints();
					itemframe:SetParent(self.__ItemFrames[i]);
					itemframe:SetPoint("TOPLEFT",self.__ItemFrames[i],"TOPLEFT",0,0);
					itemframe:SetPoint("BOTTOMRIGHT",self.__ItemFrames[i],"BOTTOMRIGHT",0,0);
					if self.__CurrentList[self.__StartIndex+i-1].Icon and strtrim(self.__CurrentList[self.__StartIndex+i-1].Icon)~="" then
						itemframe.Icon:SetTexture(self.__CurrentList[self.__StartIndex+i-1].Icon);
						itemframe.Text:ClearAllPoints();
						itemframe.Text:SetPoint("TOPLEFT",itemframe.Icon,"TOPRIGHT",1,0);
						itemframe.Text:SetPoint("BOTTOMRIGHT",itemframe,"BOTTOMRIGHT",-1,0);
					else
						itemframe.Icon:SetTexture("");
						itemframe.Text:ClearAllPoints();
						itemframe.Text:SetPoint("TOPLEFT",itemframe,"TOPLEFT",1,0);
						itemframe.Text:SetPoint("BOTTOMRIGHT",itemframe,"BOTTOMRIGHT",-1,0);
					end
					itemframe.Text:SetFontObject(self.__FontObject);
					itemframe.Text:SetTextColor(self.__FontColor.red,self.__FontColor.green,self.__FontColor.blue,self.__FontColor.alpha);
					itemframe.Text:SetText(self.__CurrentList[self.__StartIndex+i-1].Text or "");
					itemframe.Text:SetJustifyH(self.__JustifyH);
				end
				itemframe:Show();
				itemframe = self.__ItemFrames[i];
				itemframe:ClearAllPoints();
				if i==1 then
					itemframe:SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
					itemframe:SetPoint("TOPRIGHT",self,"TOPRIGHT",xoffset,-self.__BorderWidth);
				else
					itemframe:SetPoint("TOPLEFT",self.__ItemFrames[i-1],"BOTTOMLEFT",0,0);
					itemframe:SetPoint("TOPRIGHT",self.__ItemFrames[i-1],"BOTTOMRIGHT",0,0);
				end
				itemframe:SetHeight(self.__ItemHeight);
				itemframe.__index = self.__StartIndex+i-1;
				if self.__CurrentList[itemframe.__index].Index ~= self.__SelectedIndex then
					itemframe.__Cover:SetTexture("");
				else
					itemframe.__Cover:SetTexture(self.__SelectedTexture);
				end
				itemframe:Show();
				self.__ItemFrames[i] = itemframe;
			end
		end
	end
	
	-- Event Handlers
	function Slider_OnValueChanged(self)
		local minvalue,maxvalue = self:GetMinMaxValues();
		if minvalue == self:GetValue() then
			self:GetParent():GetChild("SliderUp"):Disable();
			self:GetParent():GetChild("SliderDown"):Enable();
		elseif maxvalue == self:GetValue() then
			self:GetParent():GetChild("SliderUp"):Enable();
			self:GetParent():GetChild("SliderDown"):Disable();
		else
			self:GetParent():GetChild("SliderUp"):Enable();
			self:GetParent():GetChild("SliderDown"):Enable();
		end
		self:GetParent():GetParent().__StartIndex = self:GetValue();
		_UpdateListBox(self:GetParent():GetParent());
	end
	
	function SliderDown_OnClick(self)
		self:GetParent():GetChild("Slider"):SetValue(self:GetParent():GetChild("Slider"):GetValue()+1);
	end
	
	function SliderUp_OnClick(self)
		self:GetParent():GetChild("Slider"):SetValue(self:GetParent():GetChild("Slider"):GetValue()-1);
	end
	
	function ListBox_OnMouseWheel(self,dir)
		if self:GetChild("SliderContainer").Visible then
			if dir==1 then
				SliderUp_OnClick(self:GetChild("SliderContainer"):GetChild("SliderUp"));
			else
				SliderDown_OnClick(self:GetChild("SliderContainer"):GetChild("SliderDown"));
			end
		end
	end

	-- ScriptType
	local _ScriptType = {
		["OnItemMouseEnter"] = true,
		["OnItemMouseLeave"] = true,
		["OnItemRightClick"] = true,
		["OnItemSelected"] = true,
		["OnItemCanceled"] = true,
	}

	--	FuncProxy
	local _FuncProxy = {
	
		-- Get/Set BackTexture
		["GetBackTexture"] = function(self)
			return self.__BackTexture;
		end,
		["SetBackTexture"] = function(self, backtexture)
			self.__BackTexture = backtexture;
			_UpdateBackdrop(self);
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			if borderstyle == self.__BorderStyle then
				return
			end
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be no less than 0.");
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0;
			end
			self.__BorderWidth = borderwidth;
			_UpdateBackdrop(self);
			_UpdateListBox(self);
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor;
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderTexture
		["GetBorderTexture"] = function(self)
			return self.__BorderTexture
		end,
		["SetBorderTexture"] = function(self, bordertexture)
			self.__BorderTexture = bordertexture
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set FontObject
		["GetFontObject"] = function(self)
			return self.__FontObject;
		end,
		["SetFontObject"] = function(self, font)
			self.__FontObject = font;
		end,
		
		-- Get/Set FontColor
		["GetFontColor"] = function(self)
			return self.__FontColor;
		end,
		["SetFontColor"] = function(self, fontcolor)
			self.__FontColor = fontcolor
		end,
		
		-- Get/Set HoverTexture
		["GetHoverTexture"] = function(self)
			return self.__HoverTexture;
		end,
		["SetHoverTexture"] = function(self, hovertexture)
			self.__HoverTexture = hovertexture;
		end,
		
		-- Get/Set ItemHeight
		["GetItemHeight"] = function(self)
			return self.__ItemHeight;
		end,	
		["SetItemHeight"] = function(self, itemheight)
			self.__ItemHeight = itemheight;
			_UpdateListBox(self);
		end,
		
		-- Get/Set JustifyH
		["GetJustifyH"] = function(self)
			return self.__JustifyH;
		end,
		["SetJustifyH"] = function(self, justifyh)
			self.__JustifyH = justifyh;
			_UpdateListBox(self);
		end,

		-- [OVERRIDE]Get/Set Height
		["GetHeight"] = function(self)
			return self.__UI:GetHeight();
		end,	
		["SetHeight"] = function(self, height)
			self.__Height = height;
			_UpdateListBox(self);
		end,
		
		-- Get/Set List
		["GetList"] = function(self)
			return self.__List;
		end,	
		["SetList"] = function(self, list)
			self.__List = list;
			_UpdateListBox(self);
		end,
		
		-- Get/Set SelectedIndex
		["GetSelectedIndex"] = function(self)
			return self.__SelectedIndex;
		end,	
		["SetSelectedIndex"] = function(self, selectedindex)
			if not selectedindex then
				return;
			end
			if selectedindex>=-1 and selectedindex<=getn(self.__List) then
				self.__SelectedIndex = selectedindex;
				_UpdateListBox(self);
			else
				error("SelectedIndex must be within -1 and the nsize of List");
			end
		end,
		
		-- Get/Set SelectedTexture
		["GetSelectedTexture"] = function(self)
			return self.__SelectedTexture;
		end,
		["SetSelectedTexture"] = function(self, selectedtexture)
			self.__SelectedTexture = selectedtexture;
		end,
		
		-- Get/Set WheelScroll
		["GetWheelScroll"] = function(self)
			return self.MouseWheelEnabled;
		end,
		["SetWheelScroll"] = function(self, wheelscroll)
			self.MouseWheelEnabled = wheelscroll;
		end,
		
		-- GenerateCurrentList
		["GenerateCurrentList"] = function(self,keyword)
			GenerateCurrentList(self,keyword);
		end,
		
		-- Update
		["Update"] = function(self)
			_UpdateListBox(self);
		end,
	}


	--	Property
	local _Property = {

		-- BackTexture(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["BackTexture"]={
			["Get"] = function(self)
				return self:GetBackTexture();
			end,
			["Set"] = function(self, value)
				self:SetBackTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},

		-- FontObject(default:"GameFontHighlight")
		["FontObject"]={
			["Get"] = function(self)
				return self:GetFontObject();
			end,
			["Set"] = function(self, value)
				self:SetFontObject(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- FontColor{red=1,green=1,blue=1,alpha=1}
		["FontColor"]={
			["Get"] = function(self)
				return self:GetFontColor();
			end,
			["Set"] = function(self, value)
				self:SetFontColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- JustifyH
		["JustifyH"] = {
			["Get"] = function(self)
				return self:GetJustifyH()
			end,
			["Set"] = function(self, value)
				self:SetJustifyH(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyH"),
		},
		
		-- Height
		["Height"] = {
			["Get"] = function(self)
				return self:GetHeight();
			end,
			["Set"] = function(self, value)
				self:SetHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- HoverTexture(default:"Interface\\BUTTONS\\UI-Common-MouseHilight")
		["HoverTexture"] = {
			["Get"] = function(self)
				return self:GetHoverTexture();
			end,
			["Set"] = function(self, value)
				self:SetHoverTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- ItemHeight
		["ItemHeight"] = {
			["Get"] = function(self)
				return self:GetItemHeight();
			end,
			["Set"] = function(self, value)
				self:SetItemHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- List
		["List"] = {
			["Get"] = function(self)
				return self:GetList();
			end,
			["Set"] = function(self, value)
				self:SetList(value);
			end,
			["Type"] = IGAS:GetPropertyType("Table"),
		},
		
		-- SelectedIndex
		["SelectedIndex"] = {
			["Get"] = function(self)
				return self:GetSelectedIndex();
			end,
			["Set"] = function(self, value)
				self:SetSelectedIndex(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- SelectedTexture(default:"Interface\\BUTTONS\\UI-Listbox-Highlight")
		["SelectedTexture"] = {
			["Get"] = function(self)
				return self:GetSelectedTexture();
			end,
			["Set"] = function(self, value)
				self:SetSelectedTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- WheelScroll
		["WheelScroll"] = {
			["Get"] = function(self)
				return self:GetWheelScroll();
			end,
			["Set"] = function(self, value)
				self:SetWheelScroll(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("Frame",nil,parent);
		frame.Movable = false;
		frame.Resizable = false;
		frame.MouseEnabled = true;
		frame.MouseWheelEnabled = true;
		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		
		frame.__BorderStyle = "SINGLE";
		frame.__BorderWidth = 1;
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8};
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackColor = {["red"]=0.3,["green"]=0.3,["blue"]=0.3,["alpha"]=0.6};
		frame.__FontObject = "GameFontNormal";
		frame.__FontColor = {["red"]=1,["green"]=1,["blue"]=1,["alpha"]=1};
		frame.__JustifyH = "LEFT";
		frame.__List = {};
		frame.__ItemHeight = 20;
		frame.__HoverTexture = "Interface\\BUTTONS\\UI-Common-MouseHilight";
		frame.__SelectedTexture = "Interface\\BUTTONS\\UI-Listbox-Highlight";
		frame.__SelectedIndex = -1;
		frame.Width = 150;
		frame.__Height = 5;
		
		local list = CreateFrame("Frame","List",frame);
		
		local slidercontainer = CreateFrame("Frame","SliderContainer",frame);
		slidercontainer:SetPoint("TOPRIGHT",frame,"TOPRIGHT",0,0);
		slidercontainer:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0);
		slidercontainer:SetBackdrop({
			bgFile = "",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 12, edgeSize = 12,
			insets = { left = 2, right = 2, top = 2, bottom = 2 }
		});
		slidercontainer:SetWidth(20);
		local sliderup = CreateFrame("Button","SliderUp",slidercontainer,"UIPanelScrollUpButtonTemplate");
		sliderup:SetPoint("TOP",slidercontainer,"TOP",0,-frame.__BorderWidth); 
		sliderup:Disable();
		sliderup:StackScript("OnClick",SliderUp_OnClick);
		local sliderdown = CreateFrame("Button","SliderDown",slidercontainer,"UIPanelScrollDownButtonTemplate");
		sliderdown:SetPoint("BOTTOM",slidercontainer,"BOTTOM",0,frame.__BorderWidth);
		sliderdown:Disable();
		sliderdown:StackScript("OnClick",SliderDown_OnClick);
		local slider = CreateFrame("Slider","Slider",slidercontainer);
		slider:SetPoint("TOP",sliderup,"BOTTOM",0,8);
		slider:SetPoint("BOTTOM",sliderdown,"TOP",0,-8);
		slider:SetWidth(20);
		slider:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
		slider:StackScript("OnValueChanged",Slider_OnValueChanged);
		frame:StackScript("OnMouseWheel",ListBox_OnMouseWheel);
		
		_UpdateBackdrop(frame);
		GenerateCurrentList(frame,"4");
		_UpdateListBox(frame);

		return frame;
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
