﻿-- Author      : Kurapica
-- Create Date : 8/03/2008 17:14

--------------------------
-- MultiLineTextBox     --
--------------------------
do
	-- Check Version
	local version = 3

	if not IGAS:RegisterFile("IGAS.GUI.Widget.MultiLineTextBox", version) then
		return
	end

    local CreateFrame = IGAS.GUI.CreateFrame
    
	local _WidgetName = "MultiLineTextBox"
    local _Base = "ScrollForm"
	
    -- Scripts
    local function OnMouseUp(self)
        self.__Text:SetFocus()
    end
    
    local function OnCursorChanged(self, x, y, w, h)		
		if y and h then
			y = -y
			
			if self.__Container.Value > y then
				self.__Container.Value = y
			elseif self.__Container.Value + self.__Container.Height < y + h then
				self.__Container.Value = y + h - self.__Container.Height
			end
		end
		
		self.__Container:Fire("OnCursorChanged", x, y, w, h)
    end
	
    local function OnEscapePressed(self, ...)
        self:ClearFocus()
		self.__Container:Fire("OnEscapePressed", ...)
    end
    
    local function OnTextChanged(self, ...)
		self.__Container:Fire("OnTextChanged", ...)
	end
	
	local function OnSizeChanged(self)
		self.__Container:FixHeight()
	end
  
    local function OnEditFocusGained(self, ...)
		self.__Container:Fire("OnEditFocusGained", ...)
	end

    local function OnEditFocusLost(self, ...)
		self.__Container:Fire("OnEditFocusLost", ...)
	end

    local function OnEnterPressed(self, ...)
		self.__Container:Fire("OnEnterPressed", ...)
	end

    local function OnInputLanguageChanged(self, ...)
		self.__Container:Fire("OnInputLanguageChanged", ...)
	end

    local function OnSpacePressed(self, ...)
		self.__Container:Fire("OnSpacePressed", ...)
	end

    local function OnTabPressed(self, ...)
		self:Insert("    ")
		self.__Container:Fire("OnTabPressed", ...)
	end

    local function OnTextSet(self, ...)
		self.__Container:Fire("OnTextSet", ...)
	end
	
	local function OnChar(self, ...)
		self.__Container:Fire("OnChar", ...)
	end
	
	local function OnCharComposition(self, ...)
		self.__Container:Fire("OnCharComposition", ...)
	end
	
	-- ScriptType
	local _ScriptType = {
        ["OnTextChanged"] = true,
		["OnCursorChanged"] = true,
		["OnEditFocusGained"] = true,
		["OnEditFocusLost"] = true,
		["OnEnterPressed"] = true,
		["OnEscapePressed"] = true,
		["OnInputLanguageChanged"] = true,
		["OnSpacePressed"] = true,
		["OnTabPressed"] = true,
		["OnTextSet"] = true,
		["OnChar"] = true,
		["OnCharComposition"] = true,
    }
	    
	--	FuncProxy
	local _FuncProxy = {		
	    -- EditBox:AddHistoryLine("text") - Add text to the edit history. 
		["AddHistoryLine"] = function(self, ...)
			return self.__Text:AddHistoryLine(...)
		end,
	    -- EditBox:ClearFocus() 
		["ClearFocus"] = function(self, ...)
			return self.__Text:ClearFocus(...)
		end,
	    -- EditBox:GetAltArrowKeyMode() - Return whether only alt+arrow keys work for navigating the edit box, not arrow keys alone. 
		["GetAltArrowKeyMode"] = function(self, ...)
			return self.__Text:GetAltArrowKeyMode(...)
		end,
	    -- EditBox:GetBlinkSpeed() - Gets the blink speed of the EditBox in seconds - New in 1.11. 
		["GetBlinkSpeed"] = function(self, ...)
			return self.__Text:GetBlinkSpeed(...)
		end,
	    -- EditBox:GetCursorPosition() - Gets the position of the cursor inside the EditBox - New in 2.3. 
		["GetCursorPosition"] = function(self, ...)
			return self.__Text:GetCursorPosition(...)
		end,
	    -- EditBox:GetHistoryLines() - Get the number of history lines for this edit box 
		["GetHistoryLines"] = function(self, ...)
			return self.__Text:GetHistoryLines(...)
		end,
	    -- EditBox:GetInputLanguage() - Get the input language (locale based not in-game) 
		["GetInputLanguage"] = function(self, ...)
			return self.__Text:GetInputLanguage(...)
		end,
	    -- EditBox:GetMaxBytes() - Gets the maximum number bytes allowed in the EditBox - New in 1.11. 
		["GetMaxBytes"] = function(self, ...)
			return self.__Text:GetMaxBytes(...)
		end,
	    -- EditBox:GetMaxLetters() - Gets the maximum number of letters allowed in the EditBox - New in 1.11. 
		["GetMaxLetters"] = function(self, ...)
			return self.__Text:GetMaxLetters(...)
		end,
	    -- EditBox:GetNumLetters() - Gets the number of letters in the box. 
		["GetNumLetters"] = function(self, ...)
			return self.__Text:GetNumLetters(...)
		end,
	    -- EditBox:GetNumber() 
		["GetNumber"] = function(self, ...)
			return self.__Text:GetNumber(...)
		end,
	    -- EditBox:GetText() - Get the current text contained in the edit box. 
		["GetText"] = function(self, ...)
			return self.__Text:GetText(...)
		end,
	    -- EditBox:GetTextInsets() - Gets the text display insets for the EditBox - New in 1.11. 
		["GetTextInsets"] = function(self, ...)
			return self.__Text:GetTextInsets(...)
		end,
	    -- EditBox:HighlightText([startPos, endPos]) - Set the highlight to all or some of the edit box text. 
		["HighlightText"] = function(self, ...)
			return self.__Text:HighlightText(...)
		end,
	    -- EditBox:Insert("text") - Insert text into the edit box. 
		["Insert"] = function(self, ...)
			return self.__Text:Insert(...)
		end,
	    -- EditBox:IsAutoFocus() - Determine if the EditBox has autofocus enabled - New in 1.11. 
		["IsAutoFocus"] = function(self, ...)
			return self.__Text:IsAutoFocus(...)
		end,
	    -- EditBox:IsMultiLine() - Determine if the EditBox accepts multiple lines - New in 1.11. 
		["IsMultiLine"] = function(self, ...)
			return self.__Text:IsMultiLine(...)
		end,
	    -- EditBox:IsNumeric() - Determine if the EditBox only accepts numeric input - New in 1.11. 
		["IsNumeric"] = function(self, ...)
			return self.__Text:IsNumeric(...)
		end,
	    -- EditBox:IsPassword() - Determine if the EditBox performs password masking - New in 1.11. 
		["IsPassword"] = function(self, ...)
			return self.__Text:IsPassword(...)
		end,
	    -- EditBox:SetAltArrowKeyMode(enable) - Make only alt+arrow keys work for navigating the edit box, not arrow keys alone. 
		["SetAltArrowKeyMode"] = function(self, ...)
			return self.__Text:SetAltArrowKeyMode(...)
		end,
	    -- EditBox:SetAutoFocus(state) - Set whether or not the editbox will attempt to get input focus when it gets shown (default: yes) - New in 1.11. 
		["SetAutoFocus"] = function(self, ...)
			return self.__Text:SetAutoFocus(...)
		end,
	    -- EditBox:SetBlinkSpeed 
		["SetBlinkSpeed"] = function(self, ...)
			return self.__Text:SetBlinkSpeed(...)
		end,
	    -- EditBox:SetCursorPosition(position) - Set the position of the cursor within the EditBox - New in 2.3. 
		["SetCursorPosition"] = function(self, ...)
			return self.__Text:SetCursorPosition(...)
		end,
	    -- EditBox:SetFocus() - Move input focus (the cursor) to this editbox 
		["SetFocus"] = function(self, ...)
			return self.__Text:SetFocus(...)
		end,
	    -- EditBox:SetFont("font", size[,"flags"]) - Set the font to use for display. 
		["SetFont"] = function(self, ...)
			return self.__Text:SetFont(...)
		end,
	    -- EditBox:SetHistoryLines() - Set the number of history lines to remember. 
		["SetHistoryLines"] = function(self, ...)
			return self.__Text:SetHistoryLines(...)
		end,
	    -- EditBox:SetMaxBytes(maxBytes) - Set the maximum byte size for entered text. 
		["SetMaxBytes"] = function(self, ...)
			return self.__Text:SetMaxBytes(...)
		end,
	    -- EditBox:SetMaxLetters(maxLetters) - Set the maximum number of letters for entered text. 
		["SetMaxLetters"] = function(self, ...)
			return self.__Text:SetMaxLetters(...)
		end,
	    -- EditBox:SetMultiLine(state) - Set the EditBox's multi-line state - New in 1.11. 
		["SetMultiLine"] = function(self, ...)
			error("This editbox must be multi-line", 2)
		end,

	    -- EditBox:SetNumber(number) 
		["SetNumber"] = function(self, ...)
			return self.__Text:SetNumber(...)
		end,
	    -- EditBox:SetNumeric(state) - Set if the EditBox only accepts numeric input - New in 1.11. 
		["SetNumeric"] = function(self, ...)
			return self.__Text:SetNumeric(...)
		end,
	    -- EditBox:SetPassword(state) - Set the EditBox's password masking state - New in 1.11. 
		["SetPassword"] = function(self, ...)
			return self.__Text:SetPassword(...)
		end,

	    -- EditBox:SetText("text") - Set the text contained in the edit box. 
		["SetText"] = function(self, text)
			self.__Text.Text = text
		end,
	    -- EditBox:SetTextInsets(l, r, t, b) 
		["SetTextInsets"] = function(self, ...)
			return self.__Text:SetTextInsets(...)
		end,
	    -- EditBox:ToggleInputLanguage() 
		["ToggleInputLanguage"] = function(self, ...)
			return self.__Text:ToggleInputLanguage(...)
		end,
		-- HasFocus
		["HasFocus"] = function(self, ...)
			return self.__Text:HasFocus(...)
		end,
    }
	
	--	Property
	local _Property = {        
        -- NumericOnly
        ["NumericOnly"] = {
			["Set"] = function(self, state)
				self:SetNumeric(state)
			end,
			
			["Get"] = function(self)
				return (self:IsNumeric() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},	 
    
        -- Password
        ["Password"] = {
			["Set"] = function(self, state)
				self:SetPassword(state)
			end,
			
			["Get"] = function(self)
				return (self:IsPassword() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},	 
    
        -- AutoFocus
        ["AutoFocus"] = {
			["Set"] = function(self, state)
				self:SetAutoFocus(state)
			end,
			
			["Get"] = function(self)
				return (self:IsAutoFocus() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},    
    
        -- HistoryLines
        ["HistoryLines"] = {
			["Set"] = function(self, num)
				self:SetHistoryLines(num)
			end,
			
			["Get"] = function(self)
				return self:GetHistoryLines()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},    
    
        -- Focus
        ["Focused"] = {
			["Set"] = function(self, focus)
				if focus then
					self:SetFocus()
				else
					self:ClearFocus() 
				end
			end,
			
			["Get"] = function(self)
				return (self:HasFocus() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},    
        
        -- AltArrowKeyMode
        ["AltArrowKeyMode"] = {
			["Set"] = function(self, enable)
				self:SetAltArrowKeyMode(enable)
			end,
			
			["Get"] = function(self)
				return (self:GetAltArrowKeyMode() and true) or false
			end,
			
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},    
    
        -- BlinkSpeed
        ["BlinkSpeed"] = {
			["Set"] = function(self, speed)
				self:SetBlinkSpeed(speed)
			end,
			
			["Get"] = function(self)
				return self:GetBlinkSpeed()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
    
        -- CursorPosition
        ["CursorPosition"] = {
			["Set"] = function(self, position)
				self:SetCursorPosition(position)
			end,
			
			["Get"] = function(self)
				return self:GetCursorPosition()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
    
        -- MaxBytes
        ["MaxBytes"] = {
			["Set"] = function(self, maxBytes)
				self:SetMaxBytes(maxBytes)
			end,
			
			["Get"] = function(self)
				return self:GetMaxBytes()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},    
    
        -- MaxLetters
        ["MaxLetters"] = {
			["Set"] = function(self, maxLetters)
				self:SetMaxLetters(maxLetters)
			end,
			
			["Get"] = function(self)
				return self:GetMaxLetters()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},    
      
        -- Number
        ["Number"] = {
			["Set"] = function(self, number)
				self:SetNumber(number)
			end,
			
			["Get"] = function(self)
				return self:GetNumber()
			end,
			
			["Type"] = IGAS:GetPropertyType("Number"),
		},
      
        -- Text
        ["Text"] = {
			["Set"] = function(self, text)
				self:SetText(text)
			end,
			
			["Get"] = function(self)
				return self:GetText()
			end,
			
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- TextInsets
		["TextInsets"] = {
			["Set"] = function(self, RectInset)
				self:SetTextInsets(RectInset.left, RectInset.right, RectInset.top, RectInset.bottom)
			end,
			
			["Get"] = function(self)
				local RectInset = {}
				RectInset.left, RectInset.right, RectInset.top, RectInset.bottom = self:GetTextInsets()
				return RectInset 
			end,
			
			["Type"] = IGAS:GetPropertyType("RectInset"),
		},
		
        -- Editable
        ["Editable"] = {
			["Set"] = function(self, flag)
				self.MouseEnabled = flag
				self.__Text.MouseEnabled = flag
				if not flag then
					self.__Text.AutoFocus = false
				end
			end,
			
			["Get"] = function(self)
				return self.__Text.MouseEnabled
			end,
            
			["Type"] = IGAS:GetPropertyType("Boolean"),
        },
    }
	
    -- Scripts        
    local function _New(parent)
		local frame = CreateFrame(_Base, nil, parent)

		local container = frame.Container
		
		local editbox = CreateFrame("EditBox", "Text", container)
		editbox:SetPoint("TOPLEFT", container, "TOPLEFT", 0, 0)
		editbox:SetPoint("TOPRIGHT", container, "TOPRIGHT", 0, 0)
		editbox:SetMultiLine(true)
		editbox:SetTextInsets(5, 5, 3, 3)
		editbox:EnableMouse(true)
		editbox:SetAutoFocus(false)
		editbox:SetFontObject("GameFontNormal")
		editbox.Text = ""
        
        frame.__Text = editbox
        editbox.__Container = frame

		frame:StackScript("OnMouseUp", OnMouseUp)

		editbox:StackScript("OnEscapePressed", OnEscapePressed)
		editbox:StackScript("OnTextChanged", OnTextChanged)
	    editbox:StackScript("OnCursorChanged", OnCursorChanged)
	    editbox:StackScript("OnEditFocusGained", OnEditFocusGained)
	    editbox:StackScript("OnEditFocusLost", OnEditFocusLost)
	    editbox:StackScript("OnEnterPressed", OnEnterPressed)
		editbox:StackScript("OnSizeChanged", OnSizeChanged)
	    editbox:StackScript("OnInputLanguageChanged", OnInputLanguageChanged)
	    editbox:StackScript("OnSpacePressed", OnSpacePressed)
	    editbox:StackScript("OnTabPressed", OnTabPressed)
	    editbox:StackScript("OnTextSet", OnTextSet)
		editbox:StackScript("OnCharComposition", OnCharComposition)
		editbox:StackScript("OnChar", OnChar)
		
		frame:FixHeight()
        
		return frame
	end
	
	-- Register
    local _WidgetInfo = {
        ["WidgetName"] = _WidgetName,
        ["Base"] = _Base,
        ["FuncProxy"] = _FuncProxy,
        ["Property"] = _Property,
        ["ScriptType"] = _ScriptType,
        ["New"] = _New,
    }
    
    IGAS:NewWidget(_WidgetInfo, true)
end
