﻿-- Author      : zys924

--------------------------
-- SimpleComboBox      	--
--------------------------
do
	-- Check Version
	local version = 1
	if not IGAS:RegisterFile("IGAS.GUI.Widget.SimpleComboBox", version) then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
	local tinsert = table.insert;
	local tremove = table.remove;
	local getn = table.getn;
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "SimpleComboBox"
	local _Base = "Frame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	
	local function _UpdateBackdrop(self)
		_FrameBackdrop.bgFile = self.__BackTexture;
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
		self:GetChild("Icon"):ClearAllPoints();
		self:GetChild("Icon"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
		self:GetChild("Icon"):SetPoint("BOTTOMLEFT",self,"BOTTOMLEFT",self.__BorderWidth,self.__BorderWidth);
		self:GetChild("Icon"):SetWidth(self.__Height-2*self.__BorderWidth);
		self:GetChild("DropDown"):ClearAllPoints();
		self:GetChild("DropDown"):SetPoint("TOPRIGHT",self,"TOPRIGHT",2-self.__BorderWidth,2-self.__BorderWidth);
		self:GetChild("DropDown"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth-2,self.__BorderWidth-2);
		self:GetChild("DropDown"):SetWidth(self.__Height-2*self.__BorderWidth+4);
		self:GetChild("List").BorderStyle = self.__BorderStyle;
		self:GetChild("List").BorderWidth = self.__BorderWidth;
		self:GetChild("List").BorderColor = self.__BorderColor;
	end

	local function _UpdateComboBox(self)
		self:GetChild("Text"):ClearAllPoints();
		if self.__Icon and strtrim(self.__Icon)~="" then
			self:GetChild("Icon"):SetTexture(self.__Icon);
			self:GetChild("Text"):SetPoint("TOPLEFT",self:GetChild("Icon"),"TOPRIGHT",1,0);
			self:GetChild("Text"):SetPoint("BOTTOMRIGHT",self:GetChild("DropDown"),"BOTTOMLEFT",0,0);
		else
			self:GetChild("Icon"):SetTexture("");
			self:GetChild("Text"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth,-self.__BorderWidth);
			self:GetChild("Text"):SetPoint("BOTTOMRIGHT",self:GetChild("DropDown"),"BOTTOMLEFT",0,0);
		end
		self:GetChild("Text"):SetText(self.__Text or "");
		self:GetChild("Text"):SetFontObject(self.__FontObject);
		self:GetChild("Text"):SetTextColor(self.__FontColor.red,self.__FontColor.green,self.__FontColor.blue,self.__FontColor.alpha);
		self:GetChild("Text"):SetJustifyH("CENTER");
		self:GetChild("List").FontObject = self.__FontObject;
		self:GetChild("List").TextColor = self.__TextColor;
		self:GetChild("List").JustifyH = self.__JustifyH;
		self.__UI:SetHeight(self.__Height);
	end
	
	-- Event Handlers
	function ComboBox_OnDropDown(self)
		if not self:GetParent():GetChild("List").Visible then
			self:GetParent():GetChild("List").List = self:GetParent().__List;
			self:GetParent():GetChild("List").SelectedIndex = self:GetParent().__SelectedIndex;
			self:GetParent():GetChild("List"):GenerateCurrentList();
			if getn(self:GetParent().__List)<=self:GetParent().__MaxPageCount then
				self:GetParent():GetChild("List").Height = self:GetParent():GetChild("List").ItemHeight*getn(self:GetParent().__List);
			else
				self:GetParent():GetChild("List").Height = self:GetParent():GetChild("List").ItemHeight*self:GetParent().__MaxPageCount;
			end
			self:GetParent():GetChild("List"):Update();
			self:GetParent():GetChild("List"):Show();
		else
			self:GetParent():GetChild("List"):Hide();
		end
	end
	
	function ComboBox_OnList(self,index)
		self:GetParent():GetChild("Text"):ClearFocus();
		self:GetParent().__SelectedIndex = index;
		self:GetParent().__Text = self:GetParent().__List[index].Text;
		self:GetParent().__Icon = self:GetParent().__List[index].Icon;
		self:Hide();
		_UpdateComboBox(self:GetParent());
		self:GetParent():Fire("OnItemSelected", index);
	end
	
	function ComboBox_OnTextChanged(self)
		self:GetParent():GetChild("List").List = self:GetParent().__List;
		if self:GetParent():GetChild("Text").Focused and self:GetParent().__AutoSearch then
			self:GetParent():GetChild("List"):GenerateCurrentList(self:GetParent():GetChild("Text"):GetText());
			--self:GetParent():GetChild("List").SelectedIndex = -1;
			self:GetParent():GetChild("List"):Update();
			if getn(self:GetParent():GetChild("List").__CurrentList)<=self:GetParent().__MaxPageCount then
				self:GetParent():GetChild("List").Height = self:GetParent():GetChild("List").ItemHeight*getn(self:GetParent():GetChild("List").__CurrentList);
			else
				self:GetParent():GetChild("List").Height = self:GetParent():GetChild("List").ItemHeight*self:GetParent().__MaxPageCount;
			end
			self:GetParent():GetChild("List"):Show();
			self:GetParent():Fire("OnTextChanged");
		end
	end
	
	function ComboBox_OnEscapePressed(self)
		self:GetParent():GetChild("Text"):ClearFocus();
		self:GetParent():Fire("OnEscapePressed");
	end
	
	function ComboBox_OnEnterPressed(self)
		self:GetParent():GetChild("Text"):ClearFocus();
		self:GetParent():Fire("OnEnterPressed");
	end
	
	function ComboBox_OnTabPressed(self)
		self:GetParent():Fire("OnTabPressed");
	end
	
	function ComboBox_OnMouseEnter(self,index)
		self:GetParent():Fire("OnItemMouseEnter",index);
	end
	
	function ComboBox_OnMouseLeave(self,index)
		self:GetParent():Fire("OnItemMouseLeave",index);
	end
	
	function ComboBox_OnRightClick(self,index)
		self:GetParent():Fire("OnItemRightClick",index);
	end
	
	function ComboBox_OnEnter(self)
		self:GetParent():Fire("OnMouseEnter",self:GetParent());
	end
	
	function ComboBox_OnLeave(self)
		self:GetParent():Fire("OnMouseLeave",self:GetParent());
	end
	
	function ComboBox_OnUpdate(self)
		if self.__OldText and self.__OldText~=self:GetText() and self:GetParent().__Locked and self.Focused and not self:GetParent().__AutoSearch then
			self:SetText(self.__OldText);
			self:SetCursorPosition(self.__OldCurPos);
		end
		self.__OldText = self:GetText();
		self.__OldCurPos = self:GetCursorPosition();
	end
	
	function ComboBox_OnFocusGained(self)
		self:GetParent():Fire("OnFocusGained");
	end
	
	function ComboBox_OnFocusLost(self)
		if self:GetParent().__Locked then
			if self:GetParent().__SelectedIndex>0 then
				self:GetParent().__Text = self:GetParent().__List[self:GetParent().__SelectedIndex].Text;
				self:GetParent().__Icon = self:GetParent().__List[self:GetParent().__SelectedIndex].Icon;
			else
				self:GetParent().__Text = "";
				self:GetParent().__Icon = "";
			end
		else
			self:GetParent().__Text = self:GetText();
		end
		_UpdateComboBox(self:GetParent());
		self:GetParent():GetChild("List"):Hide();
		self:GetParent():Fire("OnFocusLost");
	end
	
	-- ScriptType
	local _ScriptType = {
		["OnTextChanged"] = true,
		["OnItemSelected"] = true,
		["OnItemMouseLeave"] = true,
		["OnItemMouseEnter"] = true,
		["OnItemRightClick"] = true,
		["OnMouseEnter"] = true,
		["OnMouseLeave"] = true,
		["OnTabPressed"] = true,
		["OnEnterPressed"] = true,
		["OnEscapePressed"] = true,
		["OnFocusGained"] = true,
		["OnFocusLost"] = true,
	}

	--	FuncProxy
	local _FuncProxy = {
		-- Get/Set AutoSearch
		["GetAutoSearch"] = function(self)
			return self.__AutoSearch;
		end,
		["SetAutoSearch"] = function(self, autosearch)
			self.__AutoSearch = autosearch;
			_UpdateBackdrop(self);
		end,
	
		-- Get/Set BackTexture
		["GetBackTexture"] = function(self)
			return self.__BackTexture;
		end,
		["SetBackTexture"] = function(self, backtexture)
			self.__BackTexture = backtexture;
			_UpdateBackdrop(self);
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			if borderstyle == self.__BorderStyle then
				return
			end
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be no less than 0.");
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0;
			end
			self.__BorderWidth = borderwidth;
			_UpdateBackdrop(self);
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor;
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderTexture
		["GetBorderTexture"] = function(self)
			return self.__BorderTexture
		end,
		["SetBorderTexture"] = function(self, bordertexture)
			self.__BorderTexture = bordertexture
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set CursorPosition
		["GetCursorPosition"] = function(self)
			return self:GetChild("Text").CursorPosition;
		end,
		["SetCursorPosition"] = function(self, cursorposition)
			self:GetChild("Text").CursorPosition = cursorposition;
		end,
		
		-- Get/Set FontObject
		["GetFontObject"] = function(self)
			return self.__FontObject;
		end,
		["SetFontObject"] = function(self, font)
			self.__FontObject = font;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set FontColor
		["GetFontColor"] = function(self)
			return self.__FontColor;
		end,
		["SetFontColor"] = function(self, fontcolor)
			self.__FontColor = fontcolor
			_UpdateComboBox(self);
		end,
		
		-- Get/Set Focused
		["GetFocused"] = function(self)
			return self:GetChild("Text").Focused;
		end,
		["SetFocused"] = function(self, focused)
			self:GetChild("Text").Focused = focused;
		end,
		
		-- HighlightText
		["HighlightText"] = function(self,...)
			return self:GetChild("Text"):HighlightText(...);
		end,
		
		-- Get/Set JustifyH
		["GetJustifyH"] = function(self)
			return self.__JustifyH;
		end,
		["SetJustifyH"] = function(self, justifyh)
			self.__JustifyH = justifyh;
			_UpdateComboBox(self);
		end,

		-- [OVERRIDE]Get/Set Height
		["GetHeight"] = function(self)
			return self.__Height;
		end,	
		["SetHeight"] = function(self, height)
			self.__Height = height;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set Icon
		["GetIcon"] = function(self)
			return self.__Icon;
		end,	
		["SetIcon"] = function(self, icon)
			self.__Icon = icon;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set List
		["GetList"] = function(self)
			return self.__List;
		end,	
		["SetList"] = function(self, list)
			self.__List = list;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set Locked
		["GetLocked"] = function(self)
			return self.__Locked;
		end,	
		["SetLocked"] = function(self, locked)
			self.__Locked = locked;
		end,
		
		-- Get/Set MaxPageCount
		["GetMaxPageCount"] = function(self)
			return self.__MaxPageCount;
		end,	
		["SetMaxPageCount"] = function(self, maxpagecount)
			self.__MaxPageCount = maxpagecount;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set SelectedIndex
		["GetSelectedIndex"] = function(self)
			return self.__SelectedIndex;
		end,	
		["SetSelectedIndex"] = function(self, selectedindex)
			self.__SelectedIndex = selectedindex;
			_UpdateComboBox(self);
		end,
		
		-- Get/Set Text
		["GetText"] = function(self)
			return self.__Text;
		end,	
		["SetText"] = function(self, text)
			self.__Text = text;
			_UpdateComboBox(self);
		end,
	}


	--	Property
	local _Property = {
		-- AutoSearch(default:true)
		["AutoSearch"]={
			["Get"] = function(self)
				return self:GetAutoSearch();
			end,
			["Set"] = function(self, value)
				self:SetAutoSearch(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- BackTexture(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["BackTexture"]={
			["Get"] = function(self)
				return self:GetBackTexture();
			end,
			["Set"] = function(self, value)
				self:SetBackTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- CursorPosition
		["CursorPosition"]={
			["Get"] = function(self)
				return self:GetCursorPosition();
			end,
			["Set"] = function(self, value)
				self:SetCursorPosition(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},

		-- FontObject(default:"GameFontHighlight")
		["FontObject"]={
			["Get"] = function(self)
				return self:GetFontObject();
			end,
			["Set"] = function(self, value)
				self:SetFontObject(value);
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		
		-- FontColor{red=1,green=1,blue=1,alpha=1}
		["FontColor"]={
			["Get"] = function(self)
				return self:GetFontColor();
			end,
			["Set"] = function(self, value)
				self:SetFontColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- Focused
		["Focused"]={
			["Get"] = function(self)
				return self:GetFocused();
			end,
			["Set"] = function(self, value)
				self:SetFocused(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- JustifyH
		["JustifyH"] = {
			["Get"] = function(self)
				return self:GetJustifyH()
			end,
			["Set"] = function(self, value)
				self:SetJustifyH(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyH"),
		},
		
		-- Height
		["Height"] = {
			["Get"] = function(self)
				return self:GetHeight();
			end,
			["Set"] = function(self, value)
				self:SetHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Icon
		["Icon"] = {
			["Get"] = function(self)
				return self:GetIcon();
			end,
			["Set"] = function(self, value)
				self:SetIcon(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- List
		["List"] = {
			["Get"] = function(self)
				return self:GetList();
			end,
			["Set"] = function(self, value)
				self:SetList(value);
			end,
			["Type"] = IGAS:GetPropertyType("Table"),
		},
		
		-- Locked
		["Locked"] = {
			["Get"] = function(self)
				return self:GetLocked();
			end,
			["Set"] = function(self, value)
				self:SetLocked(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- MaxPageCount
		["MaxPageCount"] = {
			["Get"] = function(self)
				return self:GetMaxPageCount();
			end,
			["Set"] = function(self, value)
				self:SetMaxPageCount(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- SelectedIndex
		["SelectedIndex"] = {
			["Get"] = function(self)
				return self:GetSelectedIndex();
			end,
			["Set"] = function(self, value)
				self:SetSelectedIndex(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Text
		["Text"] = {
			["Get"] = function(self)
				return self:GetText();
			end,
			["Set"] = function(self, value)
				self:SetText(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("Frame",nil,parent);
		frame.Movable = false;
		frame.Resizable = false;
		frame.MouseEnabled = true;
		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		frame.__JustifyH = "CENTER";
		frame.__BorderStyle = "SINGLE";
		frame.__BorderWidth = 1;
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8};
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackColor = {["red"]=0.3,["green"]=0.3,["blue"]=0.3,["alpha"]=0.6};
		frame.__FontObject = "GameFontNormal";
		frame.__FontColor = {["red"]=1,["green"]=1,["blue"]=1,["alpha"]=1};
		local label=IGAS:NewFrame("Label");
		label:Hide();
		label.MouseEnabled=false;
		label.Caption="haha!";
		local label2=IGAS:NewFrame("Label");
		label2:Hide();
		label2.MouseEnabled=false;
		label2.Caption="hoho!";
		frame.__List = {{Text="test"}};
		frame.__AutoSearch = true;
		frame.__MaxPageCount = 5;
		frame.__Focused = false;
		frame.__Icon = "";
		frame.__Locked = true;
		frame.__Text = "";
		frame.__SelectedIndex = -1;
		frame.Width = 150;
		frame.__Height = 28;
		
		local comboedit = CreateFrame("EditBox","Text",frame);
		comboedit:SetAutoFocus(nil);
		comboedit:SetJustifyV("MIDDLE");
		comboedit:StackScript("OnUpdate",ComboBox_OnUpdate);
		comboedit:StackScript("OnTextChanged",ComboBox_OnTextChanged);
		comboedit:StackScript("OnEscapePressed",ComboBox_OnEscapePressed);
		comboedit:StackScript("OnEnterPressed",ComboBox_OnEnterPressed);
		comboedit:StackScript("OnTabPressed",ComboBox_OnTabPressed);
		comboedit:StackScript("OnEnter",ComboBox_OnEnter);
		comboedit:StackScript("OnLeave",ComboBox_OnLeave);
		comboedit:StackScript("OnEditFocusGained",ComboBox_OnFocusGained);
		comboedit:StackScript("OnEditFocusLost",ComboBox_OnFocusLost);
		local comboicon = frame:CreateTexture("Icon");
		local combodropdown = CreateFrame("Button","DropDown",frame);
		combodropdown:SetNormalTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Up");
		combodropdown:SetPushedTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Down");
		combodropdown:SetDisabledTexture("Interface\\CHATFRAME\\UI-ChatIcon-ScrollDown-Disabled");
		combodropdown:SetHighlightTexture("Interface\\BUTTONS\\ButtonHilight-Round");
		combodropdown:StackScript("OnClick",ComboBox_OnDropDown);
		local combolist = CreateFrame("ListBox","List",frame);
		combolist:ClearAllPoints();
		combolist:SetPoint("TOPLEFT",frame,"BOTTOMLEFT",0,-1);
		combolist:SetPoint("TOPRIGHT",frame,"BOTTOMRIGHT",0,-1);
		combolist:Hide();
		combolist:StackScript("OnItemSelected",ComboBox_OnList);
		combolist:StackScript("OnItemCanceled",ComboBox_OnList);
		combolist:StackScript("OnItemMouseEnter",ComboBox_OnMouseEnter);
		combolist:StackScript("OnItemMouseLeave",ComboBox_OnMouseLeave);
		combolist:StackScript("OnItemRightClick",ComboBox_OnRightClick);
		_UpdateBackdrop(frame);
		_UpdateComboBox(frame);
		
		return frame;
	end

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
