﻿-- Author      : zys924

--------------------------
-- TextBox	        	--
--------------------------
do
	-- Check Version
	local version = 1
	if not IGAS:RegisterFile("IGAS.GUI.Widget.TextBox", version) then
		return;
	end
	
	-- API Localization
	local pairs = pairs
    local ipairs = ipairs
    local type = type
    local error = error
    local CreateFrame = IGAS.GUI.CreateFrame
	
	-- Widget Name & Inherit
	local _WidgetName = "TextBox"
	local _Base = "Frame"

	-- Define Style
	local _StyleEnum = IGAS:NewEnum("BorderStyle", {
        "NONE",
        "SINGLE",
        "TOOLTIP",
        "DIALOG",
		"CUSTOM",
    });
	local _StyleType = IGAS:NewPropertyType("BorderStyle", {
        ["Type"] = "enum",
		["EnumType"] = _StyleEnum,
    });
	
	-- Backdrop Handlers
	local _FrameBackdrop = {
		bgFile = "Interface\\Tooltips\\UI-Tooltip-Background",
		edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
		tile = true, tileSize = 16, edgeSize = 16,
		insets = { left = 3, right = 3, top = 3, bottom = 3 }
	}
	local function _UpdateBackdrop(self)
		_FrameBackdrop.bgFile = self.__BackTexture;
		if self.__BorderStyle=="NONE" then
			_FrameBackdrop.edgeFile = "";
			_FrameBackdrop.tileSize = 0;
			_FrameBackdrop.edgeSize = 0;
			self.__BorderWidth = 0;
			_FrameBackdrop.insets.left = 0;
			_FrameBackdrop.insets.right = 0;
			_FrameBackdrop.insets.top = 0;
			_FrameBackdrop.insets.bottom = 0;
		elseif self.__BorderStyle=="SINGLE" then
			_FrameBackdrop.edgeFile = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="TOOLTIP" then
			_FrameBackdrop.edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*4.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*4.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="DIALOG" then
			_FrameBackdrop.edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border";
			_FrameBackdrop.tileSize = self.__BorderWidth*2.8;
			_FrameBackdrop.edgeSize = self.__BorderWidth*2.8;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		elseif self.__BorderStyle=="CUSTOM" then
			_FrameBackdrop.edgeFile = self.__BorderTexture;
			_FrameBackdrop.tileSize = self.__BorderWidth;
			_FrameBackdrop.edgeSize = self.__BorderWidth;
			_FrameBackdrop.insets.left = self.__BorderWidth;
			_FrameBackdrop.insets.right = self.__BorderWidth;
			_FrameBackdrop.insets.top = self.__BorderWidth;
			_FrameBackdrop.insets.bottom = self.__BorderWidth;
		end
		self:SetBackdrop(_FrameBackdrop);
		self:SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:SetBackdropColor(self.__BackColor.red,self.__BackColor.green,self.__BackColor.blue,self.__BackColor.alpha);
		local tempBD = self:GetChild("SliderContainer"):GetBackdrop();
		tempBD.edgeFile = _FrameBackdrop.edgeFile;
		tempBD.tileSize = _FrameBackdrop.tileSize;
		tempBD.edgeSize = _FrameBackdrop.edgeSize;
		tempBD.insets.left = _FrameBackdrop.insets.left;
		tempBD.insets.right = _FrameBackdrop.insets.right;
		tempBD.insets.top = _FrameBackdrop.insets.top;
		tempBD.insets.bottom = _FrameBackdrop.insets.bottom;
		self:GetChild("SliderContainer"):SetBackdrop(tempBD);
		self:GetChild("SliderContainer"):SetBackdropBorderColor(self.__BorderColor.red,self.__BorderColor.green,self.__BorderColor.blue,self.__BorderColor.alpha);
		self:GetChild("SliderContainer"):SetWidth(self:GetChild("SliderContainer"):GetChild("SliderUp"):GetWidth()+2*self.__BorderWidth);
		self:GetChild("SliderContainer"):GetChild("SliderUp"):SetPoint("TOP",self:GetChild("SliderContainer"),"TOP",0,-self.__BorderWidth);
		self:GetChild("SliderContainer"):GetChild("SliderDown"):SetPoint("BOTTOM",self:GetChild("SliderContainer"),"BOTTOM",0,self.__BorderWidth);
	end
	
	local function IsNumber(str)
		local hasdot=false;
		for i=1, strlen(str) do
			if strbyte(str,i)<46 or strbyte(str,i)>57 then
				return false;
			end
			if strbyte(str,i)==46 then
				if hasdot then
					return false;
				else
					hasdot = true;
				end
			end
		end
		return true;
	end
	
	local function Round(number)
		if number-math.floor(number)<math.ceil(number)-number then
			return math.floor(number);
		else
			return math.ceil(number);
		end
	end
	
	-- Event Handlers
	local function TextBox_OnMouseDown(self,btn)
		if btn=="LeftButton" then
			self:GetChild("Container"):GetChild("EditBox"):SetFocus();
		end
	end
	
	local function TextBox_OnFocusGained(self)
		self:GetParent():GetParent().__OldText = self:GetParent():GetParent().Text;
		self:GetParent():GetParent().__HasFocus = true;
		self:GetParent():GetParent():Fire("OnFocusGained");
	end
	
	local function TextBox_OnFocusLost(self)
		self:GetParent():GetParent().__HasFocus = false;
		self:GetParent():GetParent():Fire("OnFocusLost");
		if self:GetParent():GetParent().__MultiLine then
			self:GetParent():GetParent():Fire("OnTextSet");
		end
	end
	
	local function TextBox_OnEscapePressed(self)
		if self:GetParent():GetParent().__RestoreOnEscape then
			self:GetParent():GetParent().Text = self:GetParent():GetParent().__OldText;
			self:GetParent():GetParent():Fire("OnTextRestore");
		end
		self:GetParent():GetParent().__OldText = self:GetParent():GetParent().Text;
		self:ClearFocus();
	end
	
	local function TextBox_OnEnterPressed(self)
		if self:GetParent():GetParent().__Numeric then
			if strsub(self.Text,1,1)=="." then
				self.Text = "0" .. self.Text;
			end
			if strsub(self.Text,-1,-1)=="." then
				self.Text = strsub(self.Text,1,-2);
			end
		end
		self:GetParent():GetParent():Fire("OnTextSet");
		self:ClearFocus();
	end
	
	local function TextBox_OnUpdate(self)
		if (self:GetParent():GetParent().__Locked and self.Text~=self:GetParent():GetParent().__OriginalText) and self:GetParent():GetParent().__ViaKeyboard then
			self.Text = self:GetParent():GetParent().__OriginalText or self.Text;
			self.CursorPosition = self:GetParent():GetParent().__OriginalCursorPosition or self.CursorPosition;
		end
		if (not self:GetParent():GetParent().__HorizontalScroll) and self:GetParent():GetParent().__Full and not self:IsMultiLine() then
			if strlen(self.Text) > strlen(self:GetParent():GetParent().__OriginalText) then
				self.Text = self:GetParent():GetParent().__OriginalText or self.Text;
				self.CursorPosition = self:GetParent():GetParent().__OriginalCursorPosition or self.CursorPosition;
			end
		end
		self:GetParent():GetParent().__ViaKeyboard = true;
		if (self:GetParent():GetParent().__Numeric and not IsNumber(self.Text)) then
			self.Text = self:GetParent():GetParent().__OriginalText or self.Text;
			self.CursorPosition = self:GetParent():GetParent().__OriginalCursorPosition or self.CursorPosition;
		end
		self:GetParent():GetParent().__OriginalText = self.Text;
		self:GetParent():GetParent().__OriginalCursorPosition = self.CursorPosition;
	end
	
	local function TextBox_OnMouseWheel(self,dir)
		if not self.__MultiLine then
			return;
		end
		local minvalue,maxvalue = self:GetChild("SliderContainer"):GetChild("Slider"):GetMinMaxValues();
		if dir==1 then	--up
			local value = self:GetChild("SliderContainer"):GetChild("Slider"):GetValue() - self:GetChild("SliderContainer"):GetChild("Slider"):GetValueStep();
			if value<minvalue then
				value=minvalue;
			end
			self:GetChild("SliderContainer"):GetChild("Slider"):SetValue(value);
		else	--down
			local value = self:GetChild("SliderContainer"):GetChild("Slider"):GetValue() + self:GetChild("SliderContainer"):GetChild("Slider"):GetValueStep();
			if value>maxvalue then
				value=maxvalue;
			end
			self:GetChild("SliderContainer"):GetChild("Slider"):SetValue(value);
		end
	end
	
	local function TextBox_OnTabPressed(self)
		self:Insert(strrep(" ", self:GetParent():GetParent().__TabSpace));
	end
	
	local function SliderUp_OnClick(self)
		local minvalue,maxvalue = self:GetParent():GetChild("Slider"):GetMinMaxValues();
		local value = self:GetParent():GetChild("Slider"):GetValue()-self:GetParent():GetChild("Slider"):GetValueStep();
		if value<minvalue then
			value = minvalue;
		end
		value = math.floor(value/self:GetParent():GetParent().CursorInfo.h)*self:GetParent():GetParent().CursorInfo.h;
		self:GetParent():GetChild("Slider"):SetValue(value);
	end
	
	local function SliderDown_OnClick(self)
		local minvalue,maxvalue = self:GetParent():GetChild("Slider"):GetMinMaxValues();
		local value = self:GetParent():GetChild("Slider"):GetValue()+self:GetParent():GetChild("Slider"):GetValueStep();
		if value>maxvalue then
			value = maxvalue;
		end
		self:GetParent():GetChild("Slider"):SetValue(value);
	end
	
	local function Slider_OnValueChanged(self)	--Set the ScrollBar Status
		self:GetParent():GetParent():GetChild("Container"):SetVerticalScroll(self:GetValue()*self:GetParent():GetParent().__CursorInfo.h);
		local minvalue,maxvalue=self:GetMinMaxValues();
		if self:GetValue()==minvalue then
			self:GetParent():GetChild("SliderUp"):Disable();
			self:GetParent():GetChild("SliderDown"):Enable();
			self:GetParent():GetChild("Slider"):SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
		elseif self:GetValue()==maxvalue then
			self:GetParent():GetChild("SliderUp"):Enable();
			self:GetParent():GetChild("SliderDown"):Disable();
			self:GetParent():GetChild("Slider"):SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
		elseif minvalue<self:GetValue() and maxvalue>self:GetValue() then
			self:GetParent():GetChild("SliderUp"):Enable();
			self:GetParent():GetChild("SliderDown"):Enable();
			self:GetParent():GetChild("Slider"):SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
		else
			self:GetParent():GetChild("SliderUp"):Disable();
			self:GetParent():GetChild("SliderDown"):Disable();
			self:GetParent():GetChild("Slider"):SetThumbTexture("");
		end
	end
		
	local function TextBox_OnTextChanged(self)
		if not self:GetParent():GetParent().__Locked then
			self:GetParent():GetParent():Fire("OnTextChanged");
		end
		if self:GetParent():GetParent().MultiLine then	--Set the Scroll Bar Range
			if self:GetHeight()>=self:GetParent():GetHeight()+3 then
				local range = self:GetParent():GetVerticalScrollRange();
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetMinMaxValues(0, Round(range/self:GetParent():GetParent().__CursorInfo.h));
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValueStep(1);
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
				Slider_OnValueChanged(self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"));
			else
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetMinMaxValues(0,0);
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValue(0);
				self:GetParent():SetVerticalScroll(0);
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("SliderUp"):Disable();
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("SliderDown"):Disable();
				self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetThumbTexture("");
			end
		else
			local curpos = self:GetCursorPosition();
			self:SetCursorPosition(strlen(self:GetText()))
			if self:GetParent():GetParent().__CursorInfo.x+self:GetParent():GetParent().__CursorInfo.h > self:GetParent():GetWidth() then
				self:GetParent():GetParent().__Full = true;
			else
				self:GetParent():GetParent().__Full = false;
			end
			self:SetCursorPosition(curpos);
		end
	end
	
	local function TextBox_OnCursorChanged(self,x,y,w,h)
		self:GetParent():GetParent().__CursorInfo.x = x;
		self:GetParent():GetParent().__CursorInfo.y = y;
		self:GetParent():GetParent().__CursorInfo.w = w;
		self:GetParent():GetParent().__CursorInfo.h = h;
		if self:IsMultiLine() then	--AutoScroll when cursor changed
			local range = self:GetParent():GetVerticalScrollRange();	-- pixels
			local maxvalue = Round(range/h);	-- linenum
			self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetMinMaxValues(0, maxvalue);
			local scroll = Round(self:GetParent():GetVerticalScroll()/h);	-- linenum
			local cursorline = Round(-y/h)+1;
			local containerline = Round(self:GetParent():GetHeight()/h);
			if cursorline>scroll+containerline then
				local times = cursorline-scroll-containerline;
				if scroll+times<maxvalue then
					self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValue(scroll+times);
				else
					self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValue(maxvalue);
				end
			elseif cursorline<=scroll then
				local times=scroll-cursorline+1;
				if scroll-times>0 then
					self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValue(scroll-times);
				else
					self:GetParent():GetParent():GetChild("SliderContainer"):GetChild("Slider"):SetValue(0);
				end
			end
		end
		self:GetParent():GetParent():Fire("OnCursorChanged");
	end
	
	local function _UpdateTextBox(self)
		self:GetChild("Container"):ClearAllPoints();
		self:GetChild("Container"):GetChild("EditBox"):ClearAllPoints();
		self:GetChild("Container"):SetPoint("TOPLEFT",self,"TOPLEFT",self.__BorderWidth+self.__Inset,-self.__BorderWidth-self.__Inset);
		self:GetChild("Container"):SetPoint("BOTTOMRIGHT",self,"BOTTOMRIGHT",-self.__BorderWidth-self.__Inset,self.__BorderWidth+self.__Inset);
		if not self.__MultiLine then
			--SingleTextBox
			self:GetChild("Container"):GetChild("EditBox"):SetMultiLine(nil);
			self:GetChild("Container"):GetChild("EditBox"):SetPoint("TOPLEFT",self:GetChild("Container"),"TOPLEFT",0,0);
			self:GetChild("Container"):GetChild("EditBox"):SetPoint("BOTTOMRIGHT",self:GetChild("Container"),"BOTTOMRIGHT",0,0);
			self:GetChild("Container"):GetChild("EditBox"):SetJustifyH(self.__JustifyH);
			self:GetChild("Container"):GetChild("EditBox"):SetJustifyV(self.__JustifyV);
			self:GetChild("Container"):GetChild("EditBox"):SetScript("OnEnterPressed", TextBox_OnEnterPressed);
			self:GetChild("SliderContainer"):Hide();
			self.__UI:SetHeight(self.__Height);
		else
			self:GetChild("Container"):GetChild("EditBox"):SetMultiLine(1);
			self:GetChild("Container"):GetChild("EditBox"):SetJustifyV("TOP");
			self:GetChild("Container"):GetChild("EditBox"):SetScript("OnEnterPressed", nil);
			if self.__HasScrollBar then
				self:GetChild("SliderContainer"):Show();
				self:GetChild("Container"):SetPoint("TOPRIGHT",self:GetChild("SliderContainer"),"TOPLEFT",-self.__Inset,0);
			else
				self:GetChild("SliderContainer"):Hide();
				self:GetChild("Container"):SetPoint("TOPRIGHT",self,"TOPRIGHT",-self.__BorderWidth-self.__Inset,-self.__BorderWidth-self.__Inset);
			end
			self:GetChild("Container"):GetChild("EditBox"):SetPoint("TOPLEFT",self:GetChild("Container"),"TOPLEFT",0,0);
			self:GetChild("Container"):GetChild("EditBox"):SetWidth(self:GetChild("Container"):GetWidth());
			local netheight = self:GetChild("Container"):GetHeight();
			local height1 = netheight-self.__CursorInfo.h*math.floor(netheight/self.__CursorInfo.h);
			local height2 = self.__CursorInfo.h*(math.floor(netheight/self.__CursorInfo.h)+1)-netheight;
			if height1<height2 then
				self.__UI:SetHeight(self.__CursorInfo.h*math.floor(netheight/self.__CursorInfo.h)+2*self.__BorderWidth+2*self.__Inset);
			else
				self.__UI:SetHeight(self.__CursorInfo.h*(math.floor(netheight/self.__CursorInfo.h)+1)+2*self.__BorderWidth+2*self.__Inset);
			end
		end
	end

	-- ScriptType
	local _ScriptType = {
		["OnTextRestore"] = true,
		["OnTextSet"] = true,
		["OnFocusGained"] = true,
		["OnFocusLost"] = true,
		["OnTextChanged"] = true,
		["OnCursorChanged"] = true,
	}

	--	FuncProxy
	local _FuncProxy = {
	
		-- Get/Set BackTexture
		["GetBackTexture"] = function(self)
			return self.__BackTexture;
		end,
		["SetBackTexture"] = function(self, backtexture)
			self.__BackTexture = backtexture;
			_UpdateBackdrop(self);
		end,
		
		-- Get/Set BackColor
		["GetBackColor"] = function(self)
			return self.__BackColor
		end,
		["SetBackColor"] = function(self, backcolor)
			self.__BackColor = backcolor
			_UpdateBackdrop(self)
		end,
	
		-- Get/Set BorderStyle
		["GetBorderStyle"] = function(self)
			return self.__BorderStyle
		end,
		["SetBorderStyle"] = function(self, borderstyle)
		    if not borderstyle or type(borderstyle) ~= "string" or (not _StyleEnum[borderstyle]) then
				error("Invalid \"BorderStyle\":it must be \"NONE\",\"SINGLE\",\"TOOLTIP\",\"DIALOG\",\"CUSTOM\".", 2);
                return
            end
			if borderstyle == self.__BorderStyle then
				return
			end
			self.__BorderStyle = borderstyle
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set BorderWidth
		["GetBorderWidth"] = function(self)
			return self.__BorderWidth
		end,
		["SetBorderWidth"] = function(self, borderwidth)
			if not borderwidth or type(borderwidth) ~= "number" or borderwidth < 0 then
				error("Invalid \"BorderWidth\":it must be no less than 0.");
				return;
			end
			if _BorderStyle == "NONE" then
				borderwidth = 0;
			end
			self.__BorderWidth = borderwidth;
			_UpdateBackdrop(self);
			_UpdateTextBox(self);
		end,

		-- Get/Set BorderColor
		["GetBorderColor"] = function(self)
			return self.__BorderColor;
		end,
		["SetBorderColor"] = function(self, bordercolor)
			self.__BorderColor = bordercolor;
			_UpdateBackdrop(self)
		end,
		
		-- Get/Set CursorPosition
		["GetCursorPosition"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox"):GetCursorPosition();
		end,
		["SetCursorPosition"] = function(self, cursorposition)
			self:GetChild("Container"):GetChild("EditBox"):SetCursorPosition(cursorposition);
		end,
		
		-- Get CursorInfo
		["GetCursorInfo"] = function(self)
			return self.__CursorInfo;
		end,
		
		-- Get/Set Focus
		["GetHasFocus"] = function(self)
			return self.__HasFocus;
		end,
		["SetHasFocus"] = function(self, hasfocus)
			if hasfocus then
				self:GetChild("Container"):GetChild("EditBox"):SetFocus();
			else
				self:GetChild("Container"):GetChild("EditBox"):ClearFocus();
			end
		end,
		
		-- Get/Set FontObject
		["GetFontObject"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox").FontObject
		end,
		["SetFontObject"] = function(self, font)
			self:GetChild("Container"):GetChild("EditBox").FontObject = font
		end,
		
		-- Get/Set FontColor
		["GetFontColor"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox").TextColor
		end,
		["SetFontColor"] = function(self, fontcolor)
			self:GetChild("Container"):GetChild("EditBox").TextColor = fontcolor
		end,
		
		-- Get/Set HasScrollBar
		["GetHasScrollBar"] = function(self)
			return self.__HasScrollBar;
		end,
		["SetHasScrollBar"] = function(self, hasscrollbar)
			self.__HasScrollBar = hasscrollbar;
			_UpdateTextBox(self);
		end,
		
		-- Get/Set Height
		["GetHeight"] = function(self)
			return self.__Height;
		end,
		["SetHeight"] = function(self, height)
			self.__Height = height;
			_UpdateTextBox(self);
		end,
		
		-- Get/Set HorizontalScroll
		["GetHorizontalScroll"] = function(self)
			return self.__HorizontalScroll;
		end,
		["SetHorizontalScroll"] = function(self, horizontalscroll)
			self.__HorizontalScroll = horizontalscroll;
		end,
		
		-- Get/Set Inset
		["GetInset"] = function(self)
			return self.__Inset;
		end,
		["SetInset"] = function(self, inset)
			self.__Inset = inset;
			_UpdateTextBox(self);
		end,
		
		-- Get/Set JustifyH
		["GetJustifyH"] = function(self)
			return self.__JustifyH;
		end,
		["SetJustifyH"] = function(self, justifyh)
			self.__JustifyH = justifyh;
			_UpdateLabel(self);
		end,
		
		-- Get/Set JustifyV
		["GetJustifyV"] = function(self)
			return self.__JustifyV;
		end,
		["SetJustifyV"] = function(self, justifyv)
			self.__JustifyV = justifyv;
			_UpdateLabel(self);
		end,
		
		-- Get/Set Locked
		["GetLocked"] = function(self)
			return self.__Locked;
		end,
		["SetLocked"] = function(self, locked)
			self.__Locked = locked;
		end,
		
		-- Get/Set MaxLetters
		["GetMaxLetters"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox"):GetMaxLetters();
		end,
		["SetMaxLetters"] = function(self, maxletters)
			self:GetChild("Container"):GetChild("EditBox"):SetMaxLetters(maxletters);
		end,
		
		-- Get/Set MaxBytes
		["GetMaxBytes"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox"):GetMaxBytes();
		end,
		["SetMaxBytes"] = function(self, maxbytes)
			self:GetChild("Container"):GetChild("EditBox"):SetMaxBytes(maxbytes);
		end,
		
		-- Get/Set MultiLine
		["GetMultiLine"] = function(self)
			return self.__MultiLine;
		end,
		["SetMultiLine"] = function(self, multiline)
			self.__MultiLine = multiline;
			_UpdateTextBox(self);
		end,
		
		-- Get/Set Numeric
		["GetNumeric"] = function(self)
			return self.__Numeric;
			--if not self:GetChild("Container"):GetChild("EditBox"):IsNumeric() then
			--	return false;
			--else
			--	return true;
			--end
		end,
		["SetNumeric"] = function(self, numeric)
			self.__Numeric = numeric;
			--if numeric then
			--	self:GetChild("Container"):GetChild("EditBox"):SetNumeric(1);
			--else
			--	self:GetChild("Container"):GetChild("EditBox"):SetNumeric(nil);
			--end
		end,
		
		-- Get/Set RestoreOnEscape
		["GetRestoreOnEscape"] = function(self)
			return self.__RestoreOnEscape;
		end,
		["SetRestoreOnEscape"] = function(self, restore)
			self.__RestoreOnEscape = restore;
		end,
		
		-- Get/Set TabSpace
		["GetTabSpace"] = function(self)
			return self.__TabSpace;
		end,
		["SetTabSpace"] = function(self, tabspace)
			self.__TabSpace = tabspace
		end,
		
		-- Get/Set Text
		["GetText"] = function(self)
			return self:GetChild("Container"):GetChild("EditBox"):GetText();
		end,
		["SetText"] = function(self, text)
			self.__ViaKeyboard = false;
			self:GetChild("Container"):GetChild("EditBox"):SetText(text);
		end,
		
		-- Get/Set WheelScroll
		["GetWheelScroll"] = function(self)
			return self.__WheelScroll;
		end,
		["SetWheelScroll"] = function(self, wheelscroll)
			self.__WheelScroll = wheelscroll;
			_UpdateTextBox(self);
		end,
		
		-- HighlightText
		["HighlightText"] = function(self, startpos, endpos)
			self:GetChild("Container"):GetChild("EditBox"):HighlightText(startpos,endpos);
		end,
	}


	--	Property
	local _Property = {
		
		-- BackColor(default:{red=0.2,green=0.2,blue=0.2,alpha=0.5})
		["BackColor"]={
			["Get"] = function(self)
				return self:GetBackColor();
			end,
			["Set"] = function(self, value)
				self:SetBackColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BackTexture(default:"Interface\\ChatFrame\\CHATFRAMEBACKGROUND")
		["BackTexture"]={
			["Get"] = function(self)
				return self:GetBackTexture();
			end,
			["Set"] = function(self, value)
				self:SetBackTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- BorderStyle(default:1)
		["BorderStyle"]={
			["Get"] = function(self)
				return self:GetBorderStyle();
			end,
			["Set"] = function(self, value)
				self:SetBorderStyle(value);
			end,
			["Type"] = _StyleType,
		},
		
		-- BorderWidth(default:2)
		["BorderWidth"]={
			["Get"] = function(self)
				return self:GetBorderWidth();
			end,
			["Set"] = function(self, value)
				self:SetBorderWidth(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- BorderColor(default:{red=0.8,green=0.8,blue=0.8,alpha=0.8})
		["BorderColor"]={
			["Get"] = function(self)
				return self:GetBorderColor();
			end,
			["Set"] = function(self, value)
				self:SetBorderColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- BorderTexture
		["BorderTexture"]={
			["Get"] = function(self)
				return self:GetBorderTexture();
			end,
			["Set"] = function(self, value)
				self:SetBorderTexture(value);
			end,
			["Type"] = IGAS:GetPropertyType("Texture"),
		},
		
		-- CursorPosition(default:0)
		["CursorPosition"]={
			["Get"] = function(self)
				return self:GetCursorPosition();
			end,
			["Set"] = function(self, value)
				self:SetCursorPosition(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- CursorInfo
		["CursorInfo"]={
			["Get"] = function(self)
				return self:GetCursorInfo();
			end,
			["Type"] = IGAS:GetPropertyType("Table"),
		},
		
		-- FontObject(default:"GameFontHighlight")
		["FontObject"]={
			["Get"] = function(self)
				return self:GetFontObject();
			end,
			["Set"] = function(self, value)
				self:SetFontObject(value);
			end,
			["Type"] = IGAS:GetPropertyType("Object"),
		},
		
		-- FontColor{red=1,green=1,blue=1,alpha=1}
		["FontColor"]={
			["Get"] = function(self)
				return self:GetChild("EditBox"):GetFontColor();
			end,
			["Set"] = function(self, value)
				self:GetChild("EditBox"):SetFontColor(value);
			end,
			["Type"] = IGAS:GetPropertyType("Color"),
		},
		
		-- HasFocus(default:false)
		["HasFocus"]={
			["Get"] = function(self)
				return self:GetHasFocus();
			end,
			["Set"] = function(self, value)
				self:SetHasFocus(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- HasScrollBar(default:true)
		["HasScrollBar"]={
			["Get"] = function(self)
				return self:GetHasScrollBar();
			end,
			["Set"] = function(self, value)
				self:SetHasScrollBar(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- Height(default:100)
		["Height"]={
			["Get"] = function(self)
				return self:GetHeight();
			end,
			["Set"] = function(self, value)
				self:SetHeight(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- HorizontalScroll(default:false)
		["HorizontalScroll"]={
			["Get"] = function(self)
				return self:GetHorizontalScroll();
			end,
			["Set"] = function(self, value)
				self:SetHorizontalScroll(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- Inset(default:1)
		["Inset"]={
			["Get"] = function(self)
				return self:GetInset();
			end,
			["Set"] = function(self, value)
				self:SetInset(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Numeric(default:false)
		["Numeric"]={
			["Get"] = function(self)
				return self:GetNumeric();
			end,
			["Set"] = function(self, value)
				self:SetNumeric(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- JustifyH
		["JustifyH"] = {
			["Get"] = function(self)
				return self:GetJustifyH()
			end,
			["Set"] = function(self, value)
				self:SetJustifyH(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyH"),
		},
		
		-- JustifyV
		["JustifyV"] = {
			["Get"] = function(self)
				return self:GetJustifyV()
			end,
			["Set"] = function(self, value)
				self:SetJustifyV(value)
			end,
			["Type"] = IGAS:GetPropertyType("JustifyV"),
		},
		
		-- Locked(default:false)
		["Locked"]={
			["Get"] = function(self)
				return self:GetLocked();
			end,
			["Set"] = function(self, value)
				self:SetLocked(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- MaxLetters(default:0)
		["MaxLetters"]={
			["Get"] = function(self)
				return self:GetMaxLetters();
			end,
			["Set"] = function(self, value)
				self:SetMaxLetters(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- MultiLine(default:false)
		["MaxBytes"]={
			["Get"] = function(self)
				return self:GetMaxBytes();
			end,
			["Set"] = function(self, value)
				self:SetMaxBytes(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- MultiLine(default:false)
		["MultiLine"]={
			["Get"] = function(self)
				return self:GetMultiLine();
			end,
			["Set"] = function(self, value)
				self:SetMultiLine(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- RestoreOnEscape(default:true)
		["RestoreOnEscape"]={
			["Get"] = function(self)
				return self:GetRestoreOnEscape();
			end,
			["Set"] = function(self, value)
				self:SetRestoreOnEscape(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
		
		-- TabSpace(default:4)
		["TabSpace"]={
			["Get"] = function(self)
				return self:GetTabSpace();
			end,
			["Set"] = function(self, value)
				self:SetTabSpace(value);
			end,
			["Type"] = IGAS:GetPropertyType("Number"),
		},
		
		-- Text(default:"")
		["Text"]={
			["Get"] = function(self)
				return self:GetText();
			end,
			["Set"] = function(self, value)
				self:SetText(value);
			end,
			["Type"] = IGAS:GetPropertyType("String"),
		},
		
		-- WheelScroll(default:true)
		["WheelScroll"]={
			["Get"] = function(self)
				return self:GetWheelScroll();
			end,
			["Set"] = function(self, value)
				self:SetWheelScroll(value);
			end,
			["Type"] = IGAS:GetPropertyType("Boolean"),
		},
	}

	local function _New(parent)		
		-- New Frame
		local frame = CreateFrame("ScrollFrame",nil,parent);
		frame.Movable = false
		frame.Resizable = false
		frame.MouseEnabled = true
		frame.MouseWheelEnabled = true
		frame:SetPoint("CENTER",parent,"CENTER",0,0);
		frame.__BackTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__BackColor = {["red"]=0.3,["green"]=0.3,["blue"]=0.3,["alpha"]=0.6};
		frame.__BorderStyle = "SINGLE";
		frame.__BorderWidth = 1;
		frame.__BorderColor = {["red"]=0.8,["green"]=0.8,["blue"]=0.8,["alpha"]=0.8};
		frame.__BorderTexture = "Interface\\ChatFrame\\CHATFRAMEBACKGROUND";
		frame.__HasScrollBar = false;
		frame.__MultiLine = false;
		frame.__Locked = false;
		frame.__RestoreOnEscape = false;
		frame.__JustifyH = "LEFT";
		frame.__JustifyV = "MIDDLE";
		frame.__Numeric = false;
		frame.__CursorInfo = {x=0,y=0,w=0,h=0};
		frame.__Inset = 3;
		frame.__HorizontalScroll = true;
		frame.Width = 200;
		frame.__Height = 100;
		frame.__TabSpace = 4;
		frame.__WheelScroll = true;
		frame:StackScript("OnMouseDown", TextBox_OnMouseDown);
		frame:StackScript("OnMouseWheel", TextBox_OnMouseWheel);
		local container = CreateFrame("ScrollFrame","Container",frame);
		
		local editbox = CreateFrame("EditBox","EditBox",container);
		container:SetScrollChild(editbox);
		editbox:SetFontObject("GameFontNormal");
		editbox:SetTextColor(1,1,1,1);
		editbox:SetAutoFocus(nil);
		editbox:StackScript("OnEscapePressed", TextBox_OnEscapePressed);
		editbox:StackScript("OnEditFocusGained", TextBox_OnFocusGained);
		editbox:StackScript("OnEditFocusLost", TextBox_OnFocusLost);
		editbox:StackScript("OnUpdate", TextBox_OnUpdate);
		editbox:StackScript("OnTextChanged", TextBox_OnTextChanged);
		editbox:StackScript("OnCursorChanged", TextBox_OnCursorChanged);
		editbox:StackScript("OnTabPressed", TextBox_OnTabPressed);
		
		local slidercontainer = CreateFrame("Frame","SliderContainer",frame);
		slidercontainer:SetPoint("TOPRIGHT",frame,"TOPRIGHT",0,0);
		slidercontainer:SetPoint("BOTTOMRIGHT",frame,"BOTTOMRIGHT",0,0);
		slidercontainer:SetBackdrop({
			bgFile = "",
			edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border",
			tile = true, tileSize = 12, edgeSize = 12,
			insets = { left = 2, right = 2, top = 2, bottom = 2 }
		});
		local sliderup = CreateFrame("Button","SliderUp",slidercontainer,"UIPanelScrollUpButtonTemplate");
		sliderup:SetPoint("TOP",slidercontainer,"TOP",0,-frame.__BorderWidth-frame.__Inset); 
		sliderup:Disable();
		sliderup:StackScript("OnClick", SliderUp_OnClick);
		local sliderdown = CreateFrame("Button","SliderDown",slidercontainer,"UIPanelScrollDownButtonTemplate");
		sliderdown:SetPoint("BOTTOM",slidercontainer,"BOTTOM",0,frame.__BorderWidth+frame.__Inset);
		sliderdown:Disable();
		sliderdown:StackScript("OnClick", SliderDown_OnClick);
		slidercontainer:SetWidth(sliderup:GetWidth()+frame.__BorderWidth*2+2*frame.__Inset);
		local slider = CreateFrame("Slider","Slider",slidercontainer);
		slider:SetPoint("TOP",sliderup,"BOTTOM",0,8);
		slider:SetPoint("BOTTOM",sliderdown,"TOP",0,-8);
		slider:SetWidth(20);
		slider:SetThumbTexture("Interface\\Buttons\\UI-ScrollBar-Knob");
		slider:StackScript("OnValueChanged", Slider_OnValueChanged);
		
		_UpdateBackdrop(frame);
		_UpdateTextBox(frame);
		
		return frame;
	end	

	-- Register
	local _WidgetInfo = {
		["WidgetName"] = _WidgetName,
		["Base"] = _Base,
		["FuncProxy"] = _FuncProxy,
		["Property"] = _Property,
		["ScriptType"] = _ScriptType,
		["New"] = _New,
	}

	IGAS:NewWidget(_WidgetInfo, true)
end
