-- Author      : Kurapica
-- IGAS:MsgBox(_Message, _Map, _Callback)
-- Show a popup dialog.
-- Parameters:
--		_Message	The message to be displayed in the popupdialog.
--		_Map		Combined by "n", "c", "i"
--					"n" means show No Button
--					"c" means show Cancel Button
--					"i" means show the input box
--					for example, "inc" means show all buttons and the input box.Not case sensitive
--		_Callback	The function to receive the result.

do
	local L = IGAS:NewLocal("IGAS_Special")
	
	local tinsert = table.insert
	local tremove = table.remove
	local getn = table.getn
	local strfind = string.find
	local strlower = string.lower
	local _Queue = {}
	local _MaxQueue = 10
	
	local frmPop = IGAS:NewFrame("PopupDialog", "IGAS_GUI_MSGBOX", WorldFrame)
	frmPop.Style = "LIGHT"
	frmPop.OkayButtonText = L["Okay"]
	frmPop.NoButtonText = L["No"]
	frmPop.CancelButtonText = L["Cancel"]
	
	
	local function ShowDialog(_Message, _Map, _Callback)
		frmPop.Message = _Message
		frmPop.ShowNoButton = (strfind(_Map, "n") and true) or false
		frmPop.ShowCancelButton = (strfind(_Map, "c") and true) or false
		frmPop.ShowInputBox = (strfind(_Map, "i") and true) or false
		frmPop.Text = ""
		frmPop.__CallBack = _Callback
		
		frmPop.Visible = true
	end
	
	function frmPop:OnHide()
		if _Queue[1] then
			local info = _Queue[1]
			tremove(_Queue, 1)
			
			return ShowDialog(info.Message, info.Map, info.Callback)
		end
	end
	
	function frmPop:OnOkay()
		if not self.__CallBack and type(self.__CallBack) ~= "function" then
			return
		end
		if self.ShowInputBox then
			return self.__CallBack(self.Text)
		else
			return self.__CallBack(true)
		end
	end
	
	function frmPop:OnNo()
		if not self.__CallBack and type(self.__CallBack) ~= "function" then
			return
		end
		if self.ShowInputBox then
			return self.__CallBack(nil)
		else
			return self.__CallBack(false)
		end
	end
	
	function frmPop:OnCancel()
		if not self.__CallBack and type(self.__CallBack) ~= "function" then
			return
		end
		if self.ShowInputBox then
			return self.__CallBack(nil)
		else
			return self.__CallBack(nil)
		end
	end
	
	IGAS:RegisterFunction("MsgBox", function(self, _Message, _Map, _Callback)
		if not _Message or type(_Message) ~= "string" then
			return
		end
		if not _Map or type(_Map) ~= "string" then
			_Map = ""
		end
		if _Callback and type(_Callback) ~= "function" then
			error("The callback must be a function", 2)
		end
		
		_Map = strlower(_Map)
		
		if frmPop.Visible then
			if getn(_Queue) < _MaxQueue then
				return tinsert(_Queue, {["Message"] = _Message, ["Map"] = _Map, ["Callback"] = _Callback})
			else
				error("Too many request for MsgBox.", 2)
			end
		end
		
		return ShowDialog(_Message, _Map, _Callback)
	end)
end